//=============================================================================
// TO_GUITabDebriefing.
//=============================================================================
// This GUI tab is displayed when the mission is completed. It displays
// various statitics to the user and allows him to replay the mission or
// access the next one.
//=============================================================================
// Programmed by Mathieu 'EMH_Mark3' Mallet for Tactical Ops
// 19/08/2002
//=============================================================================

class TO_GUITabDebriefing expands TO_GUIBaseTab;

var TO_SinglePlayerGame			Game;

var TO_GUIBaseButton			ButtonReplay, ButtonNext, ButtonExit;

var int							EnemyCasualties;
var int							MinimumEnemyCasualties;
var int							FriendlyCasualties;
var int							Efficiency;
var bool						bPerfect;
var bool						bFirstPaint;
var bool						bIsCutScene;
var bool						bUnlockedCheat;

var color						ColorYourTime;
var color						ColorYourBestTime;
var color						ColorKamehanBestTime;

var color						ColorBest;
var color						ColorNormal;
var color						ColorWorse;

var string						TextAltHint;
var string						UnlockedCheatTitle;	

var int							BestReceivedMedal;
var int							TimeBronze, TimeSilver, TimeGold;

var localized string			TextSPHint;
var localized string			TextSPAltHint;
var localized string			TextTitle;

var localized string			TextDefaultCongradMSG;

var localized string			TextReplayButton;
var localized string			TextNextButton;
var localized string			TextExitButton;

var localized string			TextReplayButtonShort;
var localized string			TextNextButtonShort;
var localized string			TextExitButtonShort;

var localized string			TextHintReplay;
var localized string			TextHintNext;
var localized string			TextHintExit;
var localized string			TextHintLoadingSame;
var localized string			TextHintLoadingNext;

var localized string			TextUsedCheats;
var localized string			TextUsedCheatsHint;

var localized string			TextPlayedInEasyMode;

var localized string			TextEnemyCasualties;
var localized string			TextFriendlyCasualties;
var localized string			TextFriendlyFireIncidents;
var localized string			TextEfficiency;
var localized string			TextHealth;

var localized string			TextPerfectGame;
var localized string			TextNewCheat;

var localized string			TextYourTime;
var localized string			TextYourBestTime;
var localized string			TextUserBestTime;
var localized string			TextKamehanTime;
var localized string			TextMedalBronze;
var localized string			TextMedalSilver;
var localized string			TextMedalGold;
var localized string			TextTime;
var localized string			TextConditions;

var localized string			TextCumulativeBesttime;
var localized string			TextCumulativeKamehanTime;

var localized string			TextNoPercentage;

var localized string			TextHintStory;
var localized string			TextHintGoButton;
var localized string			TextGoButton;
var localized string			TextGoButtonShort;

var localized string			TextEndButtonShort;
var localized string			TextEndButton;
var localized string			TextHintEnd;
var localized string			TextEndNext;

var localized string			TextConclusionButtonShort;
var localized string			TextConclusionButton;
var localized string			TextConclusionNext;

var localized string			TextGoldMedal;
var localized string			TextSilverMedal;
var localized string			TextBronzeMedal;
var localized string			TextMedal;

var bool						bPlayingLastMap;
var bool						bFirstPreRound;
var bool						bLastModMap;
var bool						bTextOnly;
var bool						bDisableNext;
var bool						bFirstDebriefingCall;

var string						TextScreenTitle;
var string						TextScreenParagraph1;
var string						TextScreenParagraph2;
var string						TextScreenParagraph3;
var string						TextScreenParagraph4;
var string						TextScreenParagraph5;

var int							EfficiencyThreshold;
var int							HealthThreshold;

/*
================================================================================================================================================
   methods (engine)
================================================================================================================================================
*/

////////////////////
// Created
////////////////////

simulated function Created ()
{
	Super.Created();

	Game = TO_SinglePlayerGame(GetPlayerOwner().Level.Game);
	if (Game == None && TO_SP_CutScene(GetPlayerOwner().Level.Game) != none)
		bIsCutScene = true;

	if (bIsCutScene) {
		Hint = TextHintEnd;
	}
	else {
		Hint = TextHintStory;
		TextAltHint = TextSPAltHint;
		
		AltHint = TextAltHint;
	}

	bPlayingLastMap = IsLastMap();

	ButtonReplay = TO_GUIBaseButton(CreateWindow(class'TO_GUIBaseButton', 0, 0, WinWidth, WinHeight));
	ButtonReplay.Text = TextReplayButton;
	ButtonReplay.OwnerTab = self;

	ButtonNext = TO_GUIBaseButton(CreateWindow(class'TO_GUIBaseButton', 0, 0, WinWidth, WinHeight));
	ButtonNext.Text = TextNextButton;
	ButtonNext.OwnerTab = self;

	ButtonExit = TO_GUIBaseButton(CreateWindow(class'TO_GUIBaseButton', 0, 0, WinWidth, WinHeight));
	ButtonExit.Text = TextExitButton;
	ButtonExit.OwnerTab = self;

	ColorBest = OwnerInterface.Design.ColorGreen;
	ColorNormal = OwnerInterface.Design.ColorSuperwhite;
	ColorWorse = OwnerInterface.Design.ColorRed;

	CheckLastModMap();

	//if (bFirstPreRound)
		ButtonReplay.HideWindow();
	//if (bFirstPreRound)
		ButtonExit.HideWindow();

}

simulated function FirstPreRoundOver() {
	Hint = TextSPHint;
	bFirstPreRound = false;
	bPlayingLastMap = IsLastMap();
	bInitialized = false;
	Game.RestartRound();
}


////////////////////
// Close
////////////////////

simulated function Close (optional bool bByParent)
{
	Super.Close(bByParent);
}


////////////////////
// BeforePaint
////////////////////

simulated function BeforePaint (Canvas Canvas, float x, float y)
{
	//Super.BeforePaint(Canvas, x, y);

	if (!bInitialized)
	{
		Super.BeforePaint(Canvas, x, y);
		TODebriefing_Init(Canvas);
	}
	else
	{
		Super.BeforePaint(Canvas, x, y);
	}

	if (!bDraw)
	{
		return;
	}
	
	// setup buttons
	if (bDraw)
	{
		if (!bFirstPreRound)
			ButtonReplay.ShowWindow();
		//if (!bPlayingLastMap)
			ButtonNext.ShowWindow();
		if (!bFirstPreRound)
			ButtonExit.ShowWindow();
	}

	Title = TODebreifing_Tool_GetDescription();
}


////////////////////
// Paint
////////////////////

simulated function Paint (Canvas Canvas, float x, float y)
{
	local byte				listoffset;
	local byte				vislines;
	local float				ypos;
	local int				i;


	if (!bDraw)
	{
		return;
	}

	// background
	if (OwnerHud.bDrawBackground)
	{
		Super.Paint(Canvas, x, y);
	}

	Canvas.Style = OwnerInterface.ERenderStyle.STY_Normal;

	if (bFirstPreRound)
		TOBriefing_DrawData(1, Canvas);
	else if (bLastModMap && bTextOnly)
		TOBriefing_DrawData(2, Canvas);
	else
		TODebriefing_DrawData(Canvas);

}

////////////////////
//  TODebriefing_Init
////////////////////

simulated final function TODebriefing_Init (Canvas Canvas)
{
	local float				l, w, t, w2, w4, h;
	local int				i;


	w = Width*0.25 - Padding[Resolution];
	l = int(Width*0.125);
	w2 = int(w*0.5);
	w4 = int(w*0.4);
	h = OwnerHud.Design.LineHeight + 3;

	//OwnerInterface.Design.SetHeadlineFont(Canvas);
	
	ButtonReplay.WinLeft = Left + int(w/3);
	ButtonReplay.WinTop = Top + Height - h - int(w/3);
	ButtonReplay.SetWidth(Canvas, w - Padding[Resolution]);

	//if (!bPlayingLastMap) {
		ButtonNext.WinLeft = Left + int(Width/2 - w/2);
		ButtonNext.WinTop = Top + Height - h - int(w/3);
		ButtonNext.SetWidth(Canvas, w - Padding[Resolution]);
	//}
	
	ButtonExit.WinLeft = Left + Width - int(w * 4 / 3);
	ButtonExit.WinTop = Top + Height - h - int(w/3);
	ButtonExit.SetWidth(Canvas, w - Padding[Resolution]);

	/*if (bPlayingLastMap) {
		ButtonReplay.WinLeft = Left + int(w/2);
		ButtonExit.WinLeft = Left + Width - int(w * 3 / 2);
	}*/

	if (Resolution <= 1 ) {	// Use short text on low resolutions
		ButtonReplay.Text =	TextReplayButtonShort;
		//if (!bPlayingLastMap) {
			if (bPlayingLastMap)
				ButtonNext.Text = TextConclusionButtonShort;
			else if (bIsCutScene)
				ButtonNext.Text = TextEndButtonShort;
			else if (bFirstPreRound)
				ButtonNext.Text = TextGoButtonShort;
			else
				ButtonNext.Text = TextNextButtonShort;
		//}
		ButtonExit.Text =	TextExitButtonShort;
	}
	else {					// Use longer text on high resolutions
		ButtonReplay.Text =	TextReplayButton;
		//if (!bPlayingLastMap) {
			if (bPlayingLastMap)
				ButtonNext.Text = TextConclusionButton;
			else if (bIsCutScene)
				ButtonNext.Text = TextEndButton;
			else if (bFirstPreRound)
				ButtonNext.Text = TextGoButton;
			else
				ButtonNext.Text = TextNextButton;
		//}
		ButtonExit.Text =	TextExitButton;
	}
}

simulated function BeforeHide () {
	if (!bFirstPreRound)
		ButtonReplay.HideWindow();
	//if (!bPlayingLastMap)
		ButtonNext.HideWindow();
	if (!bFirstPreRound)
		ButtonExit.HideWindow();
	
	if (bFirstPreRound) {
		FirstPreRoundOver();
		Game.EndFirstPreRound();
	}
}


////////////////////
//  Notify
////////////////////

simulated function Notify (UWindowDialogControl control, byte event)
{
	local int						weaponid, i;
	local class<s_Weapon>			w;
	local TO_SinglePlayerGame		game;


	// Replay current mission
	if (control == ButtonReplay) {
		if (event == DE_Click) {
			game = TO_SinglePlayerGame(GetPlayerOwner().Level.Game);
			if (game == none) {
				warn("TO_GUITabDebriefing::Notify: Cannot find TO_SinglePlayerGame actor!!");
				assert(false);
			}
			Hint = TextHintLoadingSame;
			game.bChangeLevels = false;
			CloseThisTab();
			game.RestartGame();
		}
		else if (event == DE_MouseMove) {
			Hint = TextHintReplay;
			AltHint = "";
		}
		else if (event == DE_MouseLeave) {
			Hint = TextSPHint;
			AltHint = TextAltHint;
		}
	}

	// Play next mission
	else if (control == ButtonNext)
	{
		if (bIsCutScene) {
			if (event == DE_Click)
			{
				OwnerInterface.Hide();
				GetPlayerOwner().ConsoleCommand("disconnect");
				Root.Console.CloseUWindow();
				Root.CreateWindow(class<UMenuBotmatchWindow>(DynamicLoadObject("TO_SinglePlayer.TO_StartSinglePlayerSC", class'Class')), 100, 100, 200, 200, Self, True);
			}
		}
		else if (bFirstPreRound) {
			if (event == DE_Click) {
				OwnerInterface.Hide();
			}
			else if (event == DE_MouseMove) {
				Hint = TextHintGoButton;
				AltHint = "";
			}
			else if (event == DE_MouseLeave) {
				Hint = TextHintStory;
				AltHint = TextAltHint;
			}
		}
		else if (bLastModMap) {
			if (event == DE_Click) {
				bTextOnly = true;
				bFirstPaint = true;
				PlayDisabledSound();
			}
			else if (event == DE_MouseMove) {
				Hint = TextHintNext;
				AltHint = "";
			}
			else if (event == DE_MouseLeave) {
				Hint = TextSPHint;
				AltHint = TextAltHint;
			}		}
		else if (bPlayingLastMap) {
			if (event == DE_Click) {
				if (bDisableNext) {
					PlayDisabledSound();
					Hint = TextUsedCheatsHint;
				}
				else {
					game = TO_SinglePlayerGame(GetPlayerOwner().Level.Game);
					if (game == none) {
						warn("TO_GUITabDebriefing::Notify: Cannot find TO_SinglePlayerGame actor!!");
						assert(false);
					}
					Hint = TextHintLoadingNext;
					//game.bChangeLevels = true;
					CloseThisTab();
					//game.RestartGame();
					GetPlayerOwner().Level.ServerTravel("SPEndMap.unr?Game=TO_SinglePlayer.TO_SP_CutScene", false);
				}
			}
			else if (event == DE_MouseMove) {
				Hint = TextConclusionNext;
				AltHint = "";
			}
			else if (event == DE_MouseLeave) {
				Hint = TextSPHint;
				AltHint = TextAltHint;
			}
		}
		else {
			if (event == DE_Click) {
				if (bDisableNext) {
					PlayDisabledSound();
					Hint = TextUsedCheatsHint;
				}
				else {
					game = TO_SinglePlayerGame(GetPlayerOwner().Level.Game);
					if (game == none) {
						warn("TO_GUITabDebriefing::Notify: Cannot find TO_SinglePlayerGame actor!!");
						assert(false);
					}
					Hint = TextHintLoadingNext;
					game.bChangeLevels = true;
					CloseThisTab();
					game.RestartGame();
				}
			}
			else if (event == DE_MouseMove) {
				Hint = TextHintNext;
				AltHint = "";
			}
			else if (event == DE_MouseLeave) {
				Hint = TextSPHint;
				AltHint = TextAltHint;
			}
		}
	}

	// cancel
	else if (control == ButtonExit)
	{
		if (event == DE_Click)
		{
			OwnerInterface.Hide();
			GetPlayerOwner().ConsoleCommand("disconnect");
			Root.Console.CloseUWindow();
			Root.CreateWindow(class<UMenuBotmatchWindow>(DynamicLoadObject("TO_SinglePlayer.TO_StartSinglePlayerSC", class'Class')), 100, 100, 200, 200, Self, True);
		}
		else if (event == DE_MouseMove) {
			Hint = TextHintExit;
			AltHint = "";
		}
		else if (event == DE_MouseLeave) {
			Hint = TextSPHint;
			AltHint = TextAltHint;
		}
	}
}

////////////////////
// TOBriefing_DrawData
////////////////////

simulated final function TOBriefing_DrawData (int ScreenType, Canvas Canvas)
{
	local float									y, xl, yl, oneline;
	local string								currentText;
	local float									col1, col1l, col2, col2l, pwidth;

	OwnerInterface.Design.SetHeadlineFont(Canvas);
	Canvas.Style = OwnerHud.ERenderStyle.STY_NORMAL;

	y = Top + Padding[Resolution];

	if (bFirstPaint)
		LoadTextScreenData(ScreenType, Canvas);

	col1 = Left + Padding[resolution];
	col1l = Left + Width / 2 - Padding[resolution];
	col2 = Left + Width / 2 + Padding[resolution];
	col2l = Left + Width - Padding[resolution];

	// Mission title
	currentText = TextScreenTitle;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
	Canvas.SetPos(Left + Width*0.5 - 0.5*xl, y);
	Canvas.DrawText(currentText);
	y += 2 * yl;

	if (Resolution > 1 )
		y += yl;

	OwnerInterface.Design.SetScoreboardFont(Canvas);

	Canvas.StrLen("abcABC", xl, oneline);

	// Story -- Paragraph 1
	currentText = TextScreenParagraph1;
	Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
	Canvas.SetPos(col1, y);
	Tool_DrawText(Canvas, currentText, col1, y, col2l - col1, xl, yl);
	y += yl;
	
	y += oneline;

	// Story -- Paragraph 2
	if (TextScreenParagraph2 != "") {
		currentText = TextScreenParagraph2;
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1, y);
		Tool_DrawText(Canvas, currentText, col1, y, col2l - col1, xl, yl);
		y += yl;
		
		y += oneline;
	}

	// Story -- Paragraph 3
	if (TextScreenParagraph3 != "") {
		currentText = TextScreenParagraph3;
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1, y);
		Tool_DrawText(Canvas, currentText, col1, y, col2l - col1, xl, yl);
		y += yl;
		
		y += oneline;
	}

	// Story -- Paragraph 4
	if (TextScreenParagraph4 != "") {
		currentText = TextScreenParagraph4;
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1, y);
		Tool_DrawText(Canvas, currentText, col1, y, col2l - col1, xl, yl);
		y += yl;
		
		y += oneline;
	}

	// Story -- Paragraph 5
	if (TextScreenParagraph5 != "") {
		currentText = TextScreenParagraph5;
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1, y);
		Tool_DrawText(Canvas, currentText, col1, y, col2l - col1, xl, yl);
		y += yl;

		y += oneline;
	}

	bFirstPaint = false;
}

simulated function LoadTextScreenData(int ScreenType, Canvas Canvas) {
	local TournamentGameReplicationInfo			gri;
	local TO_SinglePlayerGame					game;
	local TO_SinglePlayerDefinition				SPDef;	
	local TO_SinglePlayerMaplist				MyList;
	local int									textIndex;
	local float									xl, yl;


	if (ScreenType == 1) {
		
		gri = TournamentGameReplicationInfo(OwnerPlayer.GameReplicationInfo);

		game = TO_SinglePlayerGame(GetPlayerOwner().Level.Game);
		if (game == none) {
			if (TO_SP_CutScene(GetPlayerOwner().Level.Game) != none) {
				SPDef = TO_SP_CutScene(GetPlayerOwner().Level.Game).SPDef;
			}
			else {
				Canvas.StrLen("Could not find TO_SinglePlayerGame actor !!", xl, yl);
				Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
				Canvas.SetPos(Left + Width*0.5 - 0.5*xl, Top + Height/2 - yl / 2);
				Canvas.DrawText("Could not find TO_SinglePlayerGame actor !!");
				return;
			}
		}
		else
			SPDef = game.SinglePlayerDefinition;

		TextScreenTitle = SPDef.ScenarioName;
		TextScreenParagraph1 = SPDef.Story1;
		TextScreenParagraph2 = SPDef.Story2;
		TextScreenParagraph3 = SPDef.Story3;
		TextScreenParagraph4 = SPDef.Story4;
		TextScreenParagraph5 = SPDef.Story5;

	}
	else if (ScreenType == 2) {
		myList = GetPlayerOwner().GetEntryLevel().spawn(class'TO_SinglePlayerMaplist');

		if (myList == none) {
			warn("TO_GUITabDebriefing::SetCurrentMapAsPerfect: Could not spawn maplist actor.");
			assert(false);
		}

		textIndex = myList.ModVersionIndex;
		TextScreenTitle = myList.MapName[textIndex];
		TextScreenParagraph1 = myList.Story1[textIndex];
		TextScreenParagraph2 = myList.Story2[textIndex];
		TextScreenParagraph3 = myList.Story3[textIndex];
		TextScreenParagraph4 = myList.Story4[textIndex];
		TextScreenParagraph5 = myList.Story5[textIndex];

		myList.Destroy();

	}
}

////////////////////
// TODebriefing_DrawData
////////////////////

simulated final function TODebriefing_DrawData (Canvas Canvas)
{
	local float									y, xl, yl;
	local TournamentGameReplicationInfo			gri;
	local TO_SinglePlayerGame					game;
	local string								currentText;
	local float									col1, col1l, col2, col2l, pwidth, medal_width;
	local int									MedalNumber;

	OwnerInterface.Design.SetHeadlineFont(Canvas);
	Canvas.Style = OwnerHud.ERenderStyle.STY_NORMAL;

	gri = TournamentGameReplicationInfo(OwnerPlayer.GameReplicationInfo);
	y = Top + Padding[Resolution];

	bPerfect = true;

	if (bFirstDebriefingCall)
		CheckEnableNextButton();

	game = TO_SinglePlayerGame(GetPlayerOwner().Level.Game);
	if (game == none) {
		Canvas.StrLen("Could not find TO_SinglePlayerGame actor !!", xl, yl);
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(Left + Width*0.5 - 0.5*xl, Top + Height/2 - yl / 2);
		Canvas.DrawText("Could not find TO_SinglePlayerGame actor !!");
		return;
	}

	col1 = Left + Padding[resolution];
	col1l = Left + Width / 2 - Padding[resolution];
	col2 = Left + Width / 2 + Padding[resolution];
	col2l = Left + Width - Padding[resolution];

	// Mission title
	currentText = game.SinglePlayerDefinition.ScenarioName;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
	Canvas.SetPos(Left + Width*0.5 - 0.5*xl, y);
	Canvas.DrawText(currentText);
	y += 3 * yl;

	OwnerInterface.Design.SetScoreboardFont(Canvas);

	// Congradulation message
	currentText = game.SinglePlayerDefinition.CongradMSG;
	if (currentText == "")
		currentText = TextDefaultCongradMSG;
	Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
	Canvas.SetPos(col1, y);
	Tool_DrawText(Canvas, currentText, col1, y, col2l - col1, xl, yl);
	y += yl;

	/////////////
	// Statistics
	/////////////

	// Was playing in easy mode?
	if (Game.PlayMode == Game.EPlayMode.PM_EASY) {
		currentText = TextPlayedInEasyMode;
		Canvas.StrLen(currentText, xl, yl);
		y += yl;	// Add one line of spacing between congrad message and cheaters message
		Canvas.DrawColor = OwnerInterface.Design.ColorRed;
		Canvas.SetPos(col1, y);
		Canvas.DrawText(currentText);
		y += yl;
	}

	// Used cheats?
	if (Game.bUsedCheats) {
		currentText = TextUsedCheats;
		Canvas.StrLen(currentText, xl, yl);
		y += yl;	// Add one line of spacing between congrad message and cheaters message
		Canvas.DrawColor = OwnerInterface.Design.ColorRed;
		Canvas.SetPos(col1, y);
		Canvas.DrawText(currentText);
		y += yl;
	}

	y += 2 * yl;	// Add two lines of spacing between congradulation message and statistics

	if (!game.bUseHealthAsPerfectCriteria) {
		// Ennemy casualties
		currentText = TextEnemyCasualties;
		Canvas.StrLen(currentText, xl, yl);
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1l - xl, y);
		Canvas.DrawText(currentText);

		if (game.SP_NumEnemies == 0) {
			EnemyCasualties = -1;
			currentText = TextNoPercentage;
		}
		else {
			EnemyCasualties = int(100 * float(game.SP_EnemyCasualties) / float(game.SP_NumEnemies));
			currentText = string(EnemyCasualties) $ " %";
		}
		Canvas.StrLen(currentText, xl, yl);
		if (EnemyCasualties == 100 || EnemyCasualties == -1)
			Canvas.DrawColor = ColorBest;
		else if (EnemyCasualties == 0) {
			Canvas.DrawColor = ColorWorse;
			bPerfect = false;
		}
		else {
			Canvas.DrawColor = ColorNormal;
			bPerfect = false;
		}
		Canvas.SetPos(col2, y);
		//Canvas.DrawText(currentText $ " minimum = " $ string(MinimumEnemyCasualties));
		Canvas.DrawText(currentText);
		y += yl;

		// Friendly Casualties
		currentText = TextFriendlyCasualties;
		Canvas.StrLen(currentText, xl, yl);
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1l - xl, y);
		Canvas.DrawText(currentText);

		if (game.SP_NumTeammates == 0) {
			FriendlyCasualties = -1;
			currentText = TextNoPercentage;
		}
		else {
			FriendlyCasualties = int(100 * float(game.SP_FriendlyCasualties) / float(game.SP_NumTeammates));
			currentText = string(FriendlyCasualties) $ " %";
		}
		Canvas.StrLen(currentText, xl, yl);
		if (FriendlyCasualties == 0 || FriendlyCasualties == -1)
			Canvas.DrawColor = ColorBest;
		else if (FriendlyCasualties == 100) {
			Canvas.DrawColor = ColorWorse;
			bPerfect = false;
		}
		else {
			Canvas.DrawColor = ColorNormal;
			bPerfect = false;
		}
		Canvas.SetPos(col2, y);
		Canvas.DrawText(currentText);
		y += yl;

		// Friendly Fire Incidents
		currentText = TextFriendlyFireIncidents;
		Canvas.StrLen(currentText, xl, yl);
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1l - xl, y);
		Canvas.DrawText(currentText);

		currentText = string(game.SP_FriendflyFireIncidents);
		Canvas.StrLen(currentText, xl, yl);
		if (game.SP_FriendflyFireIncidents < game.FriendlyFireIncidentsThresholdMin)
			Canvas.DrawColor = ColorBest;
		else if (game.SP_FriendflyFireIncidents > game.FriendlyFireIncidentsThresholdMax) {
			Canvas.DrawColor = ColorWorse;
			bPerfect = false;
		}
		else {
			Canvas.DrawColor = ColorNormal;
			bPerfect = false;
		}
		Canvas.SetPos(col2, y);
		Canvas.DrawText(currentText);
		y += yl;

		// Efficiency
		currentText = TextEfficiency;
		Canvas.StrLen(currentText, xl, yl);
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1l - xl, y);
		Canvas.DrawText(currentText);

		if (game.SP_ShotsFired == 0) {
			Efficiency = -1;
			currentText = TextNoPercentage;
		}
		else {
			Efficiency = int(100 * float(game.SP_ShotsHitTarget) / float(game.SP_ShotsFired));
			currentText = string(Efficiency) $ " %";
		}
		Canvas.StrLen(currentText, xl, yl);
		if (Efficiency >= EfficiencyThreshold || Efficiency == -1)
			Canvas.DrawColor = ColorBest;
		else if (Efficiency <= 5) {
			Canvas.DrawColor = ColorWorse;
			bPerfect = false;
		}
		else {
			Canvas.DrawColor = ColorNormal;
			bPerfect = false;
		}
		Canvas.SetPos(col2, y);
		Canvas.DrawText(currentText);
		y += yl;
	}
	else {		// bUseHealthAsPerfectCriteria = true
		// Player health
		currentText = TextHealth;
		Canvas.StrLen(currentText, xl, yl);
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1l - xl, y);
		Canvas.DrawText(currentText);

		if (game.GetPlayer() == none)
			currentText = "Player not found ?!";
		if (game.GetPlayer().Health < HealthThreshold) {
			Canvas.DrawColor = ColorNormal;
			currentText = string(game.GetPlayer().Health) $ " %";
			bPerfect = false;
		}
		else {
			Canvas.DrawColor = ColorBest;
			currentText = string(game.GetPlayer().Health) $ " %";
		}
		Canvas.StrLen(currentText, xl, yl);
		Canvas.SetPos(col2, y);
		Canvas.DrawText(currentText);
		y += yl;
	}

	// Perfect ?
	if (bPerfect) {
		y += yl / 2;
		currentText = TextPerfectGame;
		Canvas.StrLen(currentText, xl, yl);
		Canvas.DrawColor = ColorBest;
		Canvas.SetPos(Left + Width / 2 - xl / 2, y);
		Canvas.DrawText(currentText);
		y += yl;

		if (bFirstDebriefingCall)
			SetCurrentMapAsPerfect();
	}

	y += yl;


	////////
	// Times
	////////

	// Colors
	if (game.SP_BestTOTime < game.SP_Time && game.SP_BestTOTime < game.SP_BestTime) {	// Kamehan time is the best
		ColorYourTime = ColorNormal;
		ColorYourBestTime = ColorNormal;
		ColorKamehanBestTime = ColorBest;
	}
	else if (game.SP_Time <= game.SP_BestTime) {	// New record for the player, and it beats kamehan!
		ColorYourTime = ColorBest;
		ColorYourBestTime = ColorNormal;
		ColorKamehanBestTime = ColorNormal;
	}
	else {	// No new record for the player, but the old one beats kamehan
		ColorYourTime = ColorNormal;
		ColorYourBestTime = ColorBest;
		ColorKamehanBestTime = ColorNormal;
	}

	// Player's time
	currentText = TextYourTime;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
	Canvas.SetPos(col1l - xl, y);
	Canvas.DrawText(currentText);

	currentText = class'TO_SinglePlayerGame'.static.CalcTextTime(game.SP_Time);
	Canvas.StrLen(currentText, xl, yl);
	Canvas.DrawColor = ColorYourTime;
	Canvas.SetPos(col2, y);
	Canvas.DrawText(currentText);
	y += yl;

	// Player's best time
	if (game.bPlayingAsTOTeam)
		currentText = TextUserBestTime;
	else
		currentText = TextYourBestTime;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
	Canvas.SetPos(col1l - xl, y);
	Canvas.DrawText(currentText);

	currentText = class'TO_SinglePlayerGame'.static.CalcTextTime(game.SP_BestTime);
	Canvas.StrLen(currentText, xl, yl);
	Canvas.DrawColor = ColorYourBestTime;
	Canvas.SetPos(col2, y);
	Canvas.DrawText(currentText);
	y += yl;

	// Kamehan's time
	currentText = TextKamehanTime;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
	Canvas.SetPos(col1l - xl, y);
	Canvas.DrawText(currentText);

	currentText = class'TO_SinglePlayerGame'.static.CalcTextTime(game.SP_BestTOTime);
	if (currentText == "N/A")
		currentText = "When It's Done";
	Canvas.StrLen(currentText, xl, yl);
	Canvas.DrawColor = ColorKamehanBestTime;
	Canvas.SetPos(col2, y);
	Canvas.DrawText(currentText);
	y += yl;

	// Next medal time
	if (bFirstDebriefingCall)
		GetMedalTimes(TimeBronze, TimeSilver, TimeGold);

	if (game.bUsesSpecialMedalConditions)
		currentText = TextConditions;
	else
		currentText = TextTime;
	
	if (BestReceivedMedal == 0)
		currentText = TextMedalBronze @ currentText;
	else if (BestReceivedMedal == 3)
		currentText = TextMedalSilver @ currentText;
	else
		currentText = TextMedalGold @ currentText;

	Canvas.StrLen(currentText, xl, yl);
	Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
	Canvas.SetPos(col1l - xl, y);
	Canvas.DrawText(currentText);

	if (game.bUsesSpecialMedalConditions) {
		if (BestReceivedMedal == 0)
			currentText = game.GetSpecialMedalText(3, 1);
		else if (BestReceivedMedal == 3)
			currentText = game.GetSpecialMedalText(2, 1);
		else
			currentText = game.GetSpecialMedalText(1, 1);
	}
	else {
		if (BestReceivedMedal == 0)
			currentText = class'TO_SinglePlayerGame'.static.CalcTextTime(TimeBronze);
		else if (BestReceivedMedal == 3)
			currentText = class'TO_SinglePlayerGame'.static.CalcTextTime(TimeSilver);
		else
			currentText = class'TO_SinglePlayerGame'.static.CalcTextTime(TimeGold);
	}
	Canvas.StrLen(currentText, xl, yl);
	Canvas.DrawColor = ColorNormal;
	Canvas.SetPos(col2, y);
	Canvas.DrawText(currentText);
	y += yl;

	y += yl;

	///////////////////
	// Cumulative Times
	///////////////////
	if (game.bShowCumulatives) {
		// Player's cumulative time
		currentText = TextCumulativeBesttime;
		Canvas.StrLen(currentText, xl, yl);
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1l - xl, y);
		Canvas.DrawText(currentText);

		currentText = class'TO_SinglePlayerGame'.static.CalcTextTime(game.SP_CumTime);
		Canvas.StrLen(currentText, xl, yl);
		if (game.SP_CumTime <= game.SP_CumTOTime)
			Canvas.DrawColor = ColorBest;
		else
			Canvas.DrawColor = ColorNormal;
		Canvas.SetPos(col2, y);
		Canvas.DrawText(currentText);
		y += yl;

		// Kamehan's cumulative time
		currentText = TextCumulativeKamehanTime;
		Canvas.StrLen(currentText, xl, yl);
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(col1l - xl, y);
		Canvas.DrawText(currentText);

		currentText = class'TO_SinglePlayerGame'.static.CalcTextTime(game.SP_CumTOTime);
		Canvas.StrLen(currentText, xl, yl);
		if (game.SP_CumTime <= game.SP_CumTOTime)
			Canvas.DrawColor = ColorNormal;
		else
			Canvas.DrawColor = ColorBest;
		Canvas.SetPos(col2, y);
		Canvas.DrawText(currentText);
		y += yl;
	}

	y += yl;

	// Draw medals
	currentText = "";
	medal_width = Height / 10;
	Canvas.SetPos(Left + Width / 2 - medal_width / 2, y);
	Canvas.Style = OwnerHud.ERenderStyle.STY_Masked;
	Canvas.DrawColor = OwnerHud.Design.ColorSuperwhite;
	if (game.bUsesSpecialMedalConditions) {
		MedalNumber = game.GetSpecialMedal();

		if (medalNumber == 1) {
			Canvas.DrawTile(Texture'to_sp_medal_gold_large', medal_width, medal_width, 0, 0, 64, 64);
			currentText = TextGoldMedal;
		}
		else if (MedalNumber == 2) {
			Canvas.DrawTile(Texture'to_sp_medal_silver_large', medal_width, medal_width, 0, 0, 64, 64);
			currentText = TextSilverMedal;
		}
		else if (MedalNumber == 3) {
			Canvas.DrawTile(Texture'to_sp_medal_bronze_large', medal_width, medal_width, 0, 0, 64, 64);
			currentText = TextBronzeMedal;
		}
		else
			currentText = "";

		if (bFirstDebriefingCall && MedalNumber >= 1)
			SaveMedal(MedalNumber);
	}
	else {
		if (game.SP_Time <= TimeGold) {
			Canvas.DrawTile(Texture'to_sp_medal_gold_large', medal_width, medal_width, 0, 0, 64, 64);
			currentText = TextGoldMedal;
			if (bFirstDebriefingCall)
				SaveMedal(1);
		}
		else if (game.SP_Time <= TimeSilver) {
			Canvas.DrawTile(Texture'to_sp_medal_silver_large', medal_width, medal_width, 0, 0, 64, 64);
			currentText = TextSilverMedal;
			if (bFirstDebriefingCall)
				SaveMedal(2);
		}
		else if (game.SP_Time <= TimeBronze) {
			Canvas.DrawTile(Texture'to_sp_medal_bronze_large', medal_width, medal_width, 0, 0, 64, 64);
			currentText = TextBronzeMedal;
			if (bFirstDebriefingCall)
				SaveMedal(3);
		}
	}

	Canvas.Style = OwnerHud.ERenderStyle.STY_NORMAL;
	if (currentText != "") {
		y += medal_width;
		y += Padding[Resolution];

		if (Resolution <= 1 )	// on low resolutions, put both lines of text on same line
			currentText = currentText @ TextMedal;
		Canvas.StrLen(currentText, xl, yl);
		Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
		Canvas.SetPos(Left + Width / 2 - xl / 2, y);
		Canvas.DrawText(currentText);	
		if (!(Resolution <= 1 && bPerfect)) // check to make sure we have enough space on low resolutions
			y += yl;

		if (Resolution > 1 ) {	// on high resolutions, use two lines
			currentText = TextMedal;
			Canvas.StrLen(currentText, xl, yl);
			Canvas.DrawColor = OwnerInterface.Design.ColorSuperwhite;
			Canvas.SetPos(Left + Width / 2 - xl / 2, y);
			Canvas.DrawText(currentText);	
			y += yl;
		}
	}

	// Check if we unlocked a new cheat
	if (bFirstDebriefingCall)
		CheckIfUnlockedCheat(BestReceivedMedal, bPerfect, (game.PlayMode == game.EPlayMode.PM_HARD)) ;


	// Unlocked cheat?
	if (bUnlockedCheat) {
		y += yl;
		currentText = TextNewCheat $ ": '" $ UnlockedCheatTitle $ "' !";
		Canvas.StrLen(currentText, xl, yl);
		Canvas.DrawColor = ColorBest;
		Canvas.SetPos(Left + Width / 2 - xl / 2, y);
		Canvas.DrawText(currentText);
		y += yl;
	}

	//Canvas.DrawTile(Texture'hud_elements2', 84, 47, 172, 95, 84.0, 47.0);	// legs

	bFirstPaint = false;
	bFirstDebriefingCall = false;
}


/*
================================================================================================================================================
   tools
================================================================================================================================================
*/

////////////////////
// TODebreifing_Tool_GetDescription
////////////////////

simulated final function string TODebreifing_Tool_GetDescription () {
	return TextTitle;
}

////////////////////
//  Tool_DrawClippedText
//  Copied from TO_GUIBaseMgr.uc
////////////////////

simulated function Tool_DrawText (Canvas Canvas, string text, float x, float y, float w, out float xl, out float yl)
{
	local float		oldorigx, oldclipx;


	oldorigx = Canvas.OrgX;
	oldclipx = Canvas.ClipX;

	Canvas.OrgX = x;
	Canvas.ClipX = w;

	Canvas.SetPos(0, y);
	Canvas.StrLen(text, xl, yl);
	Canvas.DrawText(text, false);

	Canvas.OrgX = oldorigx;
	Canvas.ClipX = oldclipx;
}

function CloseThisTab() {
	s_HUD(OwnerPlayer.myHud).bForceScores = false;
	Close();
}

function SetCurrentMapAsPerfect() {
	local TO_SinglePlayerMaplist	MyList;

	if (Game.bUsedCheats || Game.PlayMode == Game.EPlayMode.PM_EASY)
		return;

	myList = GetPlayerOwner().GetEntryLevel().spawn(class'TO_SinglePlayerMaplist');

	if (myList == none) {
		warn("TO_GUITabDebriefing::SetCurrentMapAsPerfect: Could not spawn maplist actor.");
		assert(false);
	}

	myList.SetCurrentAsPerfect();
	myList.Destroy();
}

function CheckIfUnlockedCheat(int WonMedal, bool bPerfect, bool bHardMode) {
	local TO_SinglePlayerMaplist	MyList;

	if (Game.bUsedCheats || Game.PlayMode == Game.EPlayMode.PM_EASY)
		return;

	myList = GetPlayerOwner().GetEntryLevel().spawn(class'TO_SinglePlayerMaplist');

	if (myList == none) {
		warn("TO_GUITabDebriefing::CheckIfUnlockedCheat: Could not spawn maplist actor.");
		assert(false);
	}

	bUnlockedCheat = class'TO_StartSinglePlayerCheatsCW'.static.CheckIfUnlockedCheat(myList, UnlockedCheatTitle, WonMedal, bPerfect, bHardMode);
	myList.Destroy();
}

function bool IsLastMap() {
	local TO_SinglePlayerMaplist	MyList;
	local bool						result;

	if (bFirstPreRound)
		return false;

	myList = GetPlayerOwner().GetEntryLevel().spawn(class'TO_SinglePlayerMaplist');

	if (myList == none) {
		warn("TO_GUITabDebriefing::IsLastMap: Could not spawn maplist actor.");
		assert(false);
	}

	if (myList.MapNum == myList.totalNumberOfMaps - 1)
		result = true;
	else
		result = false;

	myList.Destroy();

	return result;
}

function GetMedalTimes(out int bronze, out int silver, out int gold) {
	local TO_SinglePlayerMaplist	MyList;

	myList = GetPlayerOwner().GetEntryLevel().spawn(class'TO_SinglePlayerMaplist');

	if (myList == none) {
		warn("TO_GUITabDebriefing::GetMedalTimes: Could not spawn maplist actor.");
		assert(false);
	}

	myList.GetMedalTimes(bronze, silver, gold);
	BestReceivedMedal = myList.GetMedal(myList.MapNum);
	myList.Destroy();
}

function SaveMedal(int MedalNumber) {
	// MedalNumber: 1 = gold, 2 = silver, 3 = bronze
	local TO_SinglePlayerMaplist	MyList;

	if (bFirstPreRound)
		return;

	if (game.bUsedCheats || Game.PlayMode == Game.EPlayMode.PM_EASY)
		return;

	warn("Save medal function called.");

	myList = GetPlayerOwner().GetEntryLevel().spawn(class'TO_SinglePlayerMaplist');

	if (myList == none) {
		warn("TO_GUITabDebriefing::SaveMedal: Could not spawn maplist actor.");
		assert(false);
	}

	myList.SaveMedal(MedalNumber);
	myList.Destroy();
	BestReceivedMedal = MedalNumber;
}

function CheckLastModMap() {
	local TO_SinglePlayerMaplist	MyList;

	if (TO_RootWindow(Root).bTORetail == true) {
		bLastModMap = false;
		return;
	}

	myList = GetPlayerOwner().GetEntryLevel().spawn(class'TO_SinglePlayerMaplist');

	if (myList == none) {
		warn("TO_GUITabDebriefing::CheckLastModMap: Could not spawn maplist actor.");
		assert(false);
	}

	bLastModMap = myList.IsLastModMap(false);

	myList.Destroy();
}

function CheckEnableNextButton() {
	local TO_SinglePlayerMaplist	MyList;

	warn("Function called.");

	myList = GetPlayerOwner().GetEntryLevel().spawn(class'TO_SinglePlayerMaplist');

	if (myList == none) {
		warn("TO_GUITabDebriefing::CheckEnableNextButton: Could not spawn maplist actor.");
		assert(false);
	}

	//warn("myList.GetLastCompletedMap() == myList.MapNum = " $ (myList.GetLastCompletedMap() == myList.MapNum));
	//warn("Game.bUsedCheats = " $ Game.bUsedCheats );

	if (myList.GetLastCompletedMap() == myList.MapNum && Game.bUsedCheats)
		bDisableNext = true;
	else
		bDisableNext = false;

	myList.Destroy();
}

simulated function PlayDisabledSound() {
	GetPlayerOwner().PlaySound(Sound'hithelmet', SLOT_None);
}

///////////////////////////////////////
// defaultproperties
///////////////////////////////////////

defaultproperties
{
     bFirstPaint=True
     BestReceivedMedal=-1
     TextTitle="mission debriefing"
     TextDefaultCongradMSG="Congratulations! You successfully accomplished your mission."
     TextReplayButton="Replay Mission"
     TextNextButton="Next Mission"
     TextExitButton="Exit Single Player"
     TextReplayButtonShort="Replay"
     TextNextButtonShort="Next"
     TextExitButtonShort="Exit"
     TextHintReplay="Select this option to replay this mission."
     TextHintNext="Select this option to play the next mission."
     TextHintExit="Select this option to exit the single player mode and open the TO menu."
     TextHintLoadingSame="Please wait, reloading mission..."
     TextHintLoadingNext="Please wait, loading next mission..."
     TextUsedCheats="Cheaters do not get to keep their scores... Sorry!"
     TextUsedCheatsHint="You cannot access the next mission until you complete this one without cheating."
     TextPlayedInEasyMode="Note that to keep your scores and medals, you need to complete the mission in a mode other than Easy."
     TextEnemyCasualties="Enemy Casualties:"
     TextFriendlyCasualties="Friendly Casualties:"
     TextFriendlyFireIncidents="Friendly Fire Incidents:"
     TextEfficiency="Accuracy:"
     TextHealth="Player Condition:"
     TextPerfectGame="PERFECT!"
     TextNewCheat="You have unlocked a new cheat"
     TextYourTime="Your Time:"
     TextYourBestTime="Your Best Time:"
     TextUserBestTime="User's Best Time:"
     TextKamehanTime="TO Team Time:"
     TextMedalBronze="Bronze Medal"
     TextMedalSilver="Silver Medal"
     TextMedalGold="Gold Medal"
     TextTime="Time:"
     TextConditions="Conditions:"
     TextCumulativeBesttime="Your Cumulative Best Time:"
     TextCumulativeKamehanTime="Cumulative TO Team Time:"
     TextNoPercentage="N/A"
     TextHintStory="Good luck on your mission."
     TextHintGoButton="Select this option to start the mission."
     TextGoButton="Go!"
     TextGoButtonShort="Go!"
     TextEndButtonShort="END"
     TextEndButton="THE END"
     TextHintEnd="Thank you for playing"
     TextEndNext="Select this option to exit to main menu"
     TextConclusionButtonShort="Conclusion"
     TextConclusionButton="Conclusion"
     TextConclusionNext="Select this option to see the conclusion"
     TextGoldMedal="GOLD"
     TextSilverMedal="SILVER"
     TextBronzeMedal="BRONZE"
     TextMedal="MEDAL"
     bFirstPreRound=True
     bFirstDebriefingCall=True
     EfficiencyThreshold=50
     HealthThreshold=85
     ShowNav=True
}
