//=============================================================================
// TO_SPHCB_HostageRescuePlus.
//=============================================================================
// Allows one hostage rescue to win the round. The hostages can be spawned
// anywhere in the map (but only on navigation points). The hostages also get
// names from the TeamBotNames string (the slots 10, 11 and 12 are used)
// Finally, medals are modified to allow one medal layer per hostage rescued.
//=============================================================================
// Programmed by Mathieu 'EMH_Mark3' Mallet for Tactical Ops
// 19/11/2002
//=============================================================================

class TO_SPHCB_HostageRescuePlus expands TO_SP_HardCodedBehaviors;

var	TO_SinglePlayerGame		game;

var int		HostageDataOffset;
var int		lastNBRescuedHostages;

var localized string	GoldMedalText;
var localized string	SilverMedalText;
var localized string	BronzeMedalText;

var localized string	GoldMedalChars;
var localized string	SilverMedalChars;
var localized string	BronzeMedalChars;

var int		TimeGold;

function Initialise(TO_SinglePlayerGame myGame) {
	local TO_SinglePlayerMaplist	MyList;
	local int						temp;

	game = myGame;
	game.bUsesSpecialMedalConditions = true;

	myList = spawn(class'TO_SinglePlayerMaplist');

	if (myList == none) {
		warn("TO_SPHCB_HostageRescuePlus::Initialise: Could not spawn maplist actor.");
		assert(false);
	}

	myList.GetMedalTimes(temp, temp, TimeGold);
	myList.Destroy();

	GoldMedalText = GoldMedalText $ game.CalcTextTime(TimeGold);
}

function PostRestartRound() {
	local s_NPCHostage		Hostage;
	local int i;
	local navigationpoint	nav;

	i = 0;
	foreach allactors(class's_NPCHostage', hostage) {
		i++;
		if (i == 1) {
			hostage.tag = 'TO_SP_Hostage1';
			if (game.SinglePlayerDefinition.TeamBotsNames.names[HostageDataOffset] != "None")
				hostage.SetLocation(game.FindNavigationPoint(game.SinglePlayerDefinition.TeamBotsNames.names[HostageDataOffset]).location);
			game.ChangeName(hostage, game.SinglePlayerDefinition.TeamBotsNames.strings[HostageDataOffset], false);
		}
		else if (i == 2) {
			hostage.tag = 'TO_SP_Hostage2';
			if (game.SinglePlayerDefinition.TeamBotsNames.names[HostageDataOffset+1] != "None")
				hostage.SetLocation(game.FindNavigationPoint(game.SinglePlayerDefinition.TeamBotsNames.names[HostageDataOffset+1]).location);
			game.ChangeName(hostage, game.SinglePlayerDefinition.TeamBotsNames.strings[HostageDataOffset+1], false);
		}
		else if (i == 3) {
			hostage.tag = 'TO_SP_Hostage3';
			if (game.SinglePlayerDefinition.TeamBotsNames.names[HostageDataOffset+2] != "None")
				hostage.SetLocation(game.FindNavigationPoint(game.SinglePlayerDefinition.TeamBotsNames.names[HostageDataOffset+2]).location);
			game.ChangeName(hostage, game.SinglePlayerDefinition.TeamBotsNames.strings[HostageDataOffset+2], false);
		}
	}
}

function timer() {
	if (lastNBRescuedHostages != game.nbRescuedHostages) {
		CheckIfExitTriggerShouldBeSpawned();
		lastNBRescuedHostages = game.nbRescuedHostages;
	}
}

function CheckIfExitTriggerShouldBeSpawned() {
	local bool	bImportantHostageFound;
	local s_NPCHostage		Hostage;

	bImportantHostageFound = false;

	foreach allactors(class's_NPCHostage', hostage)
		if (hostage.tag == 'TO_SP_Hostage1')
			bImportantHostageFound = true;

	if (!bImportantHostageFound) {
		game.SpawnEndGameTriggers();
		game.SpecialTriggersMSG();
	}
}

function string GetSpecialMedalText(int MedalType, int MedalTextType) {
	if (MedalTextType == 1) { // Debreifing screen text
		if (medaltype == 1) // gold
			return GoldMedalText;
		else if (medaltype == 2) // silver
			return SilverMedalText;
		else if (medaltype == 3) // bronze
			return BronzeMedalText;
		else 
			return "";
	}
	else if (MedalTextType == 2) { // On-screen medal times indicator
		if (medaltype == 1) // gold
			return GoldMedalChars;
		else if (medaltype == 2) // silver
			return SilverMedalChars;
		else if (medaltype == 3) // bronze
			return BronzeMedalChars;
		else 
			return "";		
	}
}

function int GetSpecialMedal() {
	if (game.nbRescuedHostages == 1)
		return 0;	// no medal
	else if (game.nbRescuedHostages == 2)
		return 3;	// bronze
	else if (game.nbRescuedHostages == 3) {
		if (game.SP_Time > TimeGold)
			return 2;	// silver
		else
			return 1;	// gold
	}

	// need to add something to give gold medal
	return 1;
}

function DrawSpecialMedalTimes(Canvas Canvas, TO_DesignInfo Design, s_HUD mainHud, float y) {
	local int time, CellNumber, aDigit;
	
	time = s_GameReplicationInfo(game.GameReplicationInfo).RoundStarted - s_GameReplicationInfo(game.GameReplicationInfo).Remainingtime;
	if (game.GamePeriod == game.EGamePeriod.GP_PostMatch)
		time = game.SP_Time;
	
	if (mainHUD.IsPreRound()) {
		time = TimeGold;
		CellNumber = 0;
	}
	else if (time <= TimeGold && game.nbHostagesLeft + game.nbRescuedHostages == 3) {
		time = TimeGold - time;
		CellNumber = 0;
	}
	else if (game.nbHostagesLeft + game.nbRescuedHostages == 3) {
		time = -2;
		aDigit = 3 - game.nbRescuedHostages;
		CellNumber = 1;
	}
	else if (game.nbHostagesLeft + game.nbRescuedHostages == 2) {
		time = -2;
		aDigit = 2 - game.nbRescuedHostages;
		CellNumber = 2;
	}
	else {
		time = -1;
		CellNumber = 4;
	}
	Canvas.DrawColor = Design.ColorSuperwhite;
	game.TOHud_Tool_DrawDigitPlus(Canvas, CellNumber, 1);
	game.TOHud_Tool_DrawDigitPlus(Canvas, 3, 1);
	if (time == -2) {
		if (aDigit == 1)
			Canvas.SetPos(64 + 17, y+6);
		else
			Canvas.SetPos(64 + 7, y+6);
		mainHud.TOHud_Tool_DrawDigit(Canvas, aDigit, mainHud.ETOHudFontSize.FS_SMALL, 1);
		game.TOHud_Tool_DrawDigitPlus(Canvas, 16, 1);
		if (aDigit != 1)
			game.TOHud_Tool_DrawDigitPlus(Canvas, 17, 1);
	}
	else {
		Canvas.SetPos(45, y+6);
		mainHud.TOHud_Tool_DrawTime(Canvas, time, 2);
	}
}

function HostageKilled(pawn killer, pawn Other, name damageType) {
	if (Other.tag == 'TO_SP_Hostage1') {
		game.SetWinner(0);
		game.BroadcastLocalizedMessage(class'TO_SP_Messages', 0);
		game.EndGame(class'TO_SP_Messages'.default.WinMessage[0]);
	}
}

defaultproperties
{
     HostageDataOffset=10
     GoldMedalText="3 hostages rescued & "
     SilverMedalText="3 hostages rescued"
     BronzeMedalText="2 hostages rescued"
     GoldMedalChars="9"
     SilverMedalChars="2"
     BronzeMedalChars="1"
     bHandlesHostageKills=True
}
