//=============================================================================
// TO_SPHCB_MetalStorm.
//=============================================================================
// Special mission behaviors.
// Catches the 'OICW stolen' message & prevents all bots from going after the
// player when the exit is spawned.
//=============================================================================
// Programmed by Mathieu 'EMH_Mark3' Mallet for Tactical Ops
// 16/11/2002
//=============================================================================

class TO_SPHCB_MetalStorm expands TO_SP_HardCodedBehaviors;

var	TO_SinglePlayerGame		game;
var localized string		NoExitWithoutOICWMessage;

function Initialise(TO_SinglePlayerGame myGame) {
	local TO_SendBotOrder	botorder;
	
	game = myGame;
	game.ExitSpawnedBotsAttitude = 1; // make teammembers follow the player when exit is spawned but not the ennemy bots

	tag = 'OICW_done';

	foreach allactors(class'TO_SendBotOrder', botorder)
		botorder.tag = 'TO_SP_InvalidTag'; // Used to prevent the round winning actor from being activated

}

function Trigger( actor Other, pawn EventInstigator ) {
	game.SpawnEndGameTriggers();
	game.SpecialTriggersMSG();
	game.SpawnSpecialBots();
}

function bool AllowPlayerWin() {
	local bool		bAllowPlayerWin;

	bAllowPlayerWin = CheckForOICW();

	if (!bAllowPlayerWin)
		game.BroadcastMissionMessage(NoExitWithoutOICWMessage);

	return bAllowPlayerWin;
}

function bool CheckForOICW() {
	local pawn			pawn;
	local inventory		inventory;
	local bool			bOICWFound;
	
	bOICWFound = false;

	for ( pawn=Level.PawnList; pawn!=None; pawn=pawn.nextPawn )
		if ( (pawn.PlayerReplicationInfo != None) && !pawn.PlayerReplicationInfo.bWaitingPlayer )
			if ( ((s_Bot(pawn) != None) && (!s_Bot(pawn).bNotPlaying)) || s_player(pawn) != none)
				if (pawn.PlayerReplicationInfo.team == game.SinglePlayerDefinition.PlayerTeam)
					for (inventory=pawn.Inventory; inventory!=None; inventory=inventory.Inventory )
						if (inventory.IsA('s_OICW'))
							bOICWFound = true;
	
	return bOICWFound;
}

defaultproperties
{
     NoExitWithoutOICWMessage="You can't return without the OICW!"
}
