//=============================================================================
// TO_SPHCB_TeamOICWs.
//=============================================================================
// Special mission behaviors.
// Gives OICWs to everyone on the player's team.
// Also contains custom code to modify TO-Omega (last singleplayer mission)
//=============================================================================
// Programmed by Mathieu 'EMH_Mark3' Mallet for Tactical Ops
// 08/10/2002
//=============================================================================

class TO_SPHCB_TeamOICWs expands TO_SP_HardCodedBehaviors;

enum EWeaponTeam
{
	WT_Both,
	WT_SpecialForces,					
	WT_Terrorist,
};

enum ETeams
{
	ET_Terrorists,	
	ET_SpecialForces,	
	ET_Both,					
};

var	TO_SinglePlayerGame		game;
var bool					bFirstOICWGiven;
var bool					bFirstMinutePassed;
var int						currentSpecialBot;
var int						currentSecond;

var int						SpawnSpecialBotsTime;			// in seconds
var int						SpawnSpecialBotsIntervalMin;	// in seconcds
var int						SpawnSpecialBotsIntervalMax;	// in seconcds
var int						NextSpawnSpecialBotsInterval;	// in seconcds

var localized string		DisabledCTMessage;


function Initialise(TO_SinglePlayerGame myGame) {
	local TO_ConsoleTimer	ct;
	local TO_RoundWinning   rw;
	local TO_ScenarioInfo	si;
	local TO_MessageDisplay md;

	game = myGame;
	//game.bUseMultipleC4 = true;		// Signal the game to allow multiple c4 explosions

	// Hack to add the OICW to the weapons list
	// Fix for TO 3.4: weapon # is 25 not 24
	class'TOModels.TO_WeaponsHandler'.default.NumWeapons = 25;
	class'TOModels.TO_WeaponsHandler'.default.WeaponName[25] = "OICW Prototype";
	class'TOModels.TO_WeaponsHandler'.default.WeaponStr[25] = "TO_SinglePlayer.TO_SP_OICW";
	class'TOModels.TO_WeaponsHandler'.default.BotDesirability[25] = 0.45;
	class'TOModels.TO_WeaponsHandler'.default.WeaponTeam[25] = WT_Both;
	class'TOModels.TO_WeaponsHandler'.default.BuymenuScale[25]=1.0;

	
	// ** MAP MODIFICATIONS **

	// Disable engine room's console timer
	ct = TO_ConsoleTimer(game.FindActor("TO_ConsoleTimer0"));
	ct.CTDuration = 3;
	ct.EventCompleted = 'TO_SP_DisabledCT';
	ct.bCanResumeProgress = true;
	ct.SoundCompleted = Sound'TODatas.Weapons.def_fail';

	// Disabled CT's message
	md = TO_MessageDisplay(game.FindActor("TO_MessageDisplay1"));
	md.tag = 'TO_SP_DisabledCT';
	md.message = DisabledCTMessage;

	// Intercept trigger event and end round immediatly
	ct = None;
	ct = TO_ConsoleTimer(game.FindActor("TO_ConsoleTimer1"));
	ct.EventCompleted = 'terrwin';
	ct.CTDuration = 25;
	ct.bCanResumeProgress = true;

	// Add a winning round actor
	rw = TO_RoundWinning(game.FindActor("TO_RoundWinning0"));;
	rw.winner = ET_Terrorists;
	rw.WinningMessage = "Firing sequence initiated!";

	// Change default loose message
	si = TO_ScenarioInfo(game.FindActor("TO_ScenarioInfo0"));
	si.DefaultLooseMessage = "You failed to accomplish your objectives!";



}

function PreRestartRound() {
	bFirstOICWGiven = true;
	bFirstMinutePassed = false;
	currentSpecialBot = -1;
}

function GiveWeapons(pawn aPlayerPawn) {
	if (aPlayerPawn.IsA('s_Bot')) {
		if (s_BotBase(aPlayerPawn).TO_SinglePlayerBotType == 0) { // Bot is a normal bot
			if (aPlayerPawn.PlayerReplicationInfo.Team == Game.SinglePlayerDefinition.PlayerTeam) { // Bot is on the player's team
				if (bFirstOICWGiven) {
					bFirstOICWGiven = false;
					Game.GiveWeapon(aPlayerPawn, "TO_SinglePlayer.TO_SP_OICW_Trapped");
				}
				else {
					Game.GiveWeapon(aPlayerPawn, class'TOModels.TO_WeaponsHandler'.default.WeaponStr[24]);
				}
			}
		}
	}
}

function Timer() {
	if (game.RoundStarted - game.RemainingTime > SpawnSpecialBotsTime && !bFirstMinutePassed) {
		bFirstMinutePassed = true;
		NextSpawnSpecialBotsInterval = SpawnSpecialBotsIntervalMin;
		currentSecond = NextSpawnSpecialBotsInterval;
	}

	if (bFirstMinutePassed) {
		currentSecond++;
		if (currentSecond >= NextSpawnSpecialBotsInterval) {
			currentSecond = 0;
			SpawnOneSpecialBot();
			SpawnOneSpecialBot();
			NextSpawnSpecialBotsInterval = rand(SpawnSpecialBotsIntervalMax - SpawnSpecialBotsIntervalMin) + SpawnSpecialBotsIntervalMin;
		}
	}
}

function SpawnOneSpecialBot() {
	local bot NewBot;
	local NavigationPoint StartSpot, OldStartSpot;
	local int DesiredTeam, i, MinSize;

	game.bSpecialBotsSpawned = true;
	game.SP_NumEnemies += 1;
	currentSpecialBot += 1;

	if (currentSpecialBot == 63) {
		warn("Too many special bots spawned!!");
		assert(false);
	}

	game.currentSpecialBotNumber = currentSpecialBot;

	NewBot = game.SpawnBot(StartSpot);
	if ( NewBot == none )
	{
		warn("TO_SPHCB_TeamOICWs::SpawnOneSpecialBot: Failed to spawn bot at " $ StartSpot.name);
		//assert(false);
	}

	NewBot.Health = game.SinglePlayerDefinition.SpawnSpecialEnemyBotHealth;
	if (game.SinglePlayerDefinition.PlayerTeam == 0)
		DesiredTeam = 1;
	else
		DesiredTeam = 0;
	NewBot.PlayerReplicationInfo.bIsABot = True;

	NewBot.PlayerReplicationInfo.Team = DesiredTeam;

	//log("TO_SinglePlayer::SpawnSpecialBots: Spawnspot == " $ StartSpot.Name);

	NewBot.SetLocation(StartSpot.Location);
	NewBot.SetRotation(StartSpot.Rotation);
	NewBot.ViewRotation = StartSpot.Rotation;
	NewBot.SetRotation(NewBot.Rotation);

	game.ChangePModel(NewBot, game.SinglePlayerDefinition.SpawnSpecialEnemyBotSkin, NewBot.PlayerReplicationInfo.Team, false);
	game.DecodeAndGiveWeapons(NewBot, game.SinglePlayerDefinition.SpawnSpecialEnemyBotGivenWeapons);

}

event Destroyed ()
{
	class'TOModels.TO_WeaponsHandler'.default.NumWeapons = 24;	// Fix for TO 3.4
}

defaultproperties
{
     currentSpecialBot=-1
     SpawnSpecialBotsTime=45
     SpawnSpecialBotsIntervalMin=5
     SpawnSpecialBotsIntervalMax=15
     DisabledCTMessage="This console is defective!"
}
