//=============================================================================
// TO_SP_CutScene.
//=============================================================================
// Cutscene gametype for Single Player TO.
// Sends the player trough an interpolated path.
//=============================================================================
// Programmed by Mathieu 'EMH_Mark3' Mallet for Tactical Ops
// 13/10/2002
//=============================================================================

//class TO_SP_CutScene extends TO_SinglePlayerGame;
class TO_SP_CutScene extends TournamentGameInfo;
//class TO_SP_CutScene extends DeathmatchPlus;

var bool bOpened;
var int TickCount;

var config string CityIntroHUDClass;

var TO_SinglePlayerDefinition	SPDef;

var bool					bNoviceMode;
var	bool					bThreePlus;
var	ChallengeBotInfo		BotConfig;
var class<ChallengeBotInfo>	BotConfigType;
var int						NumBots;
var float					AirControl;
var playerpawn				PlayerLink;

event PreBeginPlay() {
	local TO_SinglePlayerMaplist	myList;
	local int						CutSceneIndex;

	CityIntroHUDClass = "TO_SinglePlayer.TO_SP_CutSceneHUD";

	HUDType = class<HUD>( DynamicLoadObject(CityIntroHUDClass, class'Class') );

	Super.PreBeginPlay();

	// Hack to add the OICW to the weapons list
	class'TOModels.TO_WeaponsHandler'.default.NumWeapons = 24;
	class'TOModels.TO_WeaponsHandler'.default.WeaponName[24] = "OICW Prototype";
	class'TOModels.TO_WeaponsHandler'.default.WeaponStr[24] = "TO_SinglePlayer.TO_SP_OICW";
	class'TOModels.TO_WeaponsHandler'.default.BotDesirability[24] = 0.45;
	class'TOModels.TO_WeaponsHandler'.default.WeaponTeam[24] = WT_Both;
	class'TOModels.TO_WeaponsHandler'.default.BuymenuScale[24]=1.0;

	/*bFirstPreRound = false;
	TO_SinglePlayerGame(Level.Game).RoundDelay = 10;
	TO_SinglePlayerGame(Level.Game).RoundLimit = 999;
	TO_SinglePlayerGame(Level.Game).RoundDuration = 9999;
	TO_SinglePlayerGame(Level.Game).PreRoundDuration1 = 0;
	TO_SinglePlayerGame(Level.Game).PreRoundDelay = 0;
	TO_SinglePlayerGame(Level.Game).MinAllowedScore = 30000;
	
	remainingSWATBots = 0;
	remainingTerrBots = 0;

	GamePeriod = GP_RoundPlaying;*/

	myList = spawn(Class'TOSystem.TO_SinglePlayerMaplist');
	CutSceneIndex = myList.CutSceneIndex;
	
	SPDef = spawn(Class'TO_SinglePlayer.TO_SinglePlayerDefinition');
	SPDef.Story1 = myList.Story1[CutSceneIndex];
	SPDef.Story2 = myList.Story2[CutSceneIndex];
	SPDef.Story3 = myList.Story3[CutSceneIndex];
	SPDef.Story4 = myList.Story4[CutSceneIndex];
	SPDef.Story5 = myList.Story5[CutSceneIndex];

}

function PostBeginPlay() {
	local TO_SP_BotSpawnPoint	sp;
	local Bot					myBot;
	local Navigationpoint		nav;

	BotConfig = spawn(BotConfigType);

	foreach allactors(class'TO_SP_BotSpawnPoint', sp) {
		myBot = SpawnBotPlus(sp, nav);
		ChangePModel(myBot, sp.Skin, int(sp.Team), false);
		myBot.Health = sp.Health;
	}

	foreach allactors(class'bot', myBot) {
		SetBotObjectives(s_Bot(myBot));
	}

	tag = 'finishedpath';
}

event InitGame( string Options, out string Error ) {
	Super.InitGame(Options, Error);

	//GamePeriod = GP_RoundPlaying;
}

event playerpawn Login
(
	string Portal,
	string Options,
	out string Error,
	class<playerpawn> SpawnClass
)
{
	local PlayerPawn NewPlayer;
	local s_Player	 myPlayer;
	local SpectatorCam Cam;

	//GamePeriod = GP_RoundPlaying;

	// Don't allow player to be a spectator
	if( !SpawnClass.Default.bCollideActors )
		SpawnClass = class'TMale2';

	bRatedGame = true;
	NewPlayer = Super.Login(Portal, Options, Error, SpawnClass);
	bRatedGame = false;
	NewPlayer.bHidden = True;

	/*if (NewPlayer.IsA('s_Player')) {
		//log("TO_SP_CutScene::Login: Player is a s_player!");
		myPlayer = s_Player(NewPlayer);
		myPlayer.GotoState('PlayerSpectating');
	}*/

	foreach AllActors(class'SpectatorCam', Cam) 
		NewPlayer.ViewTarget = Cam;

	PlayerLink = NewPlayer;

	return NewPlayer;
}

/* AcceptInventory()
Examine the passed player's inventory, and accept or discard each item
* AcceptInventory needs to gracefully handle the case of some inventory
being accepted but other inventory not being accepted (such as the default
weapon).  There are several things that can go wrong: A weapon's
AmmoType not being accepted but the weapon being accepted -- the weapon
should be killed off. Or the player's selected inventory item, active
weapon, etc. not being accepted, leaving the player weaponless or leaving
the HUD inventory rendering messed up (AcceptInventory should pick another
applicable weapon/item as current).
*/
function AcceptInventory(pawn PlayerPawn)
{
	local inventory Inv;
	local LadderInventory LadderObj;

	// DeathMatchPlus accepts LadderInventory
	for( Inv=PlayerPawn.Inventory; Inv!=None; Inv=Inv.Inventory )
	{
		if (Inv.IsA('LadderInventory'))
		{
			LadderObj = LadderInventory(Inv);
		} 
		else 	
			Inv.Destroy();
	}
	PlayerPawn.Weapon = None;
	PlayerPawn.SelectedItem = None;
}

function PlayTeleportEffect( actor Incoming, bool bOut, bool bSound)
{
}

function float PlaySpawnEffect(inventory Inv)
{
}

/*function bool SetPause( BOOL bPause, PlayerPawn P )
{
	return False;
}*/

auto state Startup
{
	function Tick(float DeltaTime)
	{
		local Pawn P;

		if( Level.LevelAction == LEVACT_Connecting )
		{
			bOpened = True;
			Disable('Tick');
			return;
		}

		TickCount++;
		if(TickCount < 2)
			return;

		if (DemoBuild == 1 && !bOpened )
		{
			for ( P=Level.PawnList; P!=None; P=P.NextPawn )
			{
				if ( P.IsA('PlayerPawn') && (PlayerPawn(P) != None)
					&& (PlayerPawn(P).Player != None)
					&& (TournamentConsole(PlayerPawn(P).Player.Console) != None) 
					&& PlayerPawn(P).ProgressMessage[0] == "")
					TournamentConsole(PlayerPawn(P).Player.Console).LaunchUWindow();
			}
		}
		bOpened = True;
		Disable('Tick');
	}
}

function bool CanSpectate( pawn Viewer, actor ViewTarget )
{
	return false;
}

function Bot SpawnBotPlus(TO_SP_BotSpawnPoint BotInfo, out NavigationPoint StartSpot) {
	local bot NewBot;
	local int BotN;
	local Pawn P;
	local int myDifficulty;
	local int botType; // 0 = normal bot, 1 = additionnal bot, 2 = special bot, 3 = VIP
	local int currentBotIndex;	// Used to read positions within an array

	//log("TO_SinglePlayerGame::SpawnBot: Called spawn bot with remainingSWATBots = " $ remainingSWATBots $ " and remainingTerrBots = " $ remainingTerrBots $ ".");
	
	botType = int(BotInfo.Attitude);
	
	myDifficulty = BotInfo.Intelligence;

	if ( myDifficulty >= 4 )
	{
		bNoviceMode = false;
		myDifficulty = myDifficulty - 4;
	}
	else
	{
		if ( myDifficulty > 3 )
		{
			myDifficulty = 3;
			bThreePlus = true;
		}
		bNoviceMode = true;
	}

	BotN = BotConfig.ChooseBotInfo();
	
	StartSpot = BotInfo;

	if( StartSpot == None )
	{
		warn("TO_SP_CutScene::SpawnBotPlus: Could not find starting spot for Bot");
		return None;
	}

	// Try to spawn the bot.
	NewBot = Spawn(BotConfig.CHGetBotClass(BotN),,,StartSpot.Location,StartSpot.Rotation);

	if ( NewBot == None ) {
		warn("TO_SP_CutScene::SpawnBotPlus: Couldn't spawn player at "$StartSpot);
		assert(false);
	}

	//if ( NewBot == None )
	//	NewBot = Spawn(BotConfig.CHGetBotClass(0),,,StartSpot.Location,StartSpot.Rotation);

	if ( NewBot != None )
	{
		// Set the player's ID.
		NewBot.PlayerReplicationInfo.PlayerID = CurrentID++;
		s_botBase(NewBot).TO_SinglePlayerBotType = botType;
		
		NewBot.PlayerReplicationInfo.Team = int(BotInfo.Team);
		
		BotConfig.CHIndividualize(NewBot, BotN, NumBots);

		Level.Game.ChangeName( NewBot, BotInfo.name, false);
		s_Bot(NewBot).TO_SinglePlayerBotAttitude = int(BotInfo.Attitude);

		NewBot.InitializeSkill(myDifficulty);

		NewBot.ViewRotation = StartSpot.Rotation;

		AddDefaultInventoryPlus( BotInfo, NewBot );
		NumBots++;
//		if ( bRequireReady && (CountDown > 0) )
//			NewBot.GotoState('Dying', 'WaitingForStart');
		NewBot.AirControl = AirControl;

	}

	return NewBot;
}

///////////////////////////////////////
// ChangePModel
///////////////////////////////////////

function ChangePModel( Pawn P, int num, int team, bool bDie) {
	ChangeModel(P, num);
}

///////////////////////////////////////
// ChangeModel
///////////////////////////////////////

function ChangeModel( Pawn P, int num) {
	num = class'TOPModels.TO_ModelHandler'.static.DressModel(P, num);
	
	// Set model num to pawn
	if ( P.IsA('s_Bot') )
		s_Bot(P).PlayerModel = num;
	else if (P.IsA('s_Player'))
		s_Player(P).PlayerModel = num;
}	

function AddDefaultInventoryPlus ( TO_SP_BotSpawnPoint sp, pawn PlayerPawn ) {
	local Bot				B;

	if ( PlayerPawn.IsA('s_Player') && PlayerPawn.PlayerReplicationInfo.bWaitingPlayer )
		return;

	//AddMoney(PlayerPawn, sp.GivenMoney);
	s_Bot(PlayerPawn).Money = sp.GivenMoney;
	DecodeAndGiveWeapons(PlayerPawn, sp.GivenWeapons);

	B = Bot(PlayerPawn);
	if ( B != None )
		B.bHasImpactHammer = false;

	BaseMutator.ModifyPlayer(PlayerPawn);

	PlayerPawn.SwitchToBestWeapon();
}	 


// Gives certain weapons to the bot, selected from the given string
function DecodeAndGiveWeapons (pawn aPlayerPawn, string CodedWeaponString) {
	local int		currentWeapon;
	local int		currentCharacterPos;
	local string	currentCharacter;

	currentWeapon = 0;
	currentCharacterPos = 0;

	if (CodedWeaponString == "" || Caps(CodedWeaponString) == "NONE" || left(CodedWeaponString,1) == " ") {
		return;		// musn't do anything, let the bot buy his stuff
	}

	KillInventory(aPlayerPawn);
	if (aPlayerPawn.IsA('s_Bot'))
		s_Bot(aPlayerPawn).money = 300;		// Just enough to buy some ammo

	while (currentWeapon <= 23 &&
		currentCharacterPos <= len(CodedWeaponString)) {
		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		if (currentCharacter != "-") {
			if (currentCharacter == caps(currentCharacter)) {
				if (class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon] == "s_SWAT.TO_M4A1")
					GiveWeapon(aPlayerPawn, "TO_SinglePlayer.TO_SP_CutSceneWeapon");
				else
					GiveWeapon(aPlayerPawn, class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon]);
			}
							
			currentWeapon++;
		}
		currentCharacterPos++;
	}

	currentCharacterPos++; // Skip the trailling '-'

	if (aPlayerPawn.IsA('s_bot')) {
		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		if (currentCharacter == caps(currentCharacter)) {
			s_bot(aPlayerPawn).HelmetCharge = 100;
		}

		currentCharacterPos++; // Skip the trailling '-'

		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		if (currentCharacter == caps(currentCharacter)) {
			s_bot(aPlayerPawn).VestCharge = 100;
		}

		currentCharacterPos++; // Skip the trailling '-'
	
		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		if (currentCharacter == caps(currentCharacter)) {
			s_bot(aPlayerPawn).LegsCharge = 100;
		}

		currentCharacterPos++; // Skip the trailling '-'

		// Hack to give OICW to bots
		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		if (currentCharacter == caps(currentCharacter) && currentCharacter != "") {
			GiveWeapon(aPlayerPawn, class'TOModels.TO_WeaponsHandler'.default.WeaponStr[24]);
		}
	}

}

final function KillInventory(Pawn P, optional bool buymenu) {
	local		Inventory		Inv, InvTmp;

	// Kill all weapons
	Inv = P.Inventory;
	While ( Inv != None )
	{
		InvTmp = Inv.Inventory;
		if ( !buymenu || ((Inv.Class != class's_SWAT.s_C4') && (Inv.Class != class's_SWAT.s_OICW')) )
		{
			Inv.Destroy();
		}
		Inv = InvTmp;
	}
	
	P.Weapon = None;
}

function GiveWeapon(Pawn PlayerPawn, string aClassName )
{
	local class<Weapon> WeaponClass;
	local Weapon NewWeapon;

	WeaponClass = class<Weapon>(DynamicLoadObject(aClassName, class'Class'));

	if( PlayerPawn.FindInventoryType(WeaponClass) != None )
		return;
	newWeapon = Spawn(WeaponClass);
	if ( newWeapon != None )
	{
		newWeapon.RespawnTime = 0.0;
		newWeapon.GiveTo(PlayerPawn);
		newWeapon.bHeldItem = true;
		newWeapon.GiveAmmo(PlayerPawn);
		newWeapon.SetSwitchPriority(PlayerPawn);
		newWeapon.WeaponSet(PlayerPawn);
		newWeapon.AmbientGlow = 0;
		if (newWeapon.IsA('s_weapon')) {
			s_weapon(newWeapon).TraceFrequency = 1;
			s_weapon(newWeapon).bTracingBullets = true;
		}
		if ( PlayerPawn.IsA('PlayerPawn') )
			newWeapon.SetHand(PlayerPawn(PlayerPawn).Handedness);
		else
			newWeapon.GotoState('Idle');
		PlayerPawn.Weapon.GotoState('DownWeapon');
		PlayerPawn.PendingWeapon = None;
		PlayerPawn.Weapon = newWeapon;
	}
}

function SetBotObjectives(s_Bot B) {
	//log("TO_SinglePlayerGame::SetBotObjectives: Func called for bot " $ string(B.name) $ " and attitude = " $ B.TO_SinglePlayerBotAttitude);
	
	switch (B.TO_SinglePlayerBotAttitude) {
		case 0:		// Do nothing, bot should get normal objectives
			break;

		case 1:		// Hold position; doesn't work too well
			B.SetOrders('Hold', B, false);
			//B.AccelRate = 0;
			B.GroundSpeed = 0;
			/*B.RotationRate.Pitch = 0;
			B.RotationRate.Roll = 0;
			B.RotationRate.Yaw = 0;*/

			//B.SetOrders('Hold', FindNavigationPoint("PathNode55").spawn(class'babycow'), false);
			break;

		case 2:		// Freelance (shouldn't be used, use 'normal objectives' instead
			B.SetOrders('Freelance', None, true);
			break;

		case 3:		// Roaming (shouln't be used, use 'normal objectives' instead
			B.SetOrders('Roaming', None, true);
			break;

		case 4:		// Special: tell the bot to kick the other bot's ass
			B.MoveTarget = NavigationPoint(FindActor("PathNode1"));
			B.Orders = 'Attack';
			//B.GotoState('Alarm', 'Attack');
			B.SetEnemy(FindBot(1));
			break;

		case 5:		// Defend at spawn; doesn't work in TO
			B.SetOrders('Defend', None, true);
			break;

	}
}

function Actor FindActor(string positionningActor) {
	local Actor a;
	foreach AllActors (class'Actor', a)
		if (string(a.name) ~= positionningActor)
			return a;

	warn("TO_SinglePlayerGame::FindActor: Could not find actor: " $ positionningActor);
	return None;
}

function s_bot FindBot(int botType) {
	local s_bot a;
	foreach AllActors (class's_bot', a)
		if (a.TO_SinglePlayerBotAttitude == botType)
			return a;

	return None;
}

function LoadAndShowDebreifingScreen() {
	local s_player	player, myPlayer;
	local TO_GUIBaseMgr UserInterface;

	foreach allactors(class's_player', player) {
		myPlayer = player;
	}

	UserInterface = s_hud(myPlayer.myHud).Userinterface;
	UserInterface.TOUI_Tool_AddTab(UserInterface.UIT_DEBRIEFING, class<TO_GUIBaseTab>(DynamicLoadObject("TO_SinglePlayer.TO_GUITabDebriefing", class'Class')));
	UserInterface.SelectTab(UserInterface.UIT_DEBRIEFING);
	
}

function Trigger( actor Other, pawn EventInstigator ) {
	if (EventInstigator.IsA('s_player')) {
		LoadAndShowDebreifingScreen();
	}
}


/*function bool SetPause( BOOL bPause, PlayerPawn P )
{
	//if( bPauseable || P.bAdmin || Level.Netmode==NM_Standalone )
	//{
		if( bPause )
			Level.Pauser=PlayerLink.PlayerReplicationInfo.PlayerName;
		else
			Level.Pauser="";
		return True;
	//}
	//else return False;
}*/

defaultproperties
{
     BotConfigType=Class's_SWAT.s_BotInfo'
     AirControl=0.350000
     Tag=finishedpath
}
