//=============================================================================
// TO_SP_CutSceneHUD.
//=============================================================================
// CutScene HUD for singleplayer. Hides unused elements of the s_HUD and
// always applies the widescreen effect.
//=============================================================================
// Programmed by Mathieu 'EMH_Mark3' Mallet for Tactical Ops
// 13/10/2002
//=============================================================================

class TO_SP_CutSceneHUD extends s_HUD;

////////////////////
//  PostRender
////////////////////

simulated function PostRender (canvas Canvas)
{
	local bool								hideCrap;
	local s_GameReplicationInfo				GRI;
	local s_Player							P;


	if ( !TOHud_Tool_BeforePaint(Canvas) )
	{
		if ( UserInterface != None )
			UserInterface.Hide();

		return;
	}

	P = s_Player(PlayerOwner);

	//------------------
	//  TO specific
	//------------------

	if ( P != None )
	{
		// widescreen
		TOHud_DrawWidescreen(Canvas, P.bHUDModFix);
	}

	//------------------
	//  main menu hide
	//------------------

	if ( (P != None) && (P.bMenuVisible) )
	{
		return;
	}

	if ( UserInterface != None )
		UserInterface.Render(Canvas);

	//------------------
	//  typing prompt
	//------------------

	if ( PlayerOwner.Player.Console.bTyping )
	{
		TOHud_DrawTypingPrompt(Canvas, PlayerOwner.Player.Console);
	}

	// startup
	if (bStartUpMessage && (Level.TimeSeconds < 5) )
	{
		bStartUpMessage = false;
		PlayerOwner.SetProgressTime(7);
	}

	// center & progress
	if (!bHideCenterMessages)
	{
		TOHud_DrawCentermessages(Canvas);
		if (PlayerOwner.ProgressTimeOut > Level.TimeSeconds)
		{
			DisplayProgressMessage(Canvas);
		}
	}	

	// mapchange
	if (PlayerOwner.GameReplicationInfo != None)
	{
		if ( PlayerPawn(Owner).GameReplicationInfo.RemainingTime == 0 )
		{
			if ( bDisplayMapChangeMessage ) 
			{
				PlayerOwner.ReceiveLocalizedMessage(class's_MessageRoundWinner', 8);
				bDisplayMapChangeMessage = false;
			}
		}
		else if ( !bDisplayMapChangeMessage )
		{
			bDisplayMapChangeMessage = true;
		}
	}

	//------------------
	//  debug
	//------------------

	if ( (P != None) && P.bShowDebug && !PlayerOwner.bShowScores )
	{
		ShowDebug(Canvas);
	}

	TOHud_DrawHints(Canvas);
	TOHud_DrawSpecials(Canvas);

}

defaultproperties
{
}
