//=============================================================================
// TO_SP_GUITabBuymenu.
//=============================================================================
// Adds functionnalities to the buy menu.
//=============================================================================
// Programmed by Mathieu 'EMH_Mark3' Mallet for Tactical Ops
// 12/11/2002
//=============================================================================

class TO_SP_GUITabBuymenu expands TO_GUITabBuymenu;
 
// Save button
var TO_GUIBaseButton			ButtonSave;
var localized string			TextSaveButton;
var localized string			HintSaveButton;

// Load button
var TO_GUIBaseButton			ButtonLoad;
var localized string			TextLoadButton;
var localized string			HintLoadButton;
var localized string			HintLoadButtonError;

// Saved state
var int							SavedAmmo[6];
var int							SavedAmmoID[6];
var int							SavedAmmoNum;

var bool						bSavedHelmet;
var bool						bSavedVest;
var bool						bSavedLegs;
var bool						bSavedNightVision;

var int							SavedPistol;
var int							SavedSub;
var int							SavedRifle;
var int							SavedGrenade;

// Has the player saved the inventory?
var bool						bSavedStatus;
var bool						bFirstAccessInRound;


simulated function Created () {
	Super.Created();
	
	ButtonSave = TO_GUIBaseButton(CreateWindow(class'TO_GUIBaseButton', 0, 0, WinWidth, WinHeight));
	ButtonSave.Text = TextSaveButton;
	ButtonSave.OwnerTab = self;

	ButtonLoad = TO_GUIBaseButton(CreateWindow(class'TO_GUIBaseButton', 0, 0, WinWidth, WinHeight));
	ButtonLoad.Text = TextLoadButton;
	ButtonLoad.OwnerTab = self;

}

simulated function BeforePaint (Canvas Canvas, float x, float y) {
	local int						wid, iid;

	if (!bInitialized)
	{
		Super.BeforePaint(Canvas, x, y);
		TOBuymenu_Init(Canvas);
		TOBuymenu_InitPlus(Canvas);
	}
	else
	{
		Super.BeforePaint(Canvas, x, y);
	}

	// setup buttons
	if (bDraw) {
		ButtonSave.ShowWindow();
		ButtonLoad.ShowWindow();
	}
}

simulated function Notify (UWindowDialogControl control, byte event) {
	local class<s_Weapon>			w;
	local int						weaponid, i;

	// Save
	if (control == ButtonSave) {
		if (event == DE_Click) {
			OwnerPlayer.PlaySound(Sound'scopezoom', SLOT_None);
			SaveWeaponConfiguration();
		}
		else if (event == DE_MouseMove)	{
			Hint = HintSaveButton;
			AltHint = "";
		}
		else if (event == DE_MouseLeave) {
			Hint = TextHintDefault;
			AltHint = TextHintDefaultAlt;
		}
	}

	// Load
	if (control == ButtonLoad) {
		if (event == DE_Click) {
			if (bSavedStatus) {
				OwnerPlayer.PlaySound(Sound'equip_nvg', SLOT_None);
				RestoreWeaponConfiguration();
			}
			else {
				OwnerPlayer.PlaySound(Sound'hithelmet', SLOT_None);
				Hint = HintLoadButtonError;
			}
		}
		else if (event == DE_MouseMove)	{
			Hint = HintLoadButton;
			AltHint = "";
		}
		else if (event == DE_MouseLeave) {
			Hint = TextHintDefault;
			AltHint = TextHintDefaultAlt;
		}
	}

	// buy
	else if (control == ButtonBuy)
	{
		if (event == DE_Click)
		{   /*
			if (RemainingMoney < 0)
			{
				OwnerPlayer.PlaySound(Sound'hithelmet', SLOT_None);
			}
			else
			{   */
				TOBuymenu_GetPlayerCapital();
//				TOBuymenu_SetPlayerInventory();
				SendEquipmentWishExtended();
//				OwnerPlayer.PlaySound(Sound'Kevlar', SLOT_None);
            /*
			if (RemainingMoney >= 0)
				OwnerInterface.Hide();
			*/
			//}
		}
		else if (event == DE_MouseMove)
		{
			Hint = TextHintBuybutton;
			AltHint = "";
		}
		else if (event == DE_MouseLeave)
		{
			Hint = TextHintDefault;
			AltHint = TextHintDefaultAlt;
		}
	}	
	else
		Super.Notify(control, event);

	if (control == ButtonBuy)
		if (event == DE_Click)
			TO_SinglePlayerGame(GetLevel().Game).PlayedBuyedStuff(OwnerPlayer);
}

simulated function BeforeShow ()
{
	Super.BeforeShow();
	if (bFirstAccessInRound) {
		RestoreWeaponConfiguration();
		OwnerPlayer.PlaySound(Sound'equip_nvg', SLOT_None);
		bFirstAccessInRound = false;
	}
}

simulated function BeforeHide () {
	Super.BeforeHide();
	ButtonSave.HideWindow();
	ButtonLoad.HideWindow();
}

simulated function TOBuymenu_InitPlus (Canvas Canvas) {
	local float				l, w, t, w2, w4;
	local int				i;

	w = Width*0.25 - Padding[Resolution];
	l = int(Width*0.125);
	w2 = int(w*0.5);
	w4 = int(w*0.4);

	ButtonSave.WinLeft = Left + Width - w - w;
	ButtonSave.WinTop = Top + int(0.85*Height);
	ButtonSave.SetWidth(Canvas, w - Padding[Resolution]);

	ButtonLoad.WinLeft = Left + Width - w - w;
	ButtonLoad.WinTop = Top + int(0.91*Height);
	ButtonLoad.SetWidth(Canvas, w - Padding[Resolution]);

}

simulated function ResetBuyMenu() {
	bFirstAccessInRound = true;
}

simulated function SaveWeaponConfiguration() {
	local int		i;

	bSavedStatus = true;

	// Save guns
	SavedPistol		= GetWeaponIDFromBox(BoxPistols);
	SavedSub		= GetWeaponIDFromBox(BoxSub);
	SavedRifle		= GetWeaponIDFromBox(BoxRifles);
	SavedGrenade	= GetWeaponIDFromBox(BoxGrenades);
	
	// Save ammo
	for (i = 0; i < NumAmmoBoxes; i++) {
		SavedAmmo[i] = AmmoBox[i].Value;
		SavedAmmoID[i] = AmmoBox[i].Data;
	}
	SavedAmmoNum = NumAmmoBoxes;

	// Save items
	bSavedHelmet =		BoxSpecialitems.IsSelectedByData(ICL_HELMET);
	bSavedVest =		BoxSpecialitems.IsSelectedByData(ICL_VEST);
	bSavedLegs =		BoxSpecialitems.IsSelectedByData(ICL_PADS);
	bSavedNightVision = BoxSpecialitems.IsSelectedByData(ICL_NIGHTVISION);
}

simulated function int GetWeaponIDFromBox(TO_GUITextListbox list) {
	local int		s;
	s = list.GetSelected();
	if (s == -1)
		return -1;
	else
		return list.GetData(s);
}

simulated function RestoreWeaponConfiguration() {
	local int i, j, weaponid;

	if (!bSavedStatus)
		return;

	// Restore weapons
	SelectCorrectWeaponFromBox(SavedPistol, BoxPistols);
	SelectCorrectWeaponFromBox(SavedSub, BoxSub);
	SelectCorrectWeaponFromBox(SavedRifle, BoxRifles);
	SelectCorrectWeaponFromBox(SavedGrenade, BoxGrenades);

	TOBuymenu_CalcInventory();
	
	// Restore ammo
	for (i = 0; i < MaxAmmoBoxes; i++) {
		for (j = 0; j < SavedAmmoNum; j++) {
			if (AmmoBox[i].data == SavedAmmoID[j]) {
				AmmoBox[i].value = SavedAmmo[j];
				weaponid = AmmoBox[i].data;
				if (weaponid >= 0)
					SelectedClips[weaponid].Primary = AmmoBox[i].value;
				else
					SelectedClips[-weaponid].Secondary = AmmoBox[i].value;
			}	
		}
	}

	// Restore items
	if (bSavedHelmet)
		SelectItemFromBox(ICL_HELMET, BoxSpecialitems);
	if (bSavedVest)
		SelectItemFromBox(ICL_VEST, BoxSpecialitems);
	if (bSavedLegs)
		SelectItemFromBox(ICL_PADS, BoxSpecialitems);
	if (bSavedNightVision)
		SelectItemFromBox(ICL_NIGHTVISION, BoxSpecialitems);

}

simulated function SelectCorrectWeaponFromBox(int ID, TO_GUITextListbox box) {
	local int i;
	if (ID == -1 || box.NumItems == 0)
		return;

	for (i = 0; i < box.NumItems; i++) {
		if (box.GetData(i) == ID) {
			box.Selected[i] = box.LBSEL_YES;
			box.SelectedIndex = i;
		}
		else
			box.Selected[i] = box.LBSEL_NO;
	}
}

simulated function SelectItemFromBox(int item, TO_GUIImageListbox box) {
	local int i;

	for (i = 0; i < box.NumItems; i++)
		if ( box.ItemsData[i] == item)
			box.Selected[i] = box.LBSEL_YES;
}

// SendEquipmentWish - conduct and send wish list to server
final simulated function SendEquipmentWishExtended()
{
	local s_Weapon		W;
	local int			i, WeaponID;

	i = 0;

	OwnerPlayer.Equip.Flags = 0;
	OwnerPlayer.Equip.Weapon2 = 0;
	OwnerPlayer.Equip.Weapon3 = 0;
	OwnerPlayer.Equip.Weapon4 = 0;
	OwnerPlayer.Equip.Weapon5 = 0;
	OwnerPlayer.Equip.Ammo = 0;
	OwnerPlayer.Equip.BackupAmmo = 0;

	// get weapons & their ammo

	// pistols
	i = BoxPistols.GetSelected();
	if (i != -1)
	{
		WeaponID = BoxPistols.GetData(i);
		OwnerPlayer.Equip.Weapon2 = WeaponID;
		OwnerPlayer.Equip.Ammo = OwnerPlayer.Equip.Ammo | (WishAmmo(WeaponID, true) & 255);
		OwnerPlayer.Equip.BackupAmmo = OwnerPlayer.Equip.BackupAmmo | (WishAmmo(WeaponID, false) & 255);
	}

	// smg/shotguns
	i = BoxSub.GetSelected();
	if (i != -1)
	{
		WeaponID = BoxSub.GetData(i);
		OwnerPlayer.Equip.Weapon3 = WeaponID;
		OwnerPlayer.Equip.Ammo = OwnerPlayer.Equip.Ammo | ((WishAmmo(WeaponID, true) & 255) << 8);
		OwnerPlayer.Equip.BackupAmmo = OwnerPlayer.Equip.BackupAmmo | ((WishAmmo(WeaponID, false) & 255) << 8);
	}

	// rifles
	i = BoxRifles.GetSelected();
	if (i != -1)
	{
		WeaponID = BoxRifles.GetData(i);
		OwnerPlayer.Equip.Weapon4 = WeaponID;
		OwnerPlayer.Equip.Ammo = OwnerPlayer.Equip.Ammo | ((WishAmmo(WeaponID, true) & 255) << 16);
		OwnerPlayer.Equip.BackupAmmo = OwnerPlayer.Equip.BackupAmmo | ((WishAmmo(WeaponID, false) & 255) << 16);
	}

	// nades
	i = BoxGrenades.GetSelected();
	if (i != -1)
	{
		WeaponID = BoxGrenades.GetData(i);
		OwnerPlayer.Equip.Weapon5 = WeaponID;
	}

	// items
	if (BoxSpecialitems.IsSelectedByData(ICL_HELMET))
	{
		OwnerPlayer.Equip.Flags = OwnerPlayer.Equip.Flags | 128;
	}
	if (BoxSpecialitems.IsSelectedByData(ICL_VEST))
	{
		OwnerPlayer.Equip.Flags = OwnerPlayer.Equip.Flags | 64;
	}
	if (BoxSpecialitems.IsSelectedByData(ICL_PADS))
	{
		OwnerPlayer.Equip.Flags = OwnerPlayer.Equip.Flags | 32;
	}
	if (BoxSpecialitems.IsSelectedByData(ICL_NIGHTVISION))
	{
		OwnerPlayer.Equip.Flags = OwnerPlayer.Equip.Flags | 16;
	}

	// knives
	OwnerPlayer.Equip.Flags = OwnerPlayer.Equip.Flags | (AmmoBox[0].Value & 7);

	// send list
	//TO_SinglePlayerGame(GetPlayerOwner().Level.Game).xxBuyList(OwnerPlayer,OwnerPlayer.Equip);
	TO_SinglePlayerGame(GetPlayerOwner().Level.Game).xxBuyList(OwnerPlayer);
}

defaultproperties
{
     TextSaveButton="save"
     HintSaveButton="Click to save the weapon configuration for use next round"
     TextLoadButton="load"
     HintLoadButton="Click to restore the weapon configuration"
     HintLoadButtonError="You must have saved your weapon configuration before being able to load it"
     bFirstAccessInRound=True
}
