//=============================================================================
// TO_SP_HUD.
//=============================================================================
// Single player hud. Extends the normal TO HUD
//=============================================================================
// Programmed by Mathieu 'EMH_Mark3' Mallet for Tactical Ops
// 03/11/2002
//=============================================================================

class TO_SP_HUD expands s_hud;

var	TO_SinglePlayerGame		Game;


var localized string		TextPlayingHardMode;
var localized string		TextPlayingLudicrousMode;
var localized string		TextPlayingEasyMode;

var int						PlayModeBarOffset;



simulated function PostBeginPlay () {
	Super.PostBeginPlay();

	Game = TO_SinglePlayerGame(Level.Game);

	if (Game == none) {
		warn("Could not spawn TO_SinglePlayerGame actor!");
		assert(false);
	}
}

////////////////////
// status
////////////////////

simulated function TOHud_DrawStatus (Canvas Canvas) {
	local int				helmet, body, legs, i;
	local color				ItemsColor;
	local float				ArmorGuyOffset;


	if (bHideHud || bHideStatus || (PawnOwner == None) )
	{
		return;
	}

	if (Game.PlayMode == Game.EPlayMode.PM_NORMAL)
		ItemsColor = Design.ColorSuperwhite;
	else if (Game.PlayMode == Game.EPlayMode.PM_HARD) {
		ItemsColor = Design.ColorOrange;
		ArmorGuyOffset = 16;
	}
	else if (Game.PlayMode == Game.EPlayMode.PM_LUDICROUS) {
		ItemsColor = Design.ColorRed;
		ArmorGuyOffset = 16;
	}
	else if (Game.PlayMode == Game.EPlayMode.PM_EASY) {
		ItemsColor = Design.ColorGreen;
		ArmorGuyOffset = 16;
	}


	// background
	if (bDrawBackground)
	{
		Canvas.DrawColor = ItemsColor;

		Canvas.Style = ERenderStyle.STY_Translucent;
		Canvas.SetPos(0, Canvas.ClipY-33);
		Canvas.DrawTile(Texture'hud_elements', 151, 26, 0, 0, 151.0, 26.0);		// health

		Canvas.DrawColor = Design.ColorSuperwhite;

		Canvas.Style = ERenderStyle.STY_Masked;
		Canvas.SetPos(0, Canvas.ClipY-33);
		Canvas.DrawTile(Texture'hud_elements', 151, 26, 0, 26, 151.0, 26.0);	// health
	}

	// armor
	if (bDrawArmorguy && IsPlayerOwner())
	{
		Canvas.DrawColor = ItemsColor;

		Canvas.Style = ERenderStyle.STY_Translucent;
		Canvas.SetPos(16, Canvas.ClipY-224-ArmorGuyOffset);
		Canvas.DrawTile(Texture'hud_elements2', 86, 162, 0, 0, 86.0, 162.0);			// armorguy

		Canvas.DrawColor = Design.ColorSuperwhite;

		Canvas.Style = ERenderStyle.STY_Masked;
		Canvas.SetPos(16, Canvas.ClipY-224-ArmorGuyOffset);
		Canvas.DrawTile(Texture'hud_elements2', 86, 162, 86, 0, 86.0, 162.0);		// armorguy

		if ( PawnOwner.IsA('s_Player') )
		{
			helmet = s_Player(PawnOwner).HelmetCharge;
			body = s_Player(PawnOwner).VestCharge;
			legs = s_Player(PawnOwner).LegsCharge;
		}
		else if ( PawnOwner.IsA('s_Bot') )
		{
			helmet = s_Bot(PawnOwner).HelmetCharge;
			body = s_Bot(PawnOwner).VestCharge;
			legs = s_Bot(PawnOwner).LegsCharge;
		}

		Canvas.Style = ERenderStyle.STY_Translucent;

		if (helmet > 0)
		{
			TOHud_Tool_SetPercentColor(Canvas, helmet);
			Canvas.SetPos(16, Canvas.ClipY-224-ArmorGuyOffset);
			Canvas.DrawTile(Texture'hud_elements2', 84, 19, 172, 0, 84.0, 19.0);		// helmet
		}

		if (body > 0)
		{
			TOHud_Tool_SetPercentColor(Canvas, body);
			Canvas.SetPos(16, Canvas.ClipY-205-ArmorGuyOffset);
			Canvas.DrawTile(Texture'hud_elements2', 84, 70, 172, 19, 84.0, 70.0);	// body
		}

		if (legs > 0)
		{
			TOHud_Tool_SetPercentColor(Canvas, legs);
			Canvas.SetPos(16, Canvas.ClipY-145-ArmorGuyOffset);
			Canvas.DrawTile(Texture'hud_elements2', 84, 47, 172, 95, 84.0, 47.0);	// legs
		}
	}

	// health
	Canvas.SetPos(16, Canvas.ClipY-42);
	Canvas.Style = ERenderStyle.STY_Masked;
	
	if ( PawnOwner.Health < 33 )
		Canvas.DrawColor = Design.ColorSuperwhite * (1.0-(1.5*TutIconBlink)) + (Design.ColorRed) * (1.5*TutIconBlink);
	else
		Canvas.DrawColor = Design.ColorSuperwhite;
	
	TOHud_Tool_DrawNum(Canvas, Max(0, PawnOwner.Health), FS_BIG, 3);

	if (ArmorGuyOffset != 0) {
		Canvas.CurY = Canvas.ClipY - PlayModeBarOffset;
		for (i = 102 - 16; i >= -20; i -= 16) {
			Canvas.CurX = i;
			Canvas.DrawColor = ItemsColor;
			Canvas.Style = ERenderStyle.STY_Translucent;
			Canvas.DrawTile(Texture'hud_elements', 16, 19, 17, 256, 16.0, -19);	// bg

			Canvas.CurX = i;
			Canvas.DrawColor = Design.ColorSuperwhite;
			Canvas.Style = ERenderStyle.STY_Masked;
			Canvas.DrawTile(Texture'hud_elements', 16, 19, 67, 256, 16.0, -19);	// fg
		}

		Canvas.DrawColor = ItemsColor;
		Canvas.Style = ERenderStyle.STY_Translucent;		// right
		Canvas.SetPos(102, Canvas.ClipY - PlayModeBarOffset);
		Canvas.DrawTile(Texture'hud_elements', 16, 19, 34, 256, 16.0, -19);

		Canvas.DrawColor = Design.ColorSuperwhite;
		Canvas.Style = ERenderStyle.STY_Masked;
		Canvas.SetPos(102 - 1, Canvas.ClipY - PlayModeBarOffset);
		Canvas.DrawTile(Texture'hud_elements', 17, 19, 84, 256, 17.0, -19);
	}
	if (Game.PlayMode == Game.EPlayMode.PM_HARD) {
		Canvas.DrawColor = ItemsColor;
		Canvas.Font = Design.Font10;
		Canvas.SetPos(4, Canvas.ClipY - PlayModeBarOffset + 5);
		Canvas.DrawText(TextPlayingHardMode, true);
	}
	else if (Game.PlayMode == Game.EPlayMode.PM_LUDICROUS) {
		Canvas.DrawColor = ItemsColor;
		Canvas.Font = Design.Font10;
		Canvas.SetPos(4, Canvas.ClipY - PlayModeBarOffset + 5);
		Canvas.DrawText(TextPlayingLudicrousMode, true);
	}
	else if (Game.PlayMode == Game.EPlayMode.PM_EASY) {
		Canvas.DrawColor = ItemsColor;
		Canvas.Font = Design.Font10;
		Canvas.SetPos(4, Canvas.ClipY - PlayModeBarOffset + 5);
		Canvas.DrawText(TextPlayingEasyMode, true);
	}


}


simulated function TOHud_DrawRoundtime (Canvas Canvas)
{
	local s_GameReplicationInfo				GRI;
	local int								time, min, secs;


	if (bHideHud || !bDrawTime) {
		return;
	}

	if (PawnOwner == None) {
		warn("TOHud_DrawRoundtime - PawnOwner == none");
		return;
	}

	time = s_SWATGame(Level.Game).GetSPCurrentTime();

	if (time >= 600) {
		if (bDrawBackground) {
			Canvas.DrawColor = Design.ColorSuperwhite;
			Canvas.Style = ERenderStyle.STY_Translucent;
			Canvas.SetPos(Canvas.ClipX - 103, 31);
			Canvas.DrawTile(Texture'to_sp_hud_elements', 103, 18, 0, 64, 103.0, 18.0);

			Canvas.Style = ERenderStyle.STY_Masked;
			Canvas.SetPos(Canvas.ClipX - 103, 31);
			Canvas.DrawTile(Texture'to_sp_hud_elements', 103, 18, 0, 82, 103.0, 18.0);
		}

		Canvas.DrawColor = Design.ColorSuperwhite;
		Canvas.SetPos(Canvas.ClipX - 88, 39);
		TOHud_Tool_DrawTime(Canvas, time, 2);
	}
	else {
		if (bDrawBackground) {
			Canvas.DrawColor = Design.ColorSuperwhite;
			Canvas.Style = ERenderStyle.STY_Translucent;
			Canvas.SetPos(Canvas.ClipX - 103, 31);
			Canvas.DrawTile(Texture'hud_elements', 103, 18, 103, 88, 103.0, 18.0);

			Canvas.Style = ERenderStyle.STY_Masked;
			Canvas.SetPos(Canvas.ClipX - 103, 31);
			Canvas.DrawTile(Texture'hud_elements', 103, 18, 103, 106, 103.0, 18.0);
		}

		Canvas.DrawColor = Design.ColorSuperwhite;
		Canvas.SetPos(Canvas.ClipX - 71, 39);
		TOHud_Tool_DrawTime(Canvas, time, 1);
	}
}

simulated function ResetHUDStuff() {
	local int i;

	// clear death messages
	s_DeathM_idx = 0;

	// clear short messages
	for (i = 0; i < 4; i++)
		ClearMessage(ShortMessageQueue[i]);

	// do not display cash 'earned' during last round
	MoneyVariationAmount = s_Player(PlayerOwner).Money;
	MoneyDrawTime = 0;
}


event Destroyed ()
{
	// Ugly hack
	class'TOModels.TO_WeaponsHandler'.default.NumWeapons = 24;	// Fix for TO 3.4
	class'TOModels.TO_WeaponsHandler'.default.WeaponName[25] = "";
	class'TOModels.TO_WeaponsHandler'.default.WeaponStr[25] = "";
	class'TOModels.TO_WeaponsHandler'.default.BotDesirability[25] = 0;
	class'TOModels.TO_WeaponsHandler'.default.BuymenuScale[25]=1.0;

	super.destroyed();
}


defaultproperties
{
     TextPlayingHardMode="      Hard Mode"
     TextPlayingLudicrousMode="LUDICROUS Mode!"
     TextPlayingEasyMode="     Easy Mode :)"
     PlayModeBarOffset=68
}
