//=============================================================================
// TO_SinglePlayerGame.
//=============================================================================
// Single player main code. Where most of the magic happens.
//=============================================================================
// Programmed by Mathieu 'EMH_Mark3' Mallet for Tactical Ops
// 19/08/2002
//=============================================================================

class TO_SinglePlayerGame expands s_SWATGame;

var		TO_SinglePlayerDefinition	SinglePlayerDefinition;
var		TO_SP_HardCodedBehaviors	hardCodedBehaviors;
var		bool						bPlayingAsTOTeam;
var		bool						bShowCumulatives;
var		bool						bFirstPreRound;
var		bool						bCutScene;
var		string						ProtagonistName;
var		bool						bUsedCheats;
var		s_Player					Protagonist;
var		bool						bAlreadyReplacedScoreboard;
var		TO_SP_GUITabScores			ScoreBoardLink;
var		TO_SP_GUITabBuymenu			BuyMenuLink;

var		float						GameSpeed;				// Scale applied to game rate.

var enum EPlayMode
{
	PM_NORMAL,
	PM_HARD,
	PM_LUDICROUS,
	PM_EASY,
} PlayMode;

	const UIT_SCORES =		10;


var		int		remainingSWATBots;
var		int		remainingTerrBots;
var		int		currentBotNumber;
//var		name	SpawnAdditionnalEnemyBotAt[16];
//var		string  SpawnAdditionnalEnemyBotName[16];
var		int     NumberOfAdditionnalEnemyBot;
var		int     NumberOfSpecialBot;
var		int		NumberOfVIP;
var		bool	bUseMultipleC4;

var		bool	bVIPDied;
var		bool	bC4ExplodedPlus;
var		bool    bEndGameTriggersSpawned;
var		bool	bSpecialBotsSpawned;
var		bool	bSpecialTriggersSpawned;
var		bool	bSpawnedSpecialBotsBeforeEndGameTriggers;
var		bool	bAlreadyWon;
var		bool	bGameWon;
var		bool	bAlreadyBroadcastedAllEnnemiesDeadMSG;
var		bool	bAlreadyBroadcastedVIPDeadMSG;
var		bool	bAlreadyBroadcastedSpecialTriggersMSG;
var		bool	bAlreadyBroadcastedC4ExplodedMSG;
var		bool	bC4Defused;
var		bool	bNoHostageRescueAllowed;
var		bool	bNoHostageDeathsAllowed;
var		actor	anExitObject;
var		int		ExitSpawnedBotsAttitude;
var		bool	bUseHealthAsPerfectCriteria;

var		int		currentAdditionnalBotNumber;
var		int		currentSpecialBotNumber;
var		int		currentTeamBotNumber;

var		int		FriendlyFireIncidentsThresholdMin;	// Number of allowed incidents that results in an 'imperfect' score
var		int		FriendlyFireIncidentsThresholdMax;	// Number of required inccidents to get a 'disaster' score

var		int		SP_CurrentRemainingTime;	// Current round time
var		int		SP_BestCurrentTime;			// Best Time displayed on HUD
var		int		SP_Time;					// Best current time
var		int		SP_BestTime;				// Best best user time
var		int		SP_BestTOTime;				// Best TO Team time
var		int		SP_CumTime;					// Cumulative player time
var		int		SP_CumTOTime;				// Cumulative TO Team time
var		int		SP_EnemyCasualties;			// Number of killed enemies
var		int		SP_NumEnemies;				// Number of enemies in the game
var		int		SP_FriendlyCasualties;		// Number of killed teammates
var		int		SP_NumTeammates;
var		int		SP_FriendflyFireIncidents;	// Number of times the player hit a friendly
var		int		SP_ShotsFired;				// Number of shots fired by the player
var		int		SP_ShotsHitTarget;			// Number of shots fired by the player that hit an ennemy
var		int		SP_ShotsHitFriendly;		// Number of shots fired by the player that hit a teammate
var		int		SP_EnemyHeadHits;
var		int		SP_EnemyBodyHits;
var		int		SP_EnemyLegsHits;
var		int		SP_PlayerHeadHits;
var		int		SP_PlayerBodyHits;
var		int		SP_PlayerLegsHits;

// Debug options
var		bool	bShowCheatedStatus;			// Bit 1 in debug number
var		bool	bShowAccuracy;				// Bit 2 in debug number
var		bool	bShowNormalScoreboard;		// bit 4 in debug number

// Game options
var		bool	bDisplayMedalTimes;
var		bool	bDisplayAccuracy;
var		bool	bUseSimpleEnemyAnalysis;

var		int		TimeBronze;
var		int		TimeSilver;
var		int		TimeGold;

var		bool	bUsesSpecialMedalConditions;
var		bool	bCrossDressers;
var		bool	bShowLastDotOnHud;


///////////////////////////////////////
// PreBeginPlay()
///////////////////////////////////////

function PreBeginPlay() {
	local TO_SinglePlayerDefinition S;
	local TO_ScenarioInfo mySI, aSI;
	local TO_SinglePlayerMaplist myList;
	local int i;
	local s_GameReplicationInfo GRI;
	local s_NPCStartPoint NPCStartPoint;
	local s_ZoneControlPoint ZoneControlPoint;

	log("TO_SinglePlayerGame::PreBeginPlay: Single Player game has started.");
	
	//myList = spawn(Class'TOSystem.TO_SinglePlayerMaplist');
	myList = spawn(Class'TO_SinglePlayerMaplist');

	if (myList == none) {
		warn("Could not spawn TO_SinglePlayerMaplist actor!");
		assert(false);
	}

	// Finds the single player map actor
	SinglePlayerDefinition = None;
	foreach allactors(class'TO_SinglePlayerDefinition', S)
			SinglePlayerDefinition = S;

	if (SinglePlayerDefinition == None) {
		//log("to_singleplayergame::prebeginplay -- definition actor not found, using .ini configuration..");
		SinglePlayerDefinition = getSinglePlayerDefinition(myList);
	}

	if (SinglePlayerDefinition.HardCodedBehaviorsClass != "" && caps(SinglePlayerDefinition.HardCodedBehaviorsClass) != "NONE") {
		//log("TO_SinglePlayerGame::PreBeginPlay: Spawned HardCodedBehaviors actor.");
		hardCodedBehaviors = spawn(class<TO_SP_HardCodedBehaviors>(DynamicLoadObject(SinglePlayerDefinition.HardCodedBehaviorsClass, class'Class')));
		if (hardCodedBehaviors == none) {
			warn("TO_SinglePlayerGame::PreBeginPlay: Could not spawn hardCodedBehaviors actor! (HardCodedBehaviorsClass = " $ SinglePlayerDefinition.HardCodedBehaviorsClass $ ")");
			assert(false);
		}
		else {
			hardCodedBehaviors.Initialise(self);
		}
	}

	// Read debug values
	if ((myList.DebugOptions & 1) != 0)
		bShowCheatedStatus = true;
	if ((myList.DebugOptions & 2) != 0)
		bShowAccuracy = true;
	if ((myList.DebugOptions & 4) != 0)
		bShowNormalScoreboard = true;
	

	// Read game options

	bDisplayMedalTimes = myList.bDisplayMedalTimes;
	bDisplayAccuracy = myList.bDisplayAccuracy;
	bUseSimpleEnemyAnalysis = myList.bUseSimpleEnemyAnalysis;

	ProtagonistName = myList.ProtagonistName;

	if (myList.playingMode == 1) {		// Hard
		PlayMode = PM_HARD;
		SinglePlayerDefinition.OpposingBotsHealth *= 1.75;
		SinglePlayerDefinition.VIPHealth *= 1.75;
		SinglePlayerDefinition.SpawnSpecialEnemyBotHealth *= 1.75;
		SinglePlayerDefinition.BotsDifficulty += 2;
		SinglePlayerDefinition.VIPDifficulty += 2;
	}
	else if (myList.playingMode == 2) {	// Ludicrous
		PlayMode = PM_LUDICROUS;
		SinglePlayerDefinition.OpposingBotsHealth *= 2.5;
		SinglePlayerDefinition.VIPHealth *= 2.5;
		SinglePlayerDefinition.SpawnSpecialEnemyBotHealth *= 2.5;
		SinglePlayerDefinition.BotsDifficulty += 5;
		SinglePlayerDefinition.VIPDifficulty += 5;
	}
	else if (myList.playingMode == 3) {	// Easy
		PlayMode = PM_EASY;
		SinglePlayerDefinition.OpposingBotsHealth *= 0.75;
		SinglePlayerDefinition.VIPHealth *= 0.75;
		SinglePlayerDefinition.SpawnSpecialEnemyBotHealth *= 0.75;
		SinglePlayerDefinition.PlayerHealth *= 1.25;
	}
	else								// Normal
		PlayMode = PM_NORMAL;

	myList.GetMedalTimes(TimeBronze, TimeSilver, TimeGold);

	currentBotNumber = 0;
	if (SinglePlayerDefinition.PlayerTeam == 0) {
		remainingSWATBots = SinglePlayerDefinition.OpposingBots;
		remainingTerrBots = SinglePlayerDefinition.TeamBots;
	}
	else {
		remainingTerrBots = SinglePlayerDefinition.OpposingBots;
		remainingSWATBots = SinglePlayerDefinition.TeamBots;
	}
	
	// Get name of pathnodes for additionnal and special bot spawn points
	NumberOfAdditionnalEnemyBot = -1;
	NumberOfSpecialBot = -1;
	for (i = 0; i < 63; i++) {
		if (SinglePlayerDefinition.AdditionnalEnemyBot.names[i] != "None"
		   && SinglePlayerDefinition.AdditionnalEnemyBot.names[i] != "")
			NumberOfAdditionnalEnemyBot++;
		
		if (SinglePlayerDefinition.SpecialEnemyBot.names[i] != "None"
		   && SinglePlayerDefinition.SpecialEnemyBot.names[i] != "")
			NumberOfSpecialBot++;
	}

	if (SinglePlayerDefinition.VIPSpawnSpot != "None" && SinglePlayerDefinition.VIPSpawnSpot != "")
		NumberOfVIP = 1;
	else
		NumberOfVIP = 0;

	if (SinglePlayerDefinition.PlayerTeam == 0)
		remainingSWATBots = remainingSWATBots + NumberOfAdditionnalEnemyBot + NumberOfVIP + 1;
	else
		remainingTerrBots = remainingTerrBots + NumberOfAdditionnalEnemyBot + NumberOfVIP + 1;

	//log("TO_SinglePlayerGame::PreBeginPlay: String(SpawnAdditionnalEnemyBotAt[i]) == " $ String(SpawnAdditionnalEnemyBotAt[0]));
	//log("TO_SinglePlayerGame::PreBeginPlay: Number of Additionnal Enemy Bots: " $ NumberOfAdditionnalEnemyBot);

	//log("Playing mission #" $ myList.MapNum);
	//log("Number of opponent bots:" $ SinglePlayerDefinition.OpposingBots);
	//log("Number of team bots:" $ SinglePlayerDefinition.TeamBots);

	ReadMapData(myList);

	if ( (Level.Game != None) && Level.Game.IsA('TO_SinglePlayerGame') ) {
		TO_SinglePlayerGame(Level.Game).RoundDelay = 10;
		TO_SinglePlayerGame(Level.Game).RoundLimit = 999;
		TO_SinglePlayerGame(Level.Game).RoundDuration = SinglePlayerDefinition.RoundDuration;
		TO_SinglePlayerGame(Level.Game).PreRoundDuration1 = 7;
		TO_SinglePlayerGame(Level.Game).PreRoundDelay = 10;
		TO_SinglePlayerGame(Level.Game).MinAllowedScore = 30000;

		TO_SinglePlayerGame(Level.Game).Default.RoundDelay = 10;
		TO_SinglePlayerGame(Level.Game).Default.RoundLimit = 999;
		TO_SinglePlayerGame(Level.Game).Default.RoundDuration = SinglePlayerDefinition.RoundDuration;
		TO_SinglePlayerGame(Level.Game).Default.PreRoundDuration1 = 7;
		TO_SinglePlayerGame(Level.Game).Default.PreRoundDelay = 10;
		TO_SinglePlayerGame(Level.Game).Default.MinAllowedScore = 30000;
	}
	else
		warn("TO Single Player game: Couln't find TO_SinglePlayerGame -- This should be impossible");

	//log("to_singleplayer::prebeginplay -- player team:");
	//log(Singleplayerdefinition.playerteam);
	//log("to_singleplayer::prebeginplay -- map num:");
	//log(myList.mapnum);
	//log("to_singleplayer::prebeginplay -- map num:");

	mySI = None;
	foreach allactors(class'TO_ScenarioInfo', aSI)
			mySI = aSI;

	if (mySI == None) {
		warn("TO_SinglePlayerGame::PreBeginPlay: Could not find scenario info actor!!");
	}
	else {
		mySI.ScenarioName =				Singleplayerdefinition.ScenarioName;
		mySI.ScenarioDescription1 =		Singleplayerdefinition.ScenarioDescription1;
		mySI.ScenarioDescription2 =		Singleplayerdefinition.ScenarioDescription2;
		mySI.SF_Objective1 =			Singleplayerdefinition.ScenarioText1;
		mySI.SF_Objective2 = 			Singleplayerdefinition.ScenarioText2;
		mySI.SF_Objective3 =			Singleplayerdefinition.ScenarioText3;
		mySI.SF_Objective4 =			Singleplayerdefinition.ScenarioText4;
		mySI.Terr_Objective1 =			Singleplayerdefinition.ScenarioText1;
		mySI.Terr_Objective2 =			Singleplayerdefinition.ScenarioText2;
		mySI.Terr_Objective3 =			Singleplayerdefinition.ScenarioText3;
		mySI.Terr_Objective4 =			Singleplayerdefinition.ScenarioText4;
		
		if (mySI.EventBeginRound == '') {
			mySI.EventBeginRound = 'TO_SP_EventBeginRound';
		}
		tag = mySI.EventBeginRound;
	}

	// If necessary, delete the hostage spawn points
	if (SinglePlayerDefinition.bNoHostages) {
		foreach AllActors( class's_NPCStartPoint', NPCStartPoint )
			NPCStartPoint.bHostage = false;
		foreach AllActors( class's_ZoneControlPoint', ZoneControlPoint )
			ZoneControlPoint.bRescuePoint = false;
	}

	// If necessary, spawn a buy zone for the player
	if (SinglePlayerDefinition.PlayerSpawnSpot != "" &&
	 caps(SinglePlayerDefinition.PlayerSpawnSpot) != "NONE") {
		SpawnBuyZoneForPlayer();
	}

	Super.PreBeginPlay();

	SetGameSpeed(1.0);

	// Check if playing is using cheats
	if (class'TO_StartSinglePlayerCW'.default.MutatorList != "")
		bUsedCheats = true;
	else
		bUsedCheats = false;
}

///////////////////////////////////////
// PostBeginPlay()
///////////////////////////////////////

event PostBeginPlay()
{
	local s_GameReplicationInfo GRI;

	Super.PostBeginPlay();
	RemainingBots = SinglePlayerDefinition.OpposingBots + SinglePlayerDefinition.TeamBots + NumberOfVIP + (NumberOfAdditionnalEnemyBot + 1);

	GRI = s_GameReplicationInfo(GameReplicationInfo);
	if (GRI == None) {
		warn("TO_SinglePlayerGame::PreBeginPlay: Could not find s_GameReplicationInfo!");
		assert(false);
	}
	else {
		GRI.RoundDuration = SinglePlayerDefinition.RoundDuration;

		GRI.bAllowGhostCam = true;
		GRI.bMirrorDamage = false;
		GRI.bEnableBallistics = true;
		GRI.FriendlyFireScale = 100;
		GRI.bPlayersBalanceTeams = false;

		if (GRI.SpecialGameBehaviors == None)
			GRI.SpecialGameBehaviors = spawn(class 'TO_SpecialGameBehaviors');
		GRI.SpecialGameBehaviors.bDisallowTeamSwitchs = true;

		DecodeAndSetWeaponsPermissions(SinglePlayerDefinition.AllowedPlayerWeapons);
	}

}

/*function playerpawn Login
(
	string Portal,
	string Options,
	out string Error,
	class<playerpawn> SpawnClass
)
{
	local PlayerPawn			newPlayer;
	newPlayer = Super.Login(Portal, Options, Error, SpawnClass);
	
	if (newPlayer.IsA('s_Player'))
		s_Hud(s_Player(newPlayer).myHud).bForceScores = true;

	return newPlayer;

}*/





///////////////////////////////////////
// SpawnBot
///////////////////////////////////////

function Bot SpawnBot(out NavigationPoint StartSpot)
{
	local bot NewBot;
	local int BotN;
	local Pawn P;
	local int myDifficulty;
	local int botType; // 0 = normal bot, 1 = additionnal bot, 2 = special bot, 3 = VIP
	local int currentBotIndex;	// Used to read positions within an array
	local bool bTeamBot;

	//log("TO_SinglePlayerGame::SpawnBot: Called spawn bot with remainingSWATBots = " $ remainingSWATBots $ " and remainingTerrBots = " $ remainingTerrBots $ ".");
	
	if (remainingSWATBots == 0 && remainingTerrBots == 0 && GamePeriod != GP_RoundPlaying && 
	 SinglePlayerDefinition.bSpawnSpecialEnemy != 4)  // If round has already started or if bspawnspecialenemy = 4, allow spawning of special bots
		return None;

	if ( bRatedGame )
		return SpawnRatedBot(StartSpot);

	if (currentBotNumber <= NumberOfAdditionnalEnemyBot)
		botType = 1;
	else
		botType = 0;

	if (currentBotNumber == NumberOfAdditionnalEnemyBot + 1 && numberOfVIP == 1)
		botType = 3;

	if (bSpecialBotsSpawned)
		botType = 2;
	
	// Figure which team the bot will be on and give proper intelligence
	if (SinglePlayerDefinition.PlayerTeam == 1) { // Player is on SF
		if (remainingTerrBots != 0) { myDifficulty = SinglePlayerDefinition.BotsDifficulty;	bTeamBot = false;}
		else { myDifficulty = SinglePlayerDefinition.TeamBotsIntelligence; 	bTeamBot = true;}
	}
	else {	// Player is on terr
		if (remainingSWATBots != 0) { myDifficulty = SinglePlayerDefinition.BotsDifficulty; 	bTeamBot = false;}
		else { myDifficulty = SinglePlayerDefinition.TeamBotsIntelligence; 	bTeamBot = true;}
	}

	//myDifficulty = SinglePlayerDefinition.BotsDifficulty;
	if (botType == 3) // VIP
		myDifficulty = SinglePlayerDefinition.VIPDifficulty;
	//else if (botType == 2) // Special Bots
	//	myDifficulty = SinglePlayerDefinition.

	if ( myDifficulty >= 4 )
	{
		bNoviceMode = false;
		myDifficulty = myDifficulty - 4;
	}
	else
	{
		if ( myDifficulty > 3 )
		{
			myDifficulty = 3;
			bThreePlus = true;
		}
		bNoviceMode = true;
	}
	BotN = BotConfig.ChooseBotInfo();
	
	currentBotNumber++;

	// Find a start spot.
	if (botType == 1) { // additionnal bot
		StartSpot = FindNavigationPoint(SinglePlayerDefinition.AdditionnalEnemyBot.names[currentBotNumber-1]);
		//log("TO_SinglePlayerGame::SpawnBot: Spawning bot at " $ SpawnAdditionnalEnemyBotAt[currentBotNumber]);
	}
	else if (botType == 2) { // special bot
		StartSpot = FindNavigationPoint(SinglePlayerDefinition.SpecialEnemyBot.names[currentSpecialBotNumber]);
		//log("TO_SinglePlayerGame::SpawnBot: Spawning special bot at " $ SinglePlayerDefinition.SpecialEnemyBot.names[currentSpecialBotNumber] $ ", # = " $ currentSpecialBotNumber);
	}
	else if (botType == 3) { // VIP
		StartSpot = FindNavigationPoint(SinglePlayerDefinition.VIPSpawnSpot);
		//log("TO_SinglePlayerGame::SpawnBot: Spawning VIP at " $ SinglePlayerDefinition.VIPSpawnSpot);
	}
	else if (bTeamBot) {
		currentBotIndex = currentBotNumber - (NumberOfAdditionnalEnemyBot+1) - NumberOfVIP - SinglePlayerDefinition.OpposingBots - 1;
		//warn("botcheck: checking bot # " $ currentBotIndex $ ", pathname = " $ SinglePlayerDefinition.TeamBotsNames.names[currentBotIndex]);
		if (SinglePlayerDefinition.TeamBotsNames.names[currentBotIndex] == "None" ||
		 SinglePlayerDefinition.TeamBotsNames.names[currentBotIndex] == "")
			StartSpot = FindPlayerStart(None, 255);
		else {
			StartSpot = FindNavigationPoint(SinglePlayerDefinition.TeamBotsNames.names[currentBotIndex]);
		}
	}
	else
		StartSpot = FindPlayerStart(None, 255);

	if( StartSpot == None )
	{
		warn("TO_SinglePlayerGame::SpawnBot: Could not find starting spot for Bot");
		return None;
	}

	// Try to spawn the bot.
	NewBot = Spawn(BotConfig.CHGetBotClass(BotN),,,StartSpot.Location,StartSpot.Rotation);

	if ( NewBot == None ) {
		warn("TO_SinglePlayerGame::SpawnBot: Couldn't spawn player at "$StartSpot);
		//assert(false);
	}

	if ( (bHumansOnly || Level.bHumansOnly) && !NewBot.bIsHuman )
	{
		warn("TO_SinglePlayerGame::SpawnBot: Can't add non-human bot to this game");
		NewBot.Destroy();
		NewBot = None;
	}

	if ( NewBot == None )
		NewBot = Spawn(BotConfig.CHGetBotClass(0),,,StartSpot.Location,StartSpot.Rotation);

	if ( NewBot != None )
	{
		// Set the player's ID.
		NewBot.PlayerReplicationInfo.PlayerID = CurrentID++;
		s_botBase(NewBot).TO_SinglePlayerBotType = botType;
		//log("TO_SinglePlayerGame::SpawnBot: Set bot type to " $ s_botBase(NewBot).TO_SinglePlayerBotType);
		
		if (SinglePlayerDefinition.PlayerTeam == 1) { // Player is on SF
			if (remainingTerrBots != 0) {
				NewBot.PlayerReplicationInfo.Team = 0;
				remainingTerrBots--;
			}
			else {
				NewBot.PlayerReplicationInfo.Team = 1;
				remainingSWATBots--;
			}

		}
		else {	// Player is on terr
			if (remainingSWATBots != 0) {
				NewBot.PlayerReplicationInfo.Team = 1;
				remainingSWATBots--;
			}
			else {
				NewBot.PlayerReplicationInfo.Team = 0;
				remainingTerrBots--;
			}
		}
		//NewBot.PlayerReplicationInfo.Team = BotConfig.GetBotTeam(BotN);
		
		BotConfig.CHIndividualize(NewBot, BotN, NumBots);

		if (botType == 1) {			// Additionnal bot
			//log("TO_SinglePlayergame::SpawnBot: Trying to change the name of a bot to '" $ SinglePlayerDefinition.SpawnAdditionnalEnemyBotName[currentBotNumber-1] $ "' (currentBotNumber = " $ (currentBotNumber - 1) $ ")");
			// Change name if a different name is listed in the config file
			if (SinglePlayerDefinition.AdditionnalEnemyBot.strings[currentBotNumber-1] != "")
				Level.Game.ChangeName( NewBot, SinglePlayerDefinition.AdditionnalEnemyBot.strings[currentBotNumber-1], false);
			s_Bot(NewBot).TO_SinglePlayerBotAttitude = SinglePlayerDefinition.AdditionnalEnemyBotAttitude;
			NewBot.MenuName = "TO_SP_Bot1-" $ string(currentBotNumber-1);
		}
		else if (botType == 2) {	// Special bot
			//log("TO_SinglePlayergame::SpawnBot: Trying to change the name of a bot to '" $ SinglePlayerDefinition.SpawnAdditionnalEnemyBotName[currentBotNumber-1] $ "' (currentBotNumber = " $ (currentBotNumber - 1) $ ")");
			// Change name if a different name is listed in the config file
			if (SinglePlayerDefinition.SpecialEnemyBot.strings[currentSpecialBotNumber] != "")
				Level.Game.ChangeName( NewBot, SinglePlayerDefinition.SpecialEnemyBot.strings[currentSpecialBotNumber], false);
			s_Bot(NewBot).TO_SinglePlayerBotAttitude = SinglePlayerDefinition.SpawnSpecialEnemyBotAttitude;
			NewBot.MenuName = "TO_SP_Bot2-" $ string(currentSpecialBotNumber);
		}
		else if (botType == 3) {	// VIP
			if (SinglePlayerDefinition.VIPName != "")
				Level.Game.ChangeName( NewBot, SinglePlayerDefinition.VIPName, false);
			s_Bot(NewBot).TO_SinglePlayerBotAttitude = SinglePlayerDefinition.VIPAttitude;
			NewBot.MenuName = "TO_SP_Bot3";
		}
		else if (botType == 0 && NewBot.PlayerReplicationInfo.Team == SinglePlayerDefinition.PlayerTeam) {	// Normal team bot
			s_Bot(NewBot).TO_SinglePlayerBotAttitude = 0;
			currentBotIndex = currentBotNumber - (NumberOfAdditionnalEnemyBot+1) - NumberOfVIP - SinglePlayerDefinition.OpposingBots - 1;
			//log("TO_SinglePlayerGame::SpawnBot: Spawning normal team bot with currentBotIndex = " $ currentBotIndex);
			if (SinglePlayerDefinition.TeamBotsNames.strings[currentBotIndex] != "")
				Level.Game.ChangeName( NewBot, SinglePlayerDefinition.TeamBotsNames.strings[currentBotIndex], false);
			NewBot.MenuName = "TO_SP_Bot4-" $ string(currentBotIndex);
		}
		else if (botType == 0 && NewBot.PlayerReplicationInfo.Team != SinglePlayerDefinition.PlayerTeam) {	// Normal opponent bot
			s_Bot(NewBot).TO_SinglePlayerBotAttitude = 0;
			currentBotIndex = currentBotNumber - (NumberOfAdditionnalEnemyBot+1) - NumberOfVIP - 1;
			//log("TO_SinglePlayerGame::SpawnBot: Spawning normal opposing bot with currentBotIndex = " $ currentBotIndex $ " Name = '" $ SinglePlayerDefinition.OpposingBotsNames.strings[currentBotIndex] $ "'");
			
			if (SinglePlayerDefinition.OpposingBotsNames.strings[currentBotIndex] != "") {
				//log("TO_SP::SpawnBot: Trying to change the name to " $ SinglePlayerDefinition.OpposingBotsNames.strings[currentBotIndex] $ " for bot # " $ currentBotIndex);
				Level.Game.ChangeName( NewBot, SinglePlayerDefinition.OpposingBotsNames.strings[currentBotIndex], false);
			}
			NewBot.MenuName = "TO_SP_Bot5-" $ string(currentBotIndex);
		}

		NewBot.InitializeSkill(myDifficulty);

		NewBot.ViewRotation = StartSpot.Rotation;
		// broadcast a welcome message.
		//BroadcastMessage( NewBot.PlayerReplicationInfo.PlayerName$EnteredMessage, false );

		ModifyBehaviour(NewBot);
		AddDefaultInventory( NewBot );
		NumBots++;
		if ( bRequireReady && (CountDown > 0) )
			NewBot.GotoState('Dying', 'WaitingForStart');
		NewBot.AirControl = AirControl;

		if ( (Level.NetMode != NM_Standalone) && (bNetReady || bRequireReady) )
		{
			// replicate skins
			for ( P=Level.PawnList; P!=None; P=P.NextPawn )
				if ( P.bIsPlayer && (P.PlayerReplicationInfo != None) && P.PlayerReplicationInfo.bWaitingPlayer && P.IsA('PlayerPawn') )
				{
					if ( NewBot.bIsMultiSkinned )
						PlayerPawn(P).ClientReplicateSkins(NewBot.MultiSkins[0], NewBot.MultiSkins[1], NewBot.MultiSkins[2], NewBot.MultiSkins[3]);
					else
						PlayerPawn(P).ClientReplicateSkins(NewBot.Skin);	
				}						
		}
	}

	// Fugly hack to spawn special bots when SinglePlayerDefinition.bSpawnSpecialEnemy == 4
	if (currentBotNumber == (NumberOfAdditionnalEnemyBot + 1) + numberOfVIP + SinglePlayerDefinition.OpposingBots +
	 SinglePlayerDefinition.TeamBots) {
		//warn("Fugly hack hit! currentBotNumber = " $ currentBotNumber);
		if (SinglePlayerDefinition.bSpawnSpecialEnemy == 4)
			SpawnSpecialBots();
	}

	SetBotObjectives(s_bot(NewBot));

	return NewBot;
}



///////////////////////////////////////
// RestartRound
// EXTENDED
///////////////////////////////////////

function RestartRound()
{
	local Pawn						P;
	local TO_SinglePlayerTrigger	T;
	local TO_SP_ExitRing			ExitRing;
	local Decal						Decal;

	log("TO_SinglePlayerGame::RestartRound: New round has started.");

	currentAdditionnalBotNumber = -1;
	currentSpecialBotNumber = -1;
	currentTeamBotNumber = -1;

	for ( P=Level.PawnList; P!=None; P=P.nextPawn )
	{
		if ( P.PlayerReplicationInfo != None )
		{
			P.PlayerReplicationInfo.Score = 0;
			P.PlayerReplicationInfo.Deaths = 0;
		}

		KillInventory(P);

		if ( P.IsA('s_Player') )
		{
			s_Player(P).Money = 0;
			s_Player(P).HelmetCharge = 0;
			s_Player(P).VestCharge = 0;
			s_Player(P).LegsCharge = 0;
		}
		else if ( P.IsA('s_Bot') )
		{
			s_Bot(P).Money = 0;
			s_Bot(P).HelmetCharge = 0;
			s_Bot(P).VestCharge = 0;
			s_Bot(P).LegsCharge = 0;
		}
	}

	if (GameReplicationInfo != none) {
		TournamentGameReplicationInfo(GameReplicationInfo).Teams[0].Score = 0;
		TournamentGameReplicationInfo(GameReplicationInfo).Teams[1].Score = 0;
	}
	RoundNumber = 0;

	TerrAmount=0;
	CTAmount=0;

	//log("Remaining Time: " $ RemainingTime $ " -- Timelimit: " $ TimeLimit);
	RemainingTime = TimeLimit*60;
	
	currentBotNumber = 0;

	// Delete spawned triggers
	foreach AllActors (class'TO_SinglePlayerTrigger', T)
		T.destroy();

	// Delete exit rings
	foreach AllActors (class'TO_SP_ExitRing', ExitRing)
		ExitRing.destroy();

	// Delete decals
	foreach AllActors (class'Decal', Decal)
		Decal.destroy();

	// Allow hard coded behaviors actor to restart stuff
	if (hardCodedBehaviors != none)
		hardCodedBehaviors.PreRestartRound();

	for ( P=Level.PawnList; P!=None; P=P.nextPawn )
		if (P.IsA('s_Bot') && !P.IsA('s_NPC')) {
			// Used to stop spawnkills
			s_bot(P).SetCollision(false, false, false);
		}


	// Restart the round using the normal TO code
	super.RestartRound();

	// Set proper skins and other properties on each player & bot
	//foreach AllActors (class'Pawn', P)
	for ( P=Level.PawnList; P!=None; P=P.nextPawn )
		if (P.IsA('bot') || P.IsA('PlayerPawn')) {
			//log("TO_SinglePlayerGame::RestartRound: Dressing pawn " $ P.name);
			
			// RESET PLAYER
			if (P.IsA('PlayerPawn')) {
				if (SinglePlayerDefinition.SelfSkin != -1)
					ChangePModelPlus(P, SinglePlayerDefinition.SelfSkin, P.PlayerReplicationInfo.Team, false);

				// Show the breifing screen each round
				P.health = SinglePlayerDefinition.PlayerHealth;
				if (s_Player(P) != none)
					if (s_player(p).bGUIActive == true)
						Level.Game.ChangeName(P, ProtagonistName, false);

				if (Protagonist == None)		// Need to keep a link to the player
					Protagonist = s_Player(P);

				s_Player(P).bCheatsEnabled = false;

				if (!bFirstPreRound && !bCutScene)
					s_Hud(s_Player(P).myHud).UserInterface.SelectTab(s_Hud(s_Player(P).myHud).UserInterface.UIT_BRIEFING);

				TO_SP_Hud(s_PlayeR(P).myHud).ResetHUDStuff();
			}
			// RESET HOSTAGES
			else if (P.IsA('s_NPC')) {
				// nothing to reset for hostages
			}
			// RESET BOTS
			else {
				AddDefaultInventory(P);
				s_bot(P).SetCollision(true, true, true);
				SetBotObjectives(s_bot(P));
				if (s_BotBase(P).TO_SinglePlayerBotType == 3) { // VIP
					if (SinglePlayerDefinition.VIPSkin != -1)
						ChangePModelPlus(P, SinglePlayerDefinition.VIPSkin, P.PlayerReplicationInfo.Team, false);
					P.health = SinglePlayerDefinition.VIPHealth;
					DecodeAndGiveWeapons(P, SinglePlayerDefinition.VIPGivenWeapons);
				}
				else if (s_BotBase(P).TO_SinglePlayerBotType == 2) { // Special bot
					if (SinglePlayerDefinition.bSpawnSpecialEnemy == 4) {
						if (SinglePlayerDefinition.SpawnSpecialEnemyBotSkin != -1)
							ChangePModelPlus(P, SinglePlayerDefinition.SpawnSpecialEnemyBotSkin, P.PlayerReplicationInfo.Team, false);
						P.health = SinglePlayerDefinition.SpawnSpecialEnemyBotHealth;
						DecodeAndGiveWeapons(P, SinglePlayerDefinition.SpawnSpecialEnemyBotGivenWeapons);
					}
					else
						P.destroy();
					//ChangePModelPlus(P, SinglePlayerDefinition.SpawnSpecialEnemyBotSkin, P.PlayerReplicationInfo.Team, false);
				}
				else if (s_BotBase(P).TO_SinglePlayerBotType == 1) { // Additionnal bot
					if (SinglePlayerDefinition.OpponentSkins != -1)
						ChangePModelPlus(P, SinglePlayerDefinition.OpponentSkins, P.PlayerReplicationInfo.Team, false);
					P.health = SinglePlayerDefinition.OpposingBotsHealth;
					DecodeAndGiveWeapons(P, SinglePlayerDefinition.OpposingBotsGivenWeapons);
				}
				else { // Bot is a normal bot
					if (P.PlayerReplicationInfo.Team == SinglePlayerDefinition.PlayerTeam) { // Bot is on the player's team
						if (SinglePlayerDefinition.TeamSkins != -1)
							ChangePModelPlus(P, SinglePlayerDefinition.TeamSkins, P.PlayerReplicationInfo.Team, false);
						P.health = SinglePlayerDefinition.TeamBotsHealth;
						DecodeAndGiveWeapons(P, SinglePlayerDefinition.TeamBotsGivenWeapons);
					}
					else {	// Bot isn't on the player's team
						if (SinglePlayerDefinition.OpponentSkins != -1)
							ChangePModelPlus(P, SinglePlayerDefinition.OpponentSkins, P.PlayerReplicationInfo.Team, false);
						P.health = SinglePlayerDefinition.OpposingBotsHealth;
						DecodeAndGiveWeapons(P, SinglePlayerDefinition.OpposingBotsGivenWeapons);
					}
				}
			}
		}

	if (ScoreBoardLink != None)
		ScoreBoardLink.ResetScoreBoard();

	if (BuyMenuLink != None)
		BuyMenuLink.ResetBuyMenu();

	// Reset variables
	bVIPDied = false;
	bC4ExplodedPlus = false;
	bEndGameTriggersSpawned = false;
	if (SinglePlayerDefinition.bSpawnSpecialEnemy != 4)
		bSpecialBotsSpawned = false;
	bSpecialTriggersSpawned = false;
	//bAlreadyWon = false; // This is useless, as once the round is won the map changes
	bAlreadyBroadcastedAllEnnemiesDeadMSG = false;
	bAlreadyBroadcastedVIPDeadMSG = false;
	bAlreadyBroadcastedSpecialTriggersMSG = false;
	bAlreadyBroadcastedC4ExplodedMSG = false;
	bC4Defused = false;
	bSpawnedSpecialBotsBeforeEndGameTriggers = false;
	anExitObject = none;

	SP_EnemyCasualties = 0;
	SP_NumEnemies = SinglePlayerDefinition.OpposingBots + (NumberOfAdditionnalEnemyBot+1) + NumberOfVIP;
	SP_FriendlyCasualties = 0;
	SP_NumTeammates = SinglePlayerDefinition.TeamBots;
	SP_FriendflyFireIncidents = 0;
	SP_ShotsFired = 0;
	SP_ShotsHitTarget = 0;
	SP_ShotsHitFriendly = 0;
	SP_EnemyHeadHits = 0;
	SP_EnemyBodyHits = 0;
	SP_EnemyLegsHits = 0;
	SP_PlayerHeadHits = 0;
	SP_PlayerBodyHits = 0;
	SP_PlayerLegsHits = 0;


	// If end game triggers should be spawned at beginning of round, do so.
	if (SinglePlayerDefinition.bSpawnEndGameTriggers == 4) {
		SpawnEndGameTriggers();
	}

	// If special triggers should be spawned at beginning of round, do so.
	if (SinglePlayerDefinition.bSpawnSpecialTriggers == 4) {
		SpawnSpecialTriggers();
	}

	// Allow hard coded behaviors actor to restart stuff aften the game itself has restarted
	if (hardCodedBehaviors != none)
		hardCodedBehaviors.PostRestartRound();

	// Handle mutator restart round
	BaseMutator.HandleRestartGame();
}

function CheckEndGame() {
	local Pawn PawnLink;
	local int StillPlaying;
	local int t_players, t_players_t, Players;
	local int ct_players, ct_players_t;

	if ( bGameEnded || (GamePeriod != GP_RoundPlaying) )
		return;

	// Check to see if everyone is dead
	t_players = 0;
	ct_players = 0;
	for ( PawnLink=Level.PawnList; PawnLink!=None; PawnLink=PawnLink.nextPawn )	{
		if ( (PawnLink.PlayerReplicationInfo != None) && !PawnLink.PlayerReplicationInfo.bWaitingPlayer ) {
			if ( ((s_Player(PawnLink) != None) && (!s_Player(PawnLink).bNotPlaying)) 
				|| ((s_Bot(PawnLink) != None) && (!s_Bot(PawnLink).bNotPlaying))) 
			{
				StillPlaying++;
				if ( s_Player(PawnLink) != None )
					Players++;
				if ( PawnLink.PlayerReplicationInfo.Team  == 0 )
					t_players++;
				else if ( PawnLink.PlayerReplicationInfo.Team  == 1 )
					ct_players++;
			}
		}
	}

	if ( bC4Explodes && !(SinglePlayerDefinition.bSpawnEndGameTriggers > 0))
		return;

	//log ("TO_GameBasics::CheckEndGame: Making single player CheckEndGame checks... t_players == " $ t_players $ ", ct_players == " $ ct_players);
	if ( Players == 0 ) {
		if ( SinglePlayerDefinition.PlayerTeam == 0 )
			SetWinner(1);
		else
			SetWinner(0);
		BroadcastLocalizedMessage(class'TO_SP_Messages', 3);
		EndGame(class'TO_SP_Messages'.default.WinMessage[3]);
	}
	else if ( (SinglePlayerDefinition.PlayerTeam == 0 && t_players < SinglePlayerDefinition.NeededTeamBotsSurvivors + 1) ||
	 (SinglePlayerDefinition.PlayerTeam == 1 && ct_players < SinglePlayerDefinition.NeededTeamBotsSurvivors + 1) ) {
		if ( SinglePlayerDefinition.PlayerTeam == 0 )
			SetWinner(1);
		else
			SetWinner(0);
		BroadcastLocalizedMessage(class'TO_SP_Messages', 4);
		EndGame(class'TO_SP_Messages'.default.WinMessage[4]);
	}

	else if (t_players == 0 || ct_players == 0) { // No opponents left; should special stuff be spawned?
		//log ("TO_GameBasics::CheckEndGame: One of the teams is empty.");
		if (SinglePlayerDefinition.bSpawnSpecialEnemy == 1)
			SpawnSpecialBots();

		if (SinglePlayerDefinition.bSpawnEndGameTriggers == 1)
			SpawnEndGameTriggers();

		if (SinglePlayerDefinition.bSpawnSpecialTriggers == 1)
			SpawnSpecialTriggers();

		if (SinglePlayerDefinition.AllEnnemiesDeadMSG != "")
			BroadcastAllEnnemiesDeadMSG();
	}

	// If an end round trigger is used, do no end round no matter what
	if (SinglePlayerDefinition.bSpawnEndGameTriggers != 0 && SinglePlayerDefinition.bSpawnEndGameTriggers != -1)
		return;

	// Allow hard coded behaviors actor to block the rest of the end game check
	if (hardCodedBehaviors != none)
		if (hardCodedBehaviors.BlockCheckEndRound() == true)
			return;

	// Continue end round check if set to bexterminate
	if (!(SinglePlayerDefinition.bExterminate))
		return;
	
	super.CheckEndGame();
}

function C4ExplodedPlus(bool bExplodedInBombingZone, Actor BombingZone) {
	if (bC4ExplodedPlus && !bUseMultipleC4)
		return;				// Only run this function once !!

	bC4ExplodedPlus = true;

	if (SinglePlayerDefinition.bSpawnSpecialEnemy == 5)
		SpawnSpecialBots();

	if (SinglePlayerDefinition.bSpawnEndGameTriggers == 5)
		SpawnEndGameTriggers();

	if (SinglePlayerDefinition.bSpawnSpecialTriggers == 5)
		SpawnSpecialTriggers();

	if (SinglePlayerDefinition.C4ExplodedMSG != "")
		BroadcastC4ExplodedMSG();

	if (hardCodedBehaviors != None)
		hardCodedBehaviors.C4ExplodedPlus(bExplodedInBombingZone, BombingZone);

	//log("TO_SP::C4ExplodedPlus: bombing zone actor = " $ string(BombingZone.name) );

	if (TO_SP_BombingPoint(BombingZone) != None)
		TO_SP_BombingPoint(BombingZone).BombDetonated();
}

function C4Planted(vector PlantLocation, s_ExplosiveC4 PlantedC4) {
	if (hardCodedBehaviors != None)
		hardCodedBehaviors.C4Planted(PlantLocation, PlantedC4);

	if (SinglePlayerDefinition.bSpawnSpecialEnemy == 6)
		SpawnSpecialBots();

	if (SinglePlayerDefinition.bSpawnEndGameTriggers == 6)
		SpawnEndGameTriggers();

	if (SinglePlayerDefinition.bSpawnSpecialTriggers == 6)
		SpawnSpecialTriggers();
}

function bool RestartPlayer( pawn aPlayer )	
{
	local bool result;
	local NavigationPoint nav;

	//log("TO_SP::RestartPlayer: Restarting player " $ String(aPlayer));

	result = super.RestartPlayer(aPlayer);

	if (aPlayer.IsA('s_BotBase')) {
		nav = ResetPawnPosition(s_BotBase(aPlayer));
		if (nav != None)
			if (!aPlayer.SetLocation(nav.Location))
				warn("TO_SinglePlayerGame::RestartPlayerPlus: Could not move the bot!! Make sure the pathnode is accessible. (Pathnode = " $ nav.name $ ")");
	}
	else if (aPlayer.IsA('s_Player')) {
		if (SinglePlayerDefinition.PlayerSpawnSpot != "" &&
			caps(SinglePlayerDefinition.PlayerSpawnSpot) != "NONE") {
			nav = FindNavigationPoint(SinglePlayerDefinition.PlayerSpawnSpot);
			if (nav != None) {
				if (!aPlayer.SetLocation(nav.Location))
					warn("TO_SinglePlayerGame::RestartPlayerPlus: Could not move the player!! Make sure the pathnode is accessible. (Pathnode = " $ nav.name $ ")");

				aPlayer.ClientSetLocation(nav.Location, nav.Rotation );
				aPlayer.ClientSetRotation(nav.Rotation);
				aPlayer.SetRotation(nav.Rotation);
				aPlayer.ViewRotation = aPlayer.Rotation;
			}
		}
	}

	return result;
}

function EndPreRoundPlus() {
	local pawn P;
	local mutator M;

	//super.EndPreRound();

	for ( P=Level.PawnList; P!=None; P=P.nextPawn ) {
		if (P.IsA('s_Player')) {
			P.health = SinglePlayerDefinition.PlayerHealth;
		}
		/*else if (P.IsA('s_Bot')) {
			//s_bot(P).TO_SinglePlayerBotAttitude = 9;
			SetBotObjectives(s_Bot(P));
			//log("Bot extra info: " $ P.MenuName);
		}*/
	}

	for (M = BaseMutator.NextMutator; M != None; M = M.NextMutator)
		if (M.IsA('TO_SinglePlayerMutator'))
			TO_SinglePlayerMutator(M).EndPreRound();


}

function SetPlayerStartPoint( pawn aPlayer )
{
	local NavigationPoint startSpot;
	local bool						foundStart;
	local	int	i;

	if (aPlayer.IsA('s_Player')) {
		if (SinglePlayerDefinition.PlayerSpawnSpot != "" &&
		 caps(SinglePlayerDefinition.PlayerSpawnSpot) != "NONE") {
			startSpot = FindNavigationPoint(SinglePlayerDefinition.PlayerSpawnSpot);
			if (startSpot == None) {
				warn("Could not find player start '" $ SinglePlayerDefinition.PlayerSpawnSpot $ "'!");
				assert(false);
			}
			foundStart = aPlayer.SetLocation(startSpot.Location);
			
			if ( !foundStart ) {
				warn(startspot $ " Player start not useable!!!");
				assert(false);
			}
			
			aPlayer.ClientSetLocation(startSpot.Location, startSpot.Rotation );
			aPlayer.ClientSetRotation(startSpot.Rotation);

			aPlayer.SetRotation(startSpot.Rotation);
			aPlayer.ViewRotation = aPlayer.Rotation;
		}
		else 
			Super.SetPlayerStartPoint(aPlayer);
	}
	else
		Super.SetPlayerStartPoint(aPlayer);

}

function Timer() {
	Super.Timer();

	// Prevent counter from decreasing when it's the first preround (for the story screen)
	if (GamePeriod == GP_PreRound && bFirstPreRound) {
		PreRoundDelay = PreRoundDuration1;
	}

	if (Protagonist != None) {
		if (Protagonist.bCheatsEnabled)
			bUsedCheats = true;
	}

	if (Level.bPlayersOnly)
		bUsedCheats = true;

	if (nbRescuedHostages != 0 && bNoHostageRescueAllowed) {
		nbRescuedHostages = 0;

		if ( SinglePlayerDefinition.PlayerTeam == 0 )
			SetWinner(1);
		else
			SetWinner(0);
		BroadcastLocalizedMessage(class'TO_SP_Messages', 5);
		EndGame(class'TO_SP_Messages'.default.WinMessage[5]);
	}

	if (hardCodedBehaviors != None)
		hardCodedBehaviors.Timer();

}

function SetGameSpeed( Float T ) {
	if (!(T ~= 1.0))
		bUsedCheats = true;

	GameSpeed = FMax(T, 0.1);
	Level.TimeDilation = 1.1 * GameSpeed;
	SetTimer(Level.TimeDilation, true);

	//Super.SetGameSpeed(T);
}

function EndFirstPreRound() {
	bFirstPreRound = false;
	PreRoundDelay = PreRoundDuration1;
}

function Trigger( actor Other, pawn EventInstigator ) {
	EndPreRoundPlus();
}

function int GetSPPlayerTeam() {
	return SinglePlayerDefinition.PlayerTeam;
}

function int GetbSpawnEndGameTriggers() {
	return SinglePlayerDefinition.bSpawnEndGameTriggers;
}

function SetWinnerPlus(int WinTeam) {
	local s_GameReplicationInfo GRI;
	local pawn Pn;
	local int numBots, requiredBots;

	numBots = 0;

	// Anti-cheating: check if the player used killall bots
	for (Pn=Level.PawnList; Pn!=None; Pn=Pn.nextPawn) {
		if (Pn.IsA('s_Bot') && !(Pn.IsA('s_NPC')))
			numBots++;
	}

	requiredBots = SinglePlayerDefinition.OpposingBots + SinglePlayerDefinition.TeamBots +
		(NumberOfAdditionnalEnemyBot + 1) + NumberOfVIP;

	warn("numbot = " $ numbots $ ", requiredBots = " $ requiredbots $ ", oppos = " $ SinglePlayerDefinition.OpposingBots $ ", team=" $ SinglePlayerDefinition.TeamBots $ ", additionnal=" $ (NumberOfAdditionnalEnemyBot + 1) $ ", vip=" $ NumberOfVIP);

	if (numBots < requiredBots && SinglePlayerDefinition.PlayerTeam == WinTeam)
		bUsedCheats = true;


	GRI = s_GameReplicationInfo(GameReplicationInfo);
	if ( GRI != None ) {
		for (Pn=Level.PawnList; Pn!=None; Pn=Pn.nextPawn) {
			if (s_Player(Pn) != none && SinglePlayerDefinition.PlayerTeam == WinTeam) {
				SP_Time = GRI.RoundStarted - GRI.Remainingtime;
				SaveMapSettings();
				s_Hud(s_Player(Pn).myHud).bForceScores = true;
				//s_Hud(s_Player(Pn).myHud).bFirstPaint = true;
				s_Player(Pn).bFire = 0;		// If player was shooting, make him stop
				s_Player(Pn).bAltFire = 0;
				s_Player(Pn).ClientSetMusic(Level.Song, 255, 255, MTRAN_FastFade);
				s_Player(Pn).PlaySound(Sound'TODatas.missioncompleted', SLOT_None, 4, false);
				bGameWon = true;
			}
		}
	}
}

// Called when RoundDelay < 1
function EndGamePlus(string Reason) {
	local pawn		PawnLink;
	//log("to_gamebasis::endgame -- Round ended! Switching levels..");
	//bChangeLevels = true;
	
	GamePeriod = GP_PostMatch;
	postGameTime = 1000000;

	for ( PawnLink=Level.PawnList; PawnLink!=None; PawnLink=PawnLink.nextPawn )	{
		if ( (PawnLink.PlayerReplicationInfo != None) && !PawnLink.PlayerReplicationInfo.bWaitingPlayer ) {
			if ( ((s_Bot(PawnLink) != None) && (!s_Bot(PawnLink).bNotPlaying)))	{
				s_Bot(PawnLink).GotoState('PreRound');
			}
		}
	}

}

function ProcessServerTravel( string URL, bool bItems ) {
	TO_CleanUp();
	Super(GameInfo).ProcessServerTravel(URL, bItems);
}



function SaveMapSettings() {
	local TO_SinglePlayerMapList myList;
	local pawn player;
	local bool bRetail;

	if (bUsedCheats)
		return;

	player = GetPlayer();
	bRetail = TO_RootWindow(s_Hud(s_player(player).myHud).Root).bTORetail;

	myList = spawn(Class'TO_SinglePlayerMaplist');
	myList.saveBestTime(SP_Time);

	if ( myList.MapNum == myList.GetLastCompletedMap() && !myList.AllMapsCompleted() && !myList.IsLastModMap(bRetail)) 
	{
		myList.IncrementLastCompletedMap();
		myList.SaveConfig();
		//bShowCumulatives = myList.ShouldDisplayCumulatives();
	}

	if ( PlayMode == PM_HARD && !myList.IsCompletedInHardMode(myList.MapNum) ) {
		myList.SetCurrentAsCompletedInHardMode();
	}
	if ( PlayMode == PM_LUDICROUS && !myList.IsCompletedInLudicrousMode(myList.MapNum) ) {
		myList.SetCurrentAsCompletedInLudicrousMode();
	}

	ReadMapData(myList);
}


function HostageKilled(pawn killer, pawn Other, name damageType) {
	if (hardCodedBehaviors != none)
		if (hardCodedBehaviors.bHandlesHostageKills) {
			hardCodedBehaviors.HostageKilled(killer, Other, damageType);
			return;
		}
	
	if (SinglePlayerDefinition.PlayerTeam == 1) {
		SetWinner(0);
		BroadcastLocalizedMessage(class'TO_SP_Messages', 2);
		EndGame(class'TO_SP_Messages'.default.WinMessage[2]);
	}
	else if (SinglePlayerDefinition.PlayerTeam == 0 && bNoHostageDeathsAllowed) {
		SetWinner(1);
		BroadcastLocalizedMessage(class'TO_SP_Messages', 2);
		EndGame(class'TO_SP_Messages'.default.WinMessage[2]);
	}
}

function Killed(pawn killer, pawn Other, name damageType) {
	super.Killed(killer, Other, damageType);

	// Check if the VIP was killed
	if (s_BotBase(Other) != None)
		if (s_BotBase(Other).TO_SinglePlayerBotType == 3) // VIP
			VIPDied();

	if (s_Bot(Other) != None) {
		if (s_Bot(Other).PlayerReplicationInfo.Team == SinglePlayerDefinition.PlayerTeam) {			// Is a teammate
			SP_FriendlyCasualties++;
			AddFriendlyCasualty(s_Bot(Other));
		}
		else if (s_Bot(Other).PlayerReplicationInfo.Team != SinglePlayerDefinition.PlayerTeam) {	// Is an enemy
			SP_EnemyCasualties++;
			AddEnemyCasualty(s_Bot(Other));
		}
	}

	BotCheckOrderObject(Other);
	CheckEndGame();

}

function AddFriendlyCasualty(s_bot Who) {
	if (ScoreBoardLink != none) {
		ScoreBoardLink.LastTeamCasualty++;
		ScoreBoardLink.TeamCasualties[ScoreBoardLink.LastTeamCasualty] = Who.PlayerReplicationInfo.PlayerName;
	}
}

function AddEnemyCasualty(s_bot Who) {
	if (ScoreBoardLink != none) {
		ScoreBoardLink.LastEnemyCasualty++;
		ScoreBoardLink.EnemyCasualties[ScoreBoardLink.LastEnemyCasualty] = Who.PlayerReplicationInfo.PlayerName;
	}
}


function int GetSPBestCurrentTime() {
	return SP_BestCurrentTime;
}

// Returns the current remaining time for the round
function int GetSPCurrentTime() {
	local s_GameReplicationInfo GRI;
	if (GamePeriod == GP_RoundPlaying) {
		GRI = s_GameReplicationInfo(GameReplicationInfo);
		if (GRI != None) {
			SP_CurrentRemainingTime = GRI.RoundDuration * 60 - (GRI.RoundStarted - GRI.Remainingtime);
			return SP_CurrentRemainingTime;
		}
		else
			return -1;
	}
	else if (GamePeriod == GP_PostMatch)
		return SP_CurrentRemainingTime;
	else if (GamePeriod == GP_PreRound)
		return -1;
	else
		return -1;
}

///////////////////////////////////////
// AddDefaultInventory
///////////////////////////////////////
// Give proper amount of cash
///////////////////////////////////////

function AddDefaultInventory( pawn PlayerPawn )
{
	local Bot				B;

	if ( PlayerPawn.IsA('s_Player') && PlayerPawn.PlayerReplicationInfo.bWaitingPlayer )
		return;

	if ( PlayerPawn.IsA('s_Player'))
		GivePlayerItems(PlayerPawn);
	else
		GiveTeamWeapons(PlayerPawn);
	
	if (PlayerPawn.IsA('s_Player')) {
		s_Player(PlayerPawn).Money = 0;
		AddMoney(PlayerPawn, SinglePlayerDefinition.PlayerMoney);
	}
	else if (PlayerPawn.IsA('s_bot')) {
		s_bot(PlayerPawn).Money = 0;
		if (s_bot(PlayerPawn).PlayerReplicationInfo.Team == SinglePlayerDefinition.PlayerTeam)
			AddMoney(PlayerPawn, SinglePlayerDefinition.TeamBotsMoney);
		else
			AddMoney(PlayerPawn, SinglePlayerDefinition.OpposingBotsMoney);
	}

	B = Bot(PlayerPawn);
	if ( B != None )
		B.bHasImpactHammer = false;

	BaseMutator.ModifyPlayer(PlayerPawn);

	PlayerPawn.SwitchToBestWeapon();
}

function GiveTeamWeapons(Pawn P) {
	local	Inventory	Inv;
	local	bool			bPistol, bKnife, bBinocs;
	local	string	CodedWeaponsString;


	if ( (P.PlayerReplicationInfo == None) || P.PlayerReplicationInfo.bIsSpectator || P.PlayerReplicationInfo.bWaitingPlayer)
		return;

	bKnife = false;
	bPistol = false;

	for ( Inv=P.Inventory; Inv!=None; Inv=Inv.Inventory )
	{	 
		if ( Inv.IsA('s_Knife') )
		{
			bKnife = true;
			break;
		}
	}
	if ( !bKnife )
		GiveWeapon(P, "s_SWAT.s_Knife");

	for ( Inv=P.Inventory; Inv!=None; Inv=Inv.Inventory )
	{
		if ( Inv.IsA('TO_Binocs') )
		{
			bBinocs = true;
			break;
		}
	}
	if ( !bBinocs )
		GiveWeapon(P, "s_SWAT.TO_Binocs");

	if (P.IsA('s_BotBase')) {
		if (s_BotBase(P).TO_SinglePlayerBotType == 3) { // Bot is the VIP
			CodedWeaponsString = SinglePlayerDefinition.VIPGivenWeapons;
		}
		else if (s_BotBase(P).TO_SinglePlayerBotType == 1) { // Bot is an additionnal bot
			CodedWeaponsString = SinglePlayerDefinition.OpposingBotsGivenWeapons;
		}
		else if (s_BotBase(P).TO_SinglePlayerBotType == 2) { // Bot is a special bot
			CodedWeaponsString = SinglePlayerDefinition.SpawnSpecialEnemyBotGivenWeapons;
		}
		else if (s_BotBase(P).TO_SinglePlayerBotType == 0) { // Bot is a normal bot
			if (P.PlayerReplicationInfo.Team == SinglePlayerDefinition.PlayerTeam) { // Bot is on the player's team
				CodedWeaponsString = SinglePlayerDefinition.TeamBotsGivenWeapons;
			}
			else {		// Bot isn't on the player's team
				CodedWeaponsString = SinglePlayerDefinition.OpposingBotsGivenWeapons;
			}
		}
		else {
			warn("TO_SinglePlayerGame::GiveTeamWeapons: Invalid bot type (" $ s_BotBase(P).TO_SinglePlayerBotType $ ")");
			assert(false);
		}
		if (CodedWeaponsString != "" && Caps(CodedWeaponsString) != "NONE")
			return;

	}

	if (!(P.IsA('PlayerPawn')))
		super.GiveTeamWeapons(P);
}

function GivePlayerItems( pawn PlayerPawn ) {
	GiveTeamWeapons(PlayerPawn);
}

// Gives certain weapons to the bot, selected from the given string
function DecodeAndGiveWeapons (pawn aPlayerPawn, string CodedWeaponString) {
	local int		currentWeapon;
	local int		currentCharacterPos;
	local string	currentCharacter;
	local Inventory Inv;

	currentWeapon = 0;
	currentCharacterPos = 0;

	if (CodedWeaponString == "" || Caps(CodedWeaponString) == "NONE" || left(CodedWeaponString,1) == " ") {
		//log("TO_SinglePlayerGame::DecodeAndGiveWeapons: No weapon string; not doing anything to bot's weapons");
		return;		// musn't do anything, let the bot buy his stuff
	}

	//log("TO_SinglePlayerGame::DecodeAndGiveWeapons: Function called with aPlayerPawn " $ aPlayerPawn.name $ " and CodedWeaponString = '" $ CodedWeaponString $ "'.");

	KillInventory(aPlayerPawn);
	if (aPlayerPawn.IsA('s_Bot'))
		s_Bot(aPlayerPawn).money = 300;		// Just enough to buy some ammo

	//while (currentWeapon <= class'TOModels.TO_WeaponsHandler'.default.NumWeapons &&
	//	currentCharacterPos <= len(CodedWeaponString)) {
	while (currentWeapon <= 23 &&
		currentCharacterPos <= len(CodedWeaponString)) {
		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		//log("TO_SinglePlayer::DecodeAndGiveWeapons: current character: " $ currentCharacter);
		if (currentCharacter != "-") {
			if (currentCharacter == caps(currentCharacter)) {
				//log("TO_SinglePlayer::DecodeAndGiveWeapons: Given weapon " $ class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon] $ " to player " $ aPlayerPawn.name);
				GiveWeapon(aPlayerPawn, class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon]);
			}
							
			currentWeapon++;
		}
		currentCharacterPos++;
	}

	currentCharacterPos++; // Skip the trailling '-'

	if (aPlayerPawn.IsA('s_Player')) {
		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		//log("TO_SinglePlayer::DecodeAndGiveWeapons: current character: " $ currentCharacter);
		if (currentCharacter == caps(currentCharacter)) {
			s_Player(aPlayerPawn).HelmetCharge = 100;
			//log("TO_SinglePlayer::DecodeAndGiveWeapons: Given Helmet to player.");
		}

		currentCharacterPos++;

		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		//log("TO_SinglePlayer::DecodeAndGiveWeapons: current character: " $ currentCharacter);
		if (currentCharacter == caps(currentCharacter)) {
			s_Player(aPlayerPawn).VestCharge = 100;
			//log("TO_SinglePlayer::DecodeAndGiveWeapons: Given Body armor to player.");
		}

		currentCharacterPos++;

		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		//log("TO_SinglePlayer::DecodeAndGiveWeapons: current character: " $ currentCharacter);
		if (currentCharacter == caps(currentCharacter)) {
			s_Player(aPlayerPawn).LegsCharge = 100;
			//log("TO_SinglePlayer::DecodeAndGiveWeapons: Given Leg Armor to player.");
		}
	}
	else if (aPlayerPawn.IsA('s_bot')) {
		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		//log("TO_SinglePlayer::DecodeAndGiveWeapons: current character: " $ currentCharacter);
		if (currentCharacter == caps(currentCharacter)) {
			s_bot(aPlayerPawn).HelmetCharge = 100;
			//log("TO_SinglePlayer::DecodeAndGiveWeapons: Given Helmet to bot.");
		}

		currentCharacterPos++;

		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		//log("TO_SinglePlayer::DecodeAndGiveWeapons: current character: " $ currentCharacter);
		if (currentCharacter == caps(currentCharacter)) {
			s_bot(aPlayerPawn).VestCharge = 100;
			//log("TO_SinglePlayer::DecodeAndGiveWeapons: Given Body armor to bot.");
		}

		currentCharacterPos++;

		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		//log("TO_SinglePlayer::DecodeAndGiveWeapons: current character: " $ currentCharacter);
		if (currentCharacter == caps(currentCharacter)) {
			s_bot(aPlayerPawn).LegsCharge = 100;
			//log("TO_SinglePlayer::DecodeAndGiveWeapons: Given Leg Armor to bot.");
		}
	}

	// Hack to make bots really hold when they have no weapons
	if (left(CodedWeaponString,28) == "gbumb-basmm-rshfc-pamgs-marm" &&
	 aPlayerPawn.IsA('s_Bot') && s_Bot(aPlayerPawn).TO_SinglePlayerBotAttitude == 1) {
		Inv = FindInventoryItem(aPlayerPawn, 's_Knife');
		if (Inv == none) {
			GiveWeapon(aPlayerPawn, "s_SWAT.s_Knife");
			Inv = FindInventoryItem(aPlayerPawn, 's_Knife');
		}
		s_Knife(Inv).AIRating = 0.401;
	}
		//GiveWeapon(aPlayerPawn, class'TOModels.TO_WeaponsHandler'.default.WeaponStr[20]); }

	
	if (hardCodedBehaviors != none)
		hardCodedBehaviors.GiveWeapons(aPlayerPawn);
}


// Allows the player to buy certain guns (even if he shouln't be able to) or not
function DecodeAndSetWeaponsPermissions (string CodedWeaponString) {
	local int					currentWeapon;
	local int					currentCharacterPos;
	local string				currentCharacter;
	local string				glockFix;	// Fix for TO 3.4 code
	local s_GameReplicationInfo	GRI;

	currentWeapon = 0;
	currentCharacterPos = 0;

	if (CodedWeaponString == "" || Caps(CodedWeaponString) == "NONE" || left(CodedWeaponString,1) == " ") {
		//log("TO_SinglePlayerGame::DecodeAndSetWeaponsPermissions: No weapon string; leaving the player's weapon inventory alone");
		return;		// musn't do anything; let the player's team # regulate what guns he can get
	}

	GRI = s_GameReplicationInfo(GameReplicationInfo);
	if (GRI == None) {
		warn("TO_SinglePlayerGame::DecodeAndSetWeaponsPermissions: s_GameReplicationInfo not found !!");
		return;
	}
	else {
		if (GRI.SpecialGameBehaviors == None)
			GRI.SpecialGameBehaviors = spawn(class 'TO_SpecialGameBehaviors');
	}


	// Clear the visible/hidden weapon list
	GRI.SpecialGameBehaviors.HiddenWeaponsListLength = 0;
	GRI.SpecialGameBehaviors.ForceShowWeaponsListLength = 0;

	//log("TO_SinglePlayerGame::DecodeAndSetWeaponsPermissions: Function called CodedWeaponString = '" $ CodedWeaponString $ "'.");

	while (currentWeapon <= class'TOModels.TO_WeaponsHandler'.default.NumWeapons &&
		currentCharacterPos <= len(CodedWeaponString)) {
		currentCharacter = Mid(CodedWeaponString, currentCharacterPos, 1);
		
		// Fix for TO 3.4 code
		if (currentWeapon == 0) {
			glockFix = currentCharacter;
			currentCharacter = "-";
			currentWeapon++;
		}
		else if (currentWeapon == 24) {
			currentCharacter = glockFix;
		}

		//log("TO_SinglePlayer::DecodeAndGiveWeapons: current character: " $ currentCharacter);
		if (currentCharacter != "-") {
			if (currentCharacter == "!" || currentCharacter == "*") {
				//log("TO_SinglePlayer::DecodeAndSetWeaponsPermissions: Leaving weapon " $ class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon] $ " alone.");
				// Do nothing: Permission on this weapon should be regulated by team #
				
				// Fix for TO 3.4 code
				if (class<s_Weapon>(DynamicLoadObject( class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon], class'Class' )).default.price > SinglePlayerDefinition.PlayerMoney) {
					GRI.SpecialGameBehaviors.addHiddenWeapon(class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon]);
				}
			}
			else if (currentCharacter == caps(currentCharacter)) {	// Weapon should be allowed
				//log("TO_SinglePlayer::DecodeAndSetWeaponsPermissions: Showing weapon " $ class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon] $ " to player.");
				GRI.SpecialGameBehaviors.addForceShowWeapon(class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon]);
			}
			else {	// Weapon should be hidden
				//log("TO_SinglePlayer::DecodeAndSetWeaponsPermissions: Hiding weapon " $ class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon] $ " to player.");
				GRI.SpecialGameBehaviors.addHiddenWeapon(class'TOModels.TO_WeaponsHandler'.default.WeaponStr[currentWeapon]);
			}
							
			currentWeapon++;
		}
		currentCharacterPos++;
	}

}


// Load the mission data from the map file
function TO_SinglePlayerDefinition getSinglePlayerDefinition(TO_SinglePlayerMaplist myList) {
	local TO_SinglePlayerDefinition def;
	local int MapNum;
	local int i;
	
	def = spawn(Class'TO_SinglePlayerDefinition');
	MapNum = myList.MapNum;

	def.Story1 =					myList.Story1[MapNum];
	def.Story2 =					myList.Story2[MapNum];
	def.Story3 =					myList.Story3[MapNum];
	def.Story4 =					myList.Story4[MapNum];
	def.Story5 =					myList.Story5[MapNum];
	if (def.Story1 == "")
		def.Story1 = "** Story text mission from configuration file!! **";

	def.ScenarioName =				myList.ScenarioName[MapNum];
	def.ScenarioDescription1 =		myList.ScenarioDescription1[MapNum];
	def.ScenarioDescription2 =		myList.ScenarioDescription2[MapNum];
	def.ScenarioText1 =				myList.ScenarioText1[MapNum];
	def.ScenarioText2 =				myList.ScenarioText2[MapNum];
	def.ScenarioText3 =				myList.ScenarioText3[MapNum];
	def.ScenarioText4 =				myList.ScenarioText4[MapNum];
	if (def.ScenarioText1 == "")
		def.ScenarioText1 = "**SCENARIO TEXT MISSING**";
	if (def.ScenarioName == "")
		def.ScenarioName = "**SCENARIO NAME MISSING**";
	if (def.ScenarioDescription1 == "")
		def.ScenarioDescription1 = "**SCENARIO DESCRIPTION MISSING**";
	def.EstimatedEnemyForces =		myList.EstimatedEnemyForces[MapNum];
	if (def.EstimatedEnemyForces == "")
		def.EstimatedEnemyForces = "** ESTIMATED ENEMY FORCES INFORMATION MISSING **";
	def.AllEnnemiesDeadMSG =		myList.AllEnnemiesDeadMSG[MapNum];
	def.VIPDeadMSG =				myList.VIPDeadMSG[MapNum];
	def.SpecialTriggersMSG =		myList.SpecialTriggersMSG[MapNum];
	def.C4ExplodedMSG =				myList.C4ExplodedMSG[MapNum];
	def.CongradMSG =				myList.CongradMSG[MapNum];

	def.AllowedPlayerWeapons =		myList.AllowedPlayerWeapons[MapNum];

	def.OpponentSkins =				myList.OpponentSkins[MapNum];
	def.TeamSkins =					myList.TeamSkins[MapNum];
	def.SelfSkin =					myList.SelfSkin[MapNum];

	def.RoundDuration =				myList.RoundDuration[MapNum];
	def.PlayerMoney	=				myList.PlayerMoney[MapNum];
	def.PlayerHealth =				myList.PlayerHealth[MapNum];
	def.PlayerSpawnSpot =			myList.PlayerSpawnSpot[MapNum];
	def.OpposingBotsMoney =			myList.OpposingBotsMoney[MapNum];
	def.TeamBotsMoney =				myList.TeamBotsMoney[MapNum];

	def.PlayerTeam =				myList.PlayerTeam[MapNum];
	def.OpposingBots =				myList.OpposingBots[MapNum];
	def.TeamBots =					myList.TeamBots[MapNum];
	def.TeamBotsIntelligence =		myList.TeamBotsIntelligence[MapNum];
	def.TeamBotsGivenWeapons =		myList.TeamBotsGivenWeapons[MapNum];
	def.TeamBotsHealth =			myList.TeamBotsHealth[MapNum];
	def.TeamBotsNames = spawn(class'TO_SPStringDecoder');
	def.TeamBotsNames.decodeStrings(myList.TeamBotsSpawnSpots[MapNum], myList.TeamBotsNames[MapNum]);


	def.BotsDifficulty =			myList.BotsDifficulty[MapNum];
	def.NeededTeamBotsSurvivors	=	myList.NeededTeamBotsSurvivors[MapNum];


	def.AdditionnalEnemyBot = spawn(class'TO_SPStringDecoder');
	def.AdditionnalEnemyBot.decodeStrings(myList.SpawnAdditionnalEnemyBotAt[MapNum], myList.SpawnAdditionnalEnemyBotName[MapNum]);
	def.AdditionnalEnemyBotAttitude =	myList.AdditionnalEnemyBotAttitude[MapNum];

	def.OpposingBotsGivenWeapons =		myList.OpposingBotsGivenWeapons[MapNum];
	def.OpposingBotsHealth =			myList.OpposingBotsHealth[MapNum];
	def.OpposingBotsNames = spawn(class'TO_SPStringDecoder');
	def.OpposingBotsNames.decodeStrings(myList.OpposingBotsNames[MapNum], myList.OpposingBotsNames[MapNum]);

	

	def.SpecialEnemyBot = spawn(class'TO_SPStringDecoder');
	def.SpecialEnemyBot.decodeStrings(myList.SpawnSpecialEnemyBotAt[MapNum], myList.SpawnSpecialEnemyBotName[MapNum]);

	def.bSpawnSpecialEnemy =				myList.bSpawnSpecialEnemy[MapNum];
	def.SpawnSpecialEnemyBotGivenWeapons =	myList.SpawnSpecialEnemyBotGivenWeapons[MapNum];
	def.SpawnSpecialEnemyBotAttitude =		myList.SpawnSpecialEnemyBotAttitude[MapNum];
	def.SpawnSpecialEnemyBotSkin =			myList.SpawnSpecialEnemyBotSkin[MapNum];
	def.SpawnSpecialEnemyBotHealth =		myList.SpawnSpecialEnemyBotHealth[MapNum];


	def.bSpawnEndGameTriggers =			myList.bSpawnEndGameTriggers[MapNum];
	def.EndGameTriggers = spawn(class'TO_SPStringDecoder');
	def.EndGameTriggers.decodeStringAndFloats(myList.SpawnEndGameTriggersAt[MapNum],
											  myList.SpawnEndGameTriggersRadius[MapNum],
											  myList.SpawnEndGameTriggersHeight[MapNum]);
	def.ExitRingZOffset =				myList.ExitRingZOffset[MapNum];

	def.bSpawnSpecialTriggers =			myList.bSpawnSpecialTriggers[MapNum];
	def.SpecialTriggers = spawn(class'TO_SPStringDecoder');
	def.SpecialTriggers.decodeStringAndFloats(myList.SpawnSpecialTriggersAt[MapNum],
											  myList.SpawnSpecialTriggersRadius[MapNum],
											  myList.SpawnSpecialTriggersHeight[MapNum]);
	def.SpawnSpecialTriggersUtility =	myList.SpawnSpecialTriggersUtility[MapNum];

	
	def.VIPSpawnSpot =		myList.VIPSpawnSpot[MapNum];
	def.VIPName =			myList.VIPName[MapNum];
	def.VIPSkin =			myList.VIPSkin[MapNum];
	def.VIPHealth =			myList.VIPHealth[MapNum];
	def.VIPGivenWeapons =	myList.VIPGivenWeapons[MapNum];
	def.VIPDifficulty =		myList.VIPDifficulty[MapNum];
	def.VIPAttitude =		myList.VIPAttitude[MapNum];

	if (myList.Exterminate[MapNum] == 1)
		def.bExterminate = true;
	else
		def.bExterminate = false;

	if (myList.NoHostages[MapNum] == 1)
		def.bNoHostages = true;
	else
		def.bNoHostages = false;

	if (def.PlayerHealth == 0)					def.PlayerHealth = 100;
	if (def.TeamBotsHealth == 0)				def.TeamBotsHealth = 100;
	if (def.OpposingBotsHealth == 0)			def.OpposingBotsHealth = 100;
	if (def.SpawnSpecialEnemyBotHealth == 0)	def.SpawnSpecialEnemyBotHealth = 100;
	if (def.VIPHealth == 0)						def.VIPHealth = 100;


	def.HardCodedBehaviorsClass = myList.HardCodedBehaviorsClass[MapNum];

	// This is used to give default names to bots
	for (i = 0; i < 33; i++) {
		if (def.TeamBotsNames.strings[i] == "")
			def.TeamBotsNames.strings[i] = "*Team" $ i $ "*";
		if (def.AdditionnalEnemyBot.strings[i] == "")
			def.AdditionnalEnemyBot.strings[i] = "*Addition" $ i $ "*";
		if (def.OpposingBotsNames.strings[i] == "")
			def.OpposingBotsNames.strings[i] = "*Normal" $ i $ "*";
		if (def.SpecialEnemyBot.strings[i] == "")
			def.SpecialEnemyBot.strings[i] = "*Special" $ i $ "*";
		if (def.VIPName == "")
			def.VIPName = "*VIP*";
	}

	return def;
}

function ReadMapData(TO_SinglePlayerMapList myList) {
	bPlayingAsTOTeam = myList.bPlayingAsTOTeam;
	bShowCumulatives = myList.ShouldDisplayCumulatives();
	
	SP_BestCurrentTime = myList.getBestTime();
	SP_BestTime = myList.getUserBestTime();
	SP_BestTOTime = myList.GetTOTeamBestTime();
	SP_CumTime = myList.getUserCumulativeTime();
	SP_CumTOTime = myList.getTOTeamCumulativeTime();
}

// Called when the VIP has been killed
function VIPDied() {
	if (bVIPDied)
		return;		// Only run this function once!
	
	bVIPDied = true;

	if (SinglePlayerDefinition.bSpawnSpecialEnemy == 2)
		SpawnSpecialBots();

	if (SinglePlayerDefinition.bSpawnEndGameTriggers == 2)
		SpawnEndGameTriggers();

	if (SinglePlayerDefinition.bSpawnSpecialTriggers == 2)
		SpawnSpecialTriggers();

	if (SinglePlayerDefinition.VIPDeadMSG != "")
		BroadcastVIPDeadMSG();



	if (SinglePlayerDefinition.bSpawnEndGameTriggers == 0) {
		if ( TO_SinglePlayerGame(Level.Game).SinglePlayerDefinition.PlayerTeam == 0 ) {
			SetWinner(0);
			//BroadcastLocalizedMessage(class's_MessageRoundWinner', 9);
		}
		else {
			SetWinner(1);
			//BroadcastLocalizedMessage(class's_MessageRoundWinner', 10);
		}
		EndGame("The VIP has been killed.");
	}

}

// Spawn Special Bots if they havn't already been spawned
function SpawnSpecialBots() {
	local bot NewBot;
	local NavigationPoint StartSpot, OldStartSpot;
	local int DesiredTeam, i, MinSize;

	//log("TO_SinglePlayerGame::SpawnSpecialBots: Function called.");

	if (NumberOfSpecialBot == -1 || bSpecialBotsSpawned)
		return;

	if (!bEndGameTriggersSpawned)
		bSpawnedSpecialBotsBeforeEndGameTriggers = true;

	//log("TO_SinglePlayerGame::SpawnSpecialBots: bSpecialBotsSpawned check passed.");

	bSpecialBotsSpawned = true;
	SP_NumEnemies += NumberOfSpecialBot + 1;

	for (i = 0; i < NumberOfSpecialBot + 1; i++) {

		//log("TO_SinglePlayerGame::SpawnSpecialBots: Spawning special bot #" $ (i+1) $ " out of " $ (NumberOfSpecialBot+1) $ ".");

		currentSpecialBotNumber = i;

		//StartSpot = FindNavigationPoint(SinglePlayerDefinition.SpecialEnemyBot.names[currentSpecialBotNumber]);
	
		NewBot = SpawnBot(StartSpot);
		if ( NewBot == none )
		{
			warn("TO_SinglePlayerGame::SpawnSpecialBots: Failed to spawn bot at " $ StartSpot.name);
			//assert(false);
		}

		NewBot.Health = SinglePlayerDefinition.SpawnSpecialEnemyBotHealth;
		if (SinglePlayerDefinition.PlayerTeam == 0)
			DesiredTeam = 1;
		else
			DesiredTeam = 0;
		NewBot.PlayerReplicationInfo.bIsABot = True;

		NewBot.PlayerReplicationInfo.Team = DesiredTeam;

		//log("TO_SinglePlayer::SpawnSpecialBots: Spawnspot == " $ StartSpot.Name);

		NewBot.SetLocation(StartSpot.Location);
		NewBot.SetRotation(StartSpot.Rotation);
		NewBot.ViewRotation = StartSpot.Rotation;
		NewBot.SetRotation(NewBot.Rotation);

		ChangePModelPlus(NewBot, SinglePlayerDefinition.SpawnSpecialEnemyBotSkin, NewBot.PlayerReplicationInfo.Team, false);
		DecodeAndGiveWeapons(NewBot, SinglePlayerDefinition.SpawnSpecialEnemyBotGivenWeapons);
	}





}

// Spawn the trigger that allows the player to end the mission
function SpawnEndGameTriggers() {
	local int						currentTriggerNumber;
	local TO_SinglePlayerTrigger	currentTrigger;
	local actor						positionningActor;
	local pawn						PawnLink;
	local TO_SP_ExitRing			ExitRing;

	//log("TO_SinglePlayerGame::SpawnEndGameTrigger: Function called.");

	if (bEndGameTriggersSpawned)
		return;		// Only spawn the trigger once

	bEndGameTriggersSpawned = true;

	// Check if there is a trigger spawn spot in the mission definition
	if (SinglePlayerDefinition.EndGameTriggers.names[0] ~= "None" ||
		SinglePlayerDefinition.EndGameTriggers.names[0] == "") {
		warn("TO_SinglePlayerGame::SpawnEndGameTriggers: No actor found to spawn end game triggers at!!");
		assert(false);
	}

	// Delete exit rings if they exist
	//foreach allactors(class'TO_SP_ExitRing', ExitRing)
	//	ExitRing.Destroy();

	// Spawn them triggers & exit rings
	currentTriggerNumber = 0;
	while ( !(SinglePlayerDefinition.EndGameTriggers.names[currentTriggerNumber] ~= "None" ||
		      SinglePlayerDefinition.EndGameTriggers.names[currentTriggerNumber] == "") ) {
		
		positionningActor = FindActor(SinglePlayerDefinition.EndGameTriggers.names[currentTriggerNumber]);
		if (positionningActor == none) {
			warn("TO_SinglePlayerGame::SpawnEndGameTriggers: Could not find actor to spawn end game trigger at! (Name == " $ SinglePlayerDefinition.EndGameTriggers.names[currentTriggerNumber] $ ")");
			assert(false);
		}

		//log("TO_SinglePlayerGame::SpawnEndGameTriggers: Spawning end game trigger at " $ SinglePlayerDefinition.EndGameTriggers.names[currentTriggerNumber] $ " with radius " $ SinglePlayerDefinition.EndGameTriggers.radiuses[currentTriggerNumber] $ " and height " $ SinglePlayerDefinition.EndGameTriggers.heights[currentTriggerNumber] $ ".");

		currentTrigger = spawn(class'TO_SinglePlayerTrigger',,, positionningActor.location);
		currentTrigger.triggerType = 0;
		currentTrigger.triggerOnceOnly = true;

		currentTrigger.SetCollisionSize(SinglePlayerDefinition.EndGameTriggers.radiuses[currentTriggerNumber],
			SinglePlayerDefinition.EndGameTriggers.heights[currentTriggerNumber]);

		currentTrigger.SetCollision(true);

		currentTrigger.triggerOnceOnly = false;

		//currentTrigger.doCollisionChecks();

		ExitRing = Spawn(class'TO_SP_ExitRing',,, currentTrigger.location + vect(0,0,1) * SinglePlayerDefinition.ExitRingZOffset);
		ExitRing.DrawScale = currentTrigger.CollisionRadius / 100;

		anExitObject = currentTrigger;

		currentTriggerNumber++;
	}

	// Make every bot follow the player
	if (SinglePlayerDefinition.bSpawnEndGameTriggers != 1) {
		for ( PawnLink=Level.PawnList; PawnLink!=None; PawnLink=PawnLink.nextPawn )	{
			if ( (PawnLink.PlayerReplicationInfo != None) && !PawnLink.PlayerReplicationInfo.bWaitingPlayer ) {
				if ( ((s_Bot(PawnLink) != None) && (!s_Bot(PawnLink).bNotPlaying))) { // Is a bot
					if (ExitSpawnedBotsAttitude == 0)
						s_Bot(PawnLink).SetOrders('Follow', GetPlayer(), false);
					else if (ExitSpawnedBotsAttitude == 1 && PawnLink.PlayerReplicationInfo.Team == SinglePlayerDefinition.PlayerTeam)
						s_Bot(PawnLink).SetOrders('Follow', GetPlayer(), false);
				}
			}
		}
	}
}


// Spawns the special purpose triggers.
function SpawnSpecialTriggers() {
	local int						currentTriggerNumber;
	local TO_SinglePlayerTrigger	currentTrigger;
	local actor						positionningActor;

	//log("TO_SinglePlayerGame::SpawnSpecialTriggers: Function called.");

	if (bSpecialTriggersSpawned)
		return;		// Only spawn the trigger once

	bSpecialTriggersSpawned = true;

	// Check if there is a trigger spawn spot in the mission definition
	if (SinglePlayerDefinition.SpecialTriggers.names[0] ~= "None" ||
		SinglePlayerDefinition.SpecialTriggers.names[0] == "") {
		warn("TO_SinglePlayerGame::SpawnSpecialTriggers: No actor found to spawn special triggers at!!");
		assert(false);
	}

	currentTriggerNumber = 0;

	while ( !(SinglePlayerDefinition.SpecialTriggers.names[currentTriggerNumber] ~= "None" ||
		      SinglePlayerDefinition.SpecialTriggers.names[currentTriggerNumber] == "") ) {
		
		positionningActor = FindActor(SinglePlayerDefinition.SpecialTriggers.names[currentTriggerNumber]);
		if (positionningActor == none) {
			warn("TO_SinglePlayerGame::SpawnSpecialTriggers: Could not find actor to spawn end game trigger at! (Name == " $ SinglePlayerDefinition.SpecialTriggers.names[currentTriggerNumber] $ ")");
			assert(false);
		}

		//log("TO_SinglePlayerGame::SpawnSpecialTriggers: Spawning special trigger at " $ SinglePlayerDefinition.SpecialTriggers.names[currentTriggerNumber] $ " with radius " $ SinglePlayerDefinition.SpecialTriggers.radiuses[currentTriggerNumber] $ ", height " $ SinglePlayerDefinition.SpecialTriggers.heights[currentTriggerNumber] $ " and type " $ SinglePlayerDefinition.SpawnSpecialTriggersUtility $ ".");

		currentTrigger = spawn(class'TO_SinglePlayerTrigger',,, positionningActor.location);
		currentTrigger.triggerType = SinglePlayerDefinition.SpawnSpecialTriggersUtility;
		currentTrigger.triggerOnceOnly = true;

		currentTrigger.SetCollisionSize(SinglePlayerDefinition.SpecialTriggers.radiuses[currentTriggerNumber],
			SinglePlayerDefinition.SpecialTriggers.heights[currentTriggerNumber]);

		currentTrigger.SetCollision(true);

		//currentTrigger.doCollisionChecks();


		currentTriggerNumber++;
	}

}


// Force the winning of the round, no matter what is going on in the game (used by the End Game Trigger)
function ForceWinRound() {
	//log("TO_SinglePlayerGame::ForceWinRound: Function called.");

	if (bAlreadyWon || bGameEnded || (GamePeriod != GP_RoundPlaying))
		return;

	if (hardCodedBehaviors != none)
		if (!(hardCodedBehaviors.AllowPlayerWin()))
			return;

	bAlreadyWon = true;

	if (SinglePlayerDefinition.PlayerTeam == 0)
		SetWinner(0);
	else
		SetWinner(1);

	//BroadcastLocalizedMessage(class'TO_SP_Messages', 6);
	EndGame(class'TO_SP_Messages'.default.WinMessage[6]);

}

// Invoque the hard coded behaviors hook for special triggers
function InvoqueHCBSpecialTriggerHook(TO_SinglePlayerTrigger trigger) {
	//log("TO_SinglePlayerGame::InvoqueHCBSpecialTriggerHook: Function called.");
	if (hardCodedBehaviors == none) {
		warn("TO_SinglePlayerGame::InvoqueHCBSpecialTriggerHook: Could not find HardCodedBehaviors actor!!");
		assert(false);
	}
	else {
		hardCodedBehaviors.SpecialTriggerCalled(trigger);
	}
}

// Resets the position of a bot (after each round)
function NavigationPoint ResetPawnPosition(s_BotBase aBot) {
	local int type, number;
	local bool				movedSucessfully;
	local NavigationPoint	startSpot;

	type = int(mid(aBot.MenuName, 9, 1));
	number = int(mid(aBot.MenuName, 11, len(aBot.MenuName) - 11));

	//log("TO_SP: Func called for name = " $ string(aBot.name) $ ", TO_SinglePlayerBotType = " $ aBot.TO_SinglePlayerBotType);

	if (type == 1) { // Additionnal bot
		currentAdditionnalBotNumber++;
		startSpot = FindNavigationPoint(SinglePlayerDefinition.AdditionnalEnemyBot.names[number]);
	}
	else if (type == 2) { // special bot
		currentSpecialBotNumber++;
		startSpot = FindNavigationPoint(SinglePlayerDefinition.SpecialEnemyBot.names[number]);
	}
	else if (type == 3) { // VIP
		startSpot = FindNavigationPoint(SinglePlayerDefinition.VIPSpawnSpot);
	}
	else if (type == 4) { // Normal teammate
		currentTeamBotNumber++;
		if (SinglePlayerDefinition.TeamBotsNames.names[number] == "None")
			return None;
		else
			startSpot = FindNavigationPoint(SinglePlayerDefinition.TeamBotsNames.names[number]);
	}
	else if (type == 5) { // Normal enemy bot
		return none;
	}
	else {
		warn("Bot has unknown type!!");
		assert(false);
	}

	/*if (aBot.TO_SinglePlayerBotType == 3) { // Bot is the VIP
		startSpot = FindNavigationPoint(SinglePlayerDefinition.VIPSpawnSpot);
	}
	else if (aBot.TO_SinglePlayerBotType == 2) { // Special Bot
		currentSpecialBotNumber++;
		startSpot = FindNavigationPoint(SinglePlayerDefinition.SpecialEnemyBot.names[currentSpecialBotNumber]);
		//log("TO_SP::ResetPawnPosition: Reset special bot # " $ currentSpecialBotNumber $ " (name = " $ SinglePlayerDefinition.SpecialEnemyBot.names[currentSpecialBotNumber] $ ", actorname = " $ string(aBot.name) $ ")");
	}
	else if (aBot.TO_SinglePlayerBotType == 1) { // Additionnal Bot
		currentAdditionnalBotNumber++;
		startSpot = FindNavigationPoint(SinglePlayerDefinition.AdditionnalEnemyBot.names[currentAdditionnalBotNumber]);
		//log("TO_SP::ResetPawnPosition: Reset additionnal bot # " $ currentAdditionnalBotNumber $ " (name = " $ SinglePlayerDefinition.AdditionnalEnemyBot.names[currentAdditionnalBotNumber] $ ", actorname = " $ string(aBot.name) $ ")");
	}
	else if (aBot.PlayerReplicationInfo.Team == SinglePlayerDefinition.PlayerTeam) {
		currentTeamBotNumber++;
		if (SinglePlayerDefinition.TeamBotsNames.names[currentTeamBotNumber] == "None")
			return None;
		else
			startSpot = FindNavigationPoint(SinglePlayerDefinition.TeamBotsNames.names[currentTeamBotNumber]);
	}
	else
		return None;*/

	return startSpot;

	//aBot.SetLocation(startSpot.Location);

}

// Do not rebalance teams
function ReBalance() { }

/*function SetBotOrders(Bot NewBot) {
	return;
}*/

function SetBotObjectives(s_Bot B) {
	//log("TO_SinglePlayerGame::SetBotObjectives: Func called for bot " $ string(B.name) $ " and attitude = " $ B.TO_SinglePlayerBotAttitude);
	//log("Bot " $ string(b.name) $ " skill: " $ b.Skill);

	switch (B.TO_SinglePlayerBotAttitude) {
		case 0:		// Do nothing, bot should get normal objectives
			break;

		case 1:		// Hold position; doesn't work too well
			//B.SetOrders('Hold', B, false);
			MakeBotHoldAtProperSpot(B);
			//B.MoveTarget = B.OrderObject;
			//B.SetOrders('Hold', FindNavigationPoint("PathNode55").spawn(class'babycow'), false);
			break;

		case 2:		// Freelance (shouldn't be used, use 'normal objectives' instead
			B.SetOrders('Freelance', None, true);
			break;

		case 3:		// Roaming (shouln't be used, use 'normal objectives' instead
			B.SetOrders('Roaming', None, true);
			break;

		case 4:		// Follow VIP
			B.SetOrders('Follow', GetVIP(), true);
			break;

		case 5:		// Defend at spawn; doesn't work in TO
			B.SetOrders('Defend', None, true);
			break;

		case 6:		// Attack player; broken for unknown reasons (bot receives obj, but do nothing about it)
			if (HoldSpot(B.OrderObject) != None) {
				B.OrderObject.Destroy();
			}
			B.OrderObject = GetPlayer();
			B.SetOrders('Attack', GetPlayer(), true);
			break;

		case 7:		// Hold at the defense point near the bot. If there is none, act 'normally'.
			B.DefensePoint = FindDefensePointNearestTo(B);
			if (B.DefensePoint != None) {	// Defense point found near the bot
				B.FovAngle = 350;
				B.SightRadius = 20000;
				B.GotoState('Defending');
			}
			else {							// No defense point found near the bot
				B.FovAngle = 90;
				B.SetPeripheralVision();
				B.GotoState('Hunting');
			}
			break;

		case 8:		// Hold at spawn point and get modified, high-accuracy guns
			break;

		case 9:
			B.SetOrders('Follow', GetPlayer(), false);
			break;

		case 10:
			//if (CanAcceptObjective(B)) {
				B.Objective = 'O_SeekForHostages';
				B.OrderObject = none;
				B.O_number = 2;				
			//}
			break;

	}
}

function MakeBotHoldAtProperSpot(s_Bot bot) {
	local int type, number;
	local navigationpoint	spot;
	//local int type, number;

	//log("Bot information = " $ bot.MenuName);
	//log("Bot lenght = " $ len(bot.MenuName));
	//log("Bot type = " $ mid(bot.MenuName, 9, 1));
	//log("Bot number = " $ mid(bot.MenuName, 11, len(bot.MenuName) - 11));
	
	type = int(mid(bot.MenuName, 9, 1));
	number = int(mid(bot.MenuName, 11, len(bot.MenuName) - 11));

	bot.SetOrders('Hold', bot, false);

	if (type == 1) // Additionnal bot
		spot = FindNavigationPoint(SinglePlayerDefinition.AdditionnalEnemyBot.names[number]);
	else if (type == 2) // special bot
		spot = FindNavigationPoint(SinglePlayerDefinition.SpecialEnemyBot.names[number]);
	else if (type == 3) // VIP
		spot = FindNavigationPoint(SinglePlayerDefinition.VIPSpawnSpot);
	else if (type == 4) { // Normal teammate
		if (SinglePlayerDefinition.TeamBotsNames.names[number] == "None" ||
		 SinglePlayerDefinition.TeamBotsNames.names[number] == "")
			return;
		else
			spot = FindNavigationPoint(SinglePlayerDefinition.TeamBotsNames.names[number]);
	}
	else if (type == 5) {
		return;
		//warn("Cannot make normal enemy bot hold.");
		//assert(false);
	}
	else {
		warn("Bot has unknown type!!");
		assert(false);
	}

	if (bot.OrderObject != none)
		bot.OrderObject.Destroy();

	bot.OrderObject = spot.Spawn(class'HoldSpot');

}

function pawn GetVIP() {
	local pawn P;
	for ( P=Level.PawnList; P!=None; P=P.nextPawn ) {
		if (P.IsA('s_Bot'))
			if (s_Bot(P).TO_SinglePlayerBotType == 3)
				return P;
	}

	warn("TO_SinglePlayerGame::GetVIP: Could not find VIP bot !");
	//assert(false);
	return none;
}

function s_Player GetPlayer() {
	local pawn P, bestP;
	local float	Score, bestScore;

	bestScore = 0;

	if (Protagonist != none)
		return Protagonist;

	for ( P=Level.PawnList; P!=None; P=P.nextPawn ) {
		if (P.IsA('s_Player') && (P.PlayerReplicationInfo != None) && s_Player(P).health != -1) {
			Score = frand();
			if (Score > bestScore) {
				bestP = P;
				bestScore = Score;
			}

		}
	}

	if (bestP == none) {
		warn("TO_SinglePlayerGame::GetVIP: Could not find any players !");
		//assert(false);
	}
	else
		Protagonist = s_player(bestP);

	return s_player(bestP);
}

function int SWATReduceDamage(int Damage, name DamageType, pawn injured, pawn instigatedBy, Vector HitLocation) {
	//log("TO_SinglePlayerGame::SWATReduceDamage: Function called!");
	if (injured.IsA('s_Bot')) {
		if (instigatedBy.IsA('s_Player')) {		// Caused by the player
			if (s_Bot(injured).PlayerReplicationInfo.Team == SinglePlayerDefinition.PlayerTeam) { // Is a teammate
				SP_FriendflyFireIncidents++;
				if (DamageType == 'shot' || DamageType == 'stabbed' || DamageType == 'Decapitated')
					SP_ShotsHitFriendly++;
			}
			else {
				if (DamageType == 'shot' || DamageType == 'stabbed' || DamageType == 'Decapitated')	{ // Do not take in consideration explosions
					SP_ShotsHitTarget++;

					if (HitLocation.Z > 28)
						SP_EnemyHeadHits++;
					else if (HitLocation.Z > 0)
						SP_EnemyBodyHits++;
					else
						SP_EnemyLegsHits++;
				}
			}
		}
	}
	else if (injured.IsA('s_player')) {
		if (DamageType == 'shot' || DamageType == 'stabbed' || DamageType == 'Decapitated')	{ // Do not take in consideration explosions
			if (HitLocation.Z > 28)
				SP_PlayerHeadHits++;
			else if (HitLocation.Z > 0)
				SP_PlayerBodyHits++;
			else
				SP_PlayerLegsHits++;
		}
	}


	return super.SWATReduceDamage(Damage, DamageType, injured, instigatedBy, HitLocation);
}

function bool PickupQuery( Pawn Other, Inventory item ) {
	local bool		result;
	local int		bUseDefault;
	
	if (hardCodedBehaviors != none) {
		result = hardCodedBehaviors.PickupQuery(Other, item, bUseDefault);
		if (bUseDefault == 0)	// false, then use the returned result
			return result;
	}
	
	return super.PickupQuery(Other, item);
}

function Inventory FindInventoryItem(pawn aPawn, name aName) {
	local	Inventory	Inv;

	for (Inv = aPawn.Inventory; Inv != None; Inv = Inv.Inventory)
		if (Inv.IsA(aName))
			return Inv;

	return none;
}

///////////////////////////////////////
// ChangeTeam
///////////////////////////////////////
// Block team switching
///////////////////////////////////////

//function bool ChangeTeam(Pawn Other, int NewTeam) { return true; }


// Do not spawn evidence
function SpawnEvidence() { }
function SpawnSpecialItems() { }

// Do not drop money
function DropMoney(Pawn Other, int PezAmount, Vector DropLocation) { }

// Block addbots
function bool ForceAddBot() { return false; }

function NavigationPoint FindNavigationPoint(string navPointName) {
	local NavigationPoint n;
	foreach AllActors (class'NavigationPoint', n)
		if (caps(string(n.name)) ~= caps(navPointName))
			return n;

	warn("TO_SinglePlayerGame::FindNavigationPoint: Could not find navigation point: " $ navPointName);
	return None;
}

function Actor FindActor(string positionningActor) {
	local Actor a;
	foreach AllActors (class'Actor', a)
		if (caps(string(a.name)) ~= caps(positionningActor))
			return a;

	warn("TO_SinglePlayerGame::FindActor: Could not find actor: " $ positionningActor);
	return None;
}

function BroadcastMissionMessage(string message) {
	local s_Player	p;
	//log("TO_SinglePlayerGame::BroadcastMissionMessage: Function called with message = " $ message);

	foreach AllActors(class's_player', p)
		P.TeamMessage( P.PlayerReplicationInfo, message, 'SPMissionMessage', true );

}

function BroadcastAllEnnemiesDeadMSG() {
	// Only send the 'all ennemies dead' message once
	if (bAlreadyBroadcastedAllEnnemiesDeadMSG)
		return;

	bAlreadyBroadcastedAllEnnemiesDeadMSG = true;

	BroadcastMissionMessage(SinglePlayerDefinition.AllEnnemiesDeadMSG);
}

function BroadcastVIPDeadMSG() {
	// Only send the 'VIP is dead' message once
	if (bAlreadyBroadcastedVIPDeadMSG)
		return;

	bAlreadyBroadcastedVIPDeadMSG = true;

	BroadcastMissionMessage(SinglePlayerDefinition.VIPDeadMSG);
}

function SpecialTriggersMSG() {
	// Only send the 'Special trigger used' message once
	if (bAlreadyBroadcastedSpecialTriggersMSG)
		return;

	bAlreadyBroadcastedSpecialTriggersMSG = true;

	BroadcastMissionMessage(SinglePlayerDefinition.SpecialTriggersMSG);
}

function BroadcastC4ExplodedMSG() {
	// Only send the 'Special trigger used' message once
	if (bAlreadyBroadcastedC4ExplodedMSG)
		return;

	bAlreadyBroadcastedC4ExplodedMSG = true;

	BroadcastMissionMessage(SinglePlayerDefinition.C4ExplodedMSG);
}

function TO_DefensePoint FindDefensePointNearestTo(s_Bot bot) {
	local TO_DefensePoint result;

	foreach RadiusActors(class'TO_DefensePoint', result, 96, bot.location)
		return result;

	return none;
}

///////////////////////
// Skin changing code
///////////////////////

final function ChangePModelPlus( Pawn P, int num, int team, bool bDie)
{
	local	Actor	StartPoint;
	local	byte	OldTeam;
	local	s_Player	sP;
	local	bool	bNoWeapons, bChangeModel, bChangeTeam, bNotPlaying;

	if ( bBalancing )
		return;

	sP = s_Player(P);

	OldTeam = P.PlayerReplicationInfo.Team;
	bChangeTeam = OldTeam != team;

	/*// Allow only one team change per round.
	if (sP != None)
	{
		if ( sP.bAlreadyChangedTeam && bChangeTeam )
		{
			sP.ReceiveLocalizedMessage(class's_MessageVote', 7);
			return;
		}

		if ( !sP.PlayerReplicationInfo.bWaitingPlayer && bChangeTeam )
			sP.bAlreadyChangedTeam = true;

		bNotPlaying = sP.bNotPlaying || sP.PlayerReplicationInfo.bWaitingPlayer;
		bNoWeapons = sP.bDead || sP.PlayerReplicationInfo.bWaitingPlayer;
		bChangeModel = (num == 255) || (num != sP.PlayerModel);

		// Prevent player from dying during team/model switch?
		if ( bNotPlaying || (!bChangeTeam) || (GamePeriod != GP_RoundPlaying) )
			bDie = false;	
	}
	else if ( P.PlayerReplicationInfo.bWaitingPlayer || P.PlayerReplicationInfo.bIsSpectator )
		bDie = false;*/

	//log("TO_GameBasics::ChangePModelPlus - P"@P.GetHumanName()@"num"@num@"team"@team@"bDie"@bDie);

	if ( bDie )
	{
		P.TakeDamage(500000, None, P.Location, vect(0,0,0), 'ChangedTeam');
	}
	else
	{
		//log("TO_GameBasics::ChangePModelPlus - don't kill");
		if ( bChangeTeam && !bNoWeapons )
		{
			//log("TO_GameBasics::ChangePModelPlus - drop inventory");
			// Drop specific inventory
			DropInventory(P, false);
		}
	}

	if ( bChangeTeam )
		ChangeTeam(P, team);

	// Change Team successful, change model
	if ( (P.PlayerReplicationInfo.Team == Team) && (Team<3) )
	{
		if ( num == 255 )
		{
			assert(false);
			if ( team == 1 )
				SetRandomSFModel(P);
			else
				SetRandomTerrModel(P);
		}
		else
			ChangeModelPlus(P, num);

		// Livefeeds of current team.
		if ( bChangeTeam && (P.GetStateName() == 'PlayerSpectating') )
			PlayerPawn(P).Fire(0.0);
	}

	// giving default pistol
	if ( !bDie && bChangeTeam && !bNoWeapons && !bNotPlaying )
	{
		GiveTeamWeapons(P);
		if ( P.Weapon == None )
			P.SwitchToBestWeapon();
	}
}

final function ChangeModelPlus( Pawn P, int num)
{
	num = DressModel(P, num);
	
	// Set model num to pawn
	if ( P.IsA('s_Bot') )
	{
		s_Bot(P).PlayerModel = num;
		s_BotInfo(BotConfig).CheckBotName(Bot(P));
	}
	else if (P.IsA('s_Player'))
		s_Player(P).PlayerModel = num;
}

final function byte DressModel(Actor A, byte num) 
{
	local	Pawn P;

	P = Pawn(A);

	//log("Switching skin of " $ p.name $ " to " $ num);

	// Mesh
	if ( P != None ) {
		if ( (num==255) || (class'TOPModels.TO_ModelHandler'.default.ModelType[num] == MT_None)) {
			assert(false);
		}
		
		// Only assign mesh if player is visible!
		if ( !P.PlayerReplicationInfo.bWaitingPlayer && !P.PlayerReplicationInfo.bIsSpectator )
			A.Mesh = class'TOPModels.TO_ModelHandler'.default.ModelMesh[num];

		P.SelectionMesh = class'TOPModels.TO_ModelHandler'.default.ModelName[num];
	}
	else
	{
		//assert(false);
		// Make sure model is valid
		//if ( (num==255) || (default.ModelType[num] == MT_None) )
		//	num = GetNextModel(num, Rand(2));
			
		A.Mesh = class'TOPModels.TO_ModelHandler'.default.ModelMesh[num];
	}

	// Skins
	A.MultiSkins[0] = Texture(DynamicLoadObject(class'TOPModels.TO_ModelHandler'.default.Skin0[num], class'Texture'));
	A.MultiSkins[1] = Texture(DynamicLoadObject(class'TOPModels.TO_ModelHandler'.default.Skin1[num], class'Texture'));
	A.MultiSkins[2] = Texture(DynamicLoadObject(class'TOPModels.TO_ModelHandler'.default.Skin2[num], class'Texture'));
	A.MultiSkins[3] = Texture(DynamicLoadObject(class'TOPModels.TO_ModelHandler'.default.Skin3[num], class'Texture'));
	A.MultiSkins[4] = Texture(DynamicLoadObject(class'TOPModels.TO_ModelHandler'.default.Skin4[num], class'Texture'));

	if ( class'TOPModels.TO_ModelHandler'.default.Skin5[num] != "" )
		A.MultiSkins[5] = Texture(DynamicLoadObject(class'TOPModels.TO_ModelHandler'.default.Skin5[num], class'Texture'));
	else
		A.MultiSkins[5] = None;

	if ( class'TOPModels.TO_ModelHandler'.default.Skin6[num] != "" )
		A.MultiSkins[6] = Texture(DynamicLoadObject(class'TOPModels.TO_ModelHandler'.default.Skin6[num], class'Texture'));
	else
		A.MultiSkins[6] = None;

	if ( class'TOPModels.TO_ModelHandler'.default.Skin7[num] != "" )
		A.MultiSkins[7] = Texture(DynamicLoadObject(class'TOPModels.TO_ModelHandler'.default.Skin7[num], class'Texture'));
	else
		A.MultiSkins[7] = None;

	// Voice
	if ( P != None )
	{
		if (bCrossDressers && num != 0 && num != 13) {	// invert male and female voices
			if ( class'TOPModels.TO_ModelHandler'.default.ModelType[num] == MT_Terrorist ) // Terrorist
			{
				if ( class'TOPModels.TO_ModelHandler'.default.bFemale[num] > 0 )
					P.PlayerReplicationInfo.VoiceType = class<VoicePack>(DynamicLoadObject("TODatas.VoiceT1", class'Class'));
				else
					P.PlayerReplicationInfo.VoiceType = class<VoicePack>(DynamicLoadObject("TODatas.VoiceFT1", class'Class'));
			}
			else // Special Forces
			{
				if ( class'TOPModels.TO_ModelHandler'.default.bFemale[num] > 0 )
					P.PlayerReplicationInfo.VoiceType = class<VoicePack>(DynamicLoadObject("TODatas.VoiceSF1", class'Class'));
				else
					P.PlayerReplicationInfo.VoiceType = class<VoicePack>(DynamicLoadObject("TODatas.VoiceFSF1", class'Class'));
			}
		}
		else { // normal voices
			if ( class'TOPModels.TO_ModelHandler'.default.ModelType[num] == MT_Terrorist ) // Terrorist
			{
				if ( class'TOPModels.TO_ModelHandler'.default.bFemale[num] > 0 )
					P.PlayerReplicationInfo.VoiceType = class<VoicePack>(DynamicLoadObject("TODatas.VoiceFT1", class'Class'));
				else
					P.PlayerReplicationInfo.VoiceType = class<VoicePack>(DynamicLoadObject("TODatas.VoiceT1", class'Class'));
			}
			else // Special Forces
			{
				if ( class'TOPModels.TO_ModelHandler'.default.bFemale[num] > 0 )
					P.PlayerReplicationInfo.VoiceType = class<VoicePack>(DynamicLoadObject("TODatas.VoiceFSF1", class'Class'));
				else
					P.PlayerReplicationInfo.VoiceType = class<VoicePack>(DynamicLoadObject("TODatas.VoiceSF1", class'Class'));
			}
		}
		
		//log("TO_ModelHandler::DressModel - Setting voice to:"@P.PlayerReplicationInfo.PlayerName@P.PlayerReplicationInfo.VoiceType);
	}

	return num;
}

/////////////

// If required, spawns a buy zone at the player's spawn spot
function SpawnBuyZoneForPlayer() {
	local s_ZoneControlPoint	zone;
	local bool					bBuyZoneFound;
	local NavigationPoint		pathnode, pathnode_actor;

	bBuyZoneFound = false;

	pathnode = FindNavigationPoint(SinglePlayerDefinition.PlayerSpawnSpot);

	if (pathnode == none) {
		warn("SpawnBuyZoneForPlayer() called, but no valid PlayerSpawnSpot specified!");
		assert(false);
	}

	foreach AllActors(class's_ZoneControlPoint', zone) {
		foreach RadiusActors(class'NavigationPoint', pathnode_actor, sqrt(zone.CollisionRadius * zone.CollisionRadius + zone.CollisionHeight * zone.CollisionHeight) + 5, zone.Location) {
			if (pathnode_actor == pathnode) {
				zone.bBuyPoint = true;
				zone.OwnedTeam = SinglePlayerDefinition.PlayerTeam;
				bBuyZoneFound = true;
			}
		}
	}

	if (bBuyZoneFound)
		return;

	zone = spawn(class'TO_SP_BuyZone',,, pathnode.location);
	zone.OwnedTeam = SinglePlayerDefinition.PlayerTeam;

}

function GiveBomb() {
	if (hardCodedBehaviors == none)
		super.GiveBomb();
	else if (hardCodedBehaviors.ShouldGiveBomb())
		super.GiveBomb();

}

// Takes a number of seconds as input, and outputs a string of the form "m:ss"
static function string CalcTextTime(int Time) {
	local string StringMin, StringSec;
	local int Minutes, Seconds;

	if (Time == 0)
		return "N/A";
	
	Seconds = Time % 60;
	Minutes = (Time - Seconds) / 60;

	StringMin = string(Minutes);

	if (Seconds < 10)
		StringSec = "0" $ string(Seconds);
	else
		StringSec = string(Seconds);

	return StringMin $ ":" $ StringSec;
}

function DrawAdditionnalHudElements(Canvas Canvas, TO_DesignInfo Design, s_HUD mainHud) {
	local float x, y, xl, yl, acc;
	local string currentText;
	local int time, CellNumber;

	if (mainHud.UserInterface != None && !bAlreadyReplacedScoreboard)
		ReplaceScoreboard(mainHud);

	//if (mainHud.UserInterface.CurrentTab == mainHud.UserInterface.UIT_BRIEFING && bFirstPreRound)
	//	mainHud.UserInterface.SelectTab(mainHud.UserInterface.UIT_DEBRIEFING);

	Canvas.Font = Design.Font18;
	Canvas.DrawColor = Design.ColorWhite;
	Canvas.Style = ERenderStyle.STY_Normal;

	x = 10;
	y = 25;
	if (!(mainHud.bHideHud || (mainHud.FrameTime >= 8))) {
		y += 24;
	}

	if (bEndGameTriggersSpawned) {
		y += 6;

		if (mainHud.bDrawBackground) {
			Canvas.DrawColor = Design.ColorSuperwhite;
			Canvas.Style = ERenderStyle.STY_Translucent;
			Canvas.SetPos(0, y);
			Canvas.DrawTile(Texture'hud_elements2', 151, 18, 0, 180, 151.0, 18.0);

			Canvas.Style = ERenderStyle.STY_Masked;
			Canvas.SetPos(0, y);
			Canvas.DrawTile(Texture'hud_elements2', 151, 18, 0, 162, 151.0, 18.0);
		}

		Canvas.SetPos(16, y+6);
		
		Canvas.DrawColor = Design.ColorSuperwhite;
		TOHud_Tool_DrawDigitPlus(Canvas, 12, 1);
		TOHud_Tool_DrawDigitPlus(Canvas, 13, 1);
		TOHud_Tool_DrawDigitPlus(Canvas, 14, 1);
		TOHud_Tool_DrawDigitPlus(Canvas, 15, 1);
		TOHud_Tool_DrawDigitPlus(Canvas, 3, 1);
		Canvas.SetPos(106, y+6);
		TOHud_Tool_DrawDirectionArrow(Canvas, TOHud_Tool_GetExitDirection());
		//Canvas.SetPos(192, y+6);
		//Canvas.DrawText(string(TOHud_Tool_GetExitDirection()), true);

		y += 18;
	}


	if (bDisplayMedalTimes) {
		y += 6;
		
		if (mainHud.bDrawBackground) {
			Canvas.DrawColor = Design.ColorSuperwhite;
			Canvas.Style = ERenderStyle.STY_Translucent;
			Canvas.SetPos(0, y);
			Canvas.DrawTile(Texture'hud_elements2', 151, 18, 0, 180, 151.0, 18.0);

			Canvas.Style = ERenderStyle.STY_Masked;
			Canvas.SetPos(0, y);
			Canvas.DrawTile(Texture'hud_elements2', 151, 18, 0, 162, 151.0, 18.0);
		}

		Canvas.SetPos(16, y+6);

		if (bUsesSpecialMedalConditions) {
			if (hardCodedBehaviors == none)
				assert(false);
			else
				hardCodedBehaviors.DrawSpecialMedalTimes(Canvas, Design, mainHud, y);			
		}
		else {
			time = s_GameReplicationInfo(GameReplicationInfo).RoundStarted - s_GameReplicationInfo(GameReplicationInfo).Remainingtime;

			if (GamePeriod == GP_PostMatch || bGameWon)
				time = SP_Time;
			
			if (mainHUD.IsPreRound()) {
				time = TimeGold;
				CellNumber = 0;
			}
			else if (time <= TimeGold) {
				time = TimeGold - time;
				CellNumber = 0;
			}
			else if (time <= TimeSilver) {
				time = TimeSilver - time;
				CellNumber = 1;
			}
			else if (time <= TimeBronze) {
				time = TimeBronze - time;
				CellNumber = 2;
			}
			else {
				time = -1;
				CellNumber = 4;
			}
			Canvas.DrawColor = Design.ColorSuperwhite;
			TOHud_Tool_DrawDigitPlus(Canvas, CellNumber, 1);
			TOHud_Tool_DrawDigitPlus(Canvas, 3, 1);
			Canvas.SetPos(45, y+6);
			mainHud.TOHud_Tool_DrawTime(Canvas, time, 2);
		}
		y += 18;
	}

	if (bDisplayAccuracy) {
		y += 6;

		if (mainHud.bDrawBackground) {
			Canvas.DrawColor = Design.ColorSuperwhite;
			Canvas.Style = ERenderStyle.STY_Translucent;
			Canvas.SetPos(0, y);
			Canvas.DrawTile(Texture'hud_elements2', 151, 18, 0, 180, 151.0, 18.0);

			Canvas.Style = ERenderStyle.STY_Masked;
			Canvas.SetPos(0, y);
			Canvas.DrawTile(Texture'hud_elements2', 151, 18, 0, 162, 151.0, 18.0);
		}

		Canvas.SetPos(16, y+6);
		
		if (SP_ShotsFired == 0)
			acc = 100;
		else
			acc = min(int(100 * float(SP_ShotsHitTarget) / float(SP_ShotsFired)), 100);

		TOHud_Tool_DrawDigitPlus(Canvas, 6, 1);
		TOHud_Tool_DrawDigitPlus(Canvas, 3, 1);

		if (acc >= class'TO_GUITabDebriefing'.default.EfficiencyThreshold)
			Canvas.DrawColor = Design.ColorGreen;
		else if (acc == 0)
			Canvas.DrawColor = Design.ColorRed;
		else
			Canvas.DrawColor = Design.ColorSuperwhite;

		Canvas.SetPos(50, y+6);

		if (acc == 100) {
			mainHUD.TOHud_Tool_DrawDigit(Canvas, 1, FS_SMALL, 1);
			mainHUD.TOHud_Tool_DrawDigit(Canvas, 0, FS_SMALL, 1);
			mainHUD.TOHud_Tool_DrawDigit(Canvas, 0, FS_SMALL, 1);
		}
		else if (acc >= 10) {
			TOHud_Tool_DrawDigitPlus(Canvas, 8, 1);
			mainHUD.TOHud_Tool_DrawDigit(Canvas, int(acc / 10), FS_SMALL, 1);
			mainHUD.TOHud_Tool_DrawDigit(Canvas, acc % 10, FS_SMALL, 1);
		}
		else {
			TOHud_Tool_DrawDigitPlus(Canvas, 8, 1);
			TOHud_Tool_DrawDigitPlus(Canvas, 8, 1);
			mainHUD.TOHud_Tool_DrawDigit(Canvas, acc, FS_SMALL, 1);
		}

		Canvas.DrawColor = Design.ColorSuperwhite;
		TOHud_Tool_DrawDigitPlus(Canvas, 7, 1);

		y += 18;
	}	



	if (bShowCheatedStatus) {
		currentText = "Cheated?:" @ bUsedCheats;
		Canvas.StrLen(currentText, xl, yl);
		Canvas.SetPos(x, y);
		Canvas.DrawText(currentText, true);
		y += yl;
	}

	if (bShowAccuracy) {
		if (SP_ShotsFired == 0)
			currentText = "Accuracy: N/A";
		else
			currentText = "Accuracy:" @ int(100 * float(SP_ShotsHitTarget) / float(SP_ShotsFired)) $ " %";
		Canvas.StrLen(currentText, xl, yl);
		Canvas.SetPos(x, y);
		Canvas.DrawText(currentText, true);
		y += yl;
	}
	
	/*currentText = "PreRoundDelay:" @ PreRoundDelay;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.SetPos(x, y);
	Canvas.DrawText(currentText, true);

	y += yl;
	currentText = "bFirstPreRound:" @ bFirstPreRound;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.SetPos(x, y);
	Canvas.DrawText(currentText, true);*/

	y += 16;
	mainHUD.YOffsetMsgs = y;



	if (bShowLastDotOnHud && mainHud.PawnOwner.Weapon != None && Level.LevelAction == LEVACT_None && !mainHud.UserInterface.Visible()) {
		Canvas.Style = ERenderStyle.STY_Translucent;
		Canvas.SetPos(Canvas.ClipX * 0.5 - 4, Canvas.ClipY * 0.5 - 4);
		Canvas.DrawTile(Texture'TODatas.Engine.LaserDot', 8, 8, 0, 0, 32, 32);
	}

	return;

// TEST VARIABLES {
	Canvas.Font = Design.Font18;
	Canvas.DrawColor = Design.ColorWhite;
	Canvas.Style = ERenderStyle.STY_Normal;

	x = 20;
	y = 60;

	yl = y;

	/*currentText = "SP_EnemyCasualties:" @ SP_EnemyCasualties;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.SetPos(x, y);
	Canvas.DrawText(currentText, true);
	y += yl;

	currentText = "SP_NumEnemies:" @ SP_NumEnemies;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.SetPos(x, y);
	Canvas.DrawText(currentText, true);
	y += yl;

	currentText = "SP_FriendlyCasualties:" @ SP_FriendlyCasualties;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.SetPos(x, y);
	Canvas.DrawText(currentText, true);
	y += yl;

	currentText = "SP_NumTeammates:" @ SP_NumTeammates;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.SetPos(x, y);
	Canvas.DrawText(currentText, true);
	y += yl;*/

	currentText = "SP_FriendflyFireIncidents:" @ SP_FriendflyFireIncidents;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.SetPos(x, y);
	Canvas.DrawText(currentText, true);
	y += yl;

	currentText = "SP_ShotsFired:" @ SP_ShotsFired;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.SetPos(x, y);
	Canvas.DrawText(currentText, true);
	y += yl;

	currentText = "SP_ShotsHitTarget:" @ SP_ShotsHitTarget;
	Canvas.StrLen(currentText, xl, yl);
	Canvas.SetPos(x, y);
	Canvas.DrawText(currentText, true);
	y += yl;

// } TEST VARIABLES

}

////////////////////
// TOHud_Tool_DrawDigitPlus
////////////////////

simulated function TOHud_Tool_DrawDigitPlus (Canvas Canvas, int digit, float inc) {
	if (digit > 15)
		Canvas.DrawTile(Texture'to_sp_hud_elements', 16, 12, ((digit-16)%4)*16+64, (((digit-16) - (digit-16)%4)/4)*16, 16.0, 12.0);
	else
		Canvas.DrawTile(Texture'to_sp_hud_elements', 16, 12, (digit%4)*16, ((digit - digit%4)/4)*16, 16.0, 12.0);
	Canvas.CurX += inc;
}

simulated function int TOHud_Tool_GetExitDirection() {
	return TOHud_Tool_GetObjectDirection(anExitObject);
}

simulated function int TOHud_Tool_GetObjectDirection(Actor theObject) {
	local s_player	thePlayer;
	local float		angle;
	local int		intangle;

	thePlayer = GetPlayer();
	if (thePlayer == none || theObject == none)
		return -1;

	angle = -(thePlayer.rotation - rotator(thePlayer.location - theObject.location)).Yaw;

	while (angle < 0)
		angle += 65536;
	while (angle > 65536)
		angle -= 65536;

	angle += 4096;

	angle = angle / (65536 / 8);
	intangle = int(angle);

	intangle -= 4;
	if (intangle < 0)
		intangle += 8;
	if (intangle == 8)
		intangle = 0;

	return intangle;
}

simulated function int TOHud_Tool_DrawDirectionArrow(Canvas Canvas, int direction) {
	// 0 = top, 1 = top right, 2 = right, 4 = lower right, etc
	local int CellNum, bFlipH, bFlipV; // 0 = false, 1 = true

	switch direction {
		case 0: CellNum = 0; bFlipH = 0; bFlipV = 0; break;
		case 1: CellNum = 1; bFlipH = 0; bFlipV = 0; break;
		case 2: CellNum = 2; bFlipH = 0; bFlipV = 0; break;
		case 3: CellNum = 1; bFlipH = 0; bFlipV = 1; break;
		case 4: CellNum = 0; bFlipH = 0; bFlipV = 1; break;
		case 5: CellNum = 1; bFlipH = 1; bFlipV = 1; break;
		case 6: CellNum = 2; bFlipH = 1; bFlipV = 0; break;
		case 7: CellNum = 1; bFlipH = 1; bFlipV = 0; break;
		default: CellNum = -1; bFlipH = 0; bFlipV = 0; break;
	}

	Canvas.DrawTile(Texture'to_sp_hud_elements', 16, 12, CellNum * 16 + 16 + bFlipH * 16, 32 + bFlipV * 12, 16.0 - bFlipH * 32, 12.0 - bFlipV * 24);
	Canvas.CurX += 1;
}

function IncrementPlayerShotsFired(pawn ShotInstigator) {
	SP_ShotsFired++;
}

simulated function ReplaceScoreboard(s_hud hud) {
	local UWindowRootWindow		Root;
	local TO_DesignInfo			Design;
	local TO_GUIBaseMgr			UserInterface;
	local float					WinWidth, WinHeight, GoodWidth;
	//local s_hud					hud;
	
	if (bAlreadyReplacedScoreboard)
		return;

	if ( (hud.PlayerOwner == None) || (hud.PlayerOwner.Player == None) || (hud.PlayerOwner.Player.Console == None) )
		return;

	if (hud.Root == None) {
		hud.Root = WindowConsole(hud.PlayerOwner.Player.Console).Root;
	}

	//hud = s_hud(s_Player(P).myhud);

	hud.UserInterface.Destroy();
	//hud.UserInterface = P.Spawn(class'TO_SinglePlayer.TO_SP_GUITabScores', P);
	hud.UserInterface = hud.PlayerOwner.Spawn(class'TO_SP_GUIBaseMgr', hud.PlayerOwner);
	hud.UserInterface.OwnerInit(hud, hud.Design);

	/*UserInterface = s_hud(s_Player(P).myhud).UserInterface;
	Root = UserInterface.root;
	Design = UserInterface.Design;
	WinWidth = root.WinWidth;
	WinHeight = root.WinHeight;
	GoodWidth = Design.GetGoodWidth(WinWidth, WinHeight);*/

	//UserInterface.Tabs = none;//TO_GUIBaseTab(Root.CreateWindow(tabclass, (Root.WinWidth - GoodWidth)/2.0, 0, GoodWidth, Root.WinHeight, None, true));

	//UserInterface.Tabs[UserInterface.UIT_SCORES].HideWindow();

	bAlreadyReplacedScoreboard = true;
}

// Return the hard coded behavior's Medal Text
function string GetSpecialMedalText(int MedalType, int MedalTextType) {
	// text type:  1 = debreifing screen, 2 = medal times indicator
	if (hardCodedBehaviors == none)
		assert(false);
	else
		return hardCodedBehaviors.GetSpecialMedalText(MedalType, MedalTextType);
}

// Return the hard coded behavior's player medal awarded
function int GetSpecialMedal() {
	if (hardCodedBehaviors == none)
		assert(false);
	else
		return hardCodedBehaviors.GetSpecialMedal();
}

function PlayedBuyedStuff(s_player P) {
	local mutator M;
	for (M = BaseMutator.NextMutator; M != None; M = M.NextMutator)
		if (M.IsA('TO_SinglePlayerMutator'))
			TO_SinglePlayerMutator(M).PlayedBuyedStuff(P);
}


// BuyWeapon
// Originally in TO_GameBasics
final function s_Weapon BuyWeaponExtended(Pawn P, int weaponnum, optional bool nocheck)
{
	local class<Weapon>					WeaponClass;
	local Weapon								NewWeapon;
	local int										Price, i;
	local Texture NewSkin;
	local	Inventory							Inv;
	local vector								X, Y, Z;

	// Check for allowed weapons
	if ( !nocheck )
	{
		//log("TO_GameBasics::BuyWeapon: ShouldWeaponBeShown == " $ ShouldWeaponBeShown(class'TOModels.TO_WeaponsHandler'.default.WeaponStr[WeaponNum]) );
		//log("TO_GameBasics::BuyWeapon: ShouldWeaponBeHidden == " $ ShouldWeaponBeHidden(class'TOModels.TO_WeaponsHandler'.default.WeaponStr[WeaponNum]) );
		if ( ( (class'TOModels.TO_WeaponsHandler'.default.WeaponStr[weaponnum] == "")
			|| (!(class'TOModels.TO_WeaponsHandler'.static.IsTeamMatch(P, WeaponNum))) ) 
				 && !ShouldWeaponBeShown(class'TOModels.TO_WeaponsHandler'.default.WeaponStr[WeaponNum])
			|| ShouldWeaponBeHidden(class'TOModels.TO_WeaponsHandler'.default.WeaponStr[WeaponNum]) )
			return none;
	}

	WeaponClass = class<Weapon>(DynamicLoadObject(class'TOModels.TO_WeaponsHandler'.default.WeaponStr[weaponnum], class'Class'));

	// Pawn can buy the weapon?
	if ( (P.FindInventoryType(WeaponClass) != None)
		|| (P.IsA('s_Player') && (s_Player(P).bNotPlaying || s_Player(P).bBuyingWeapon)) )
		return none;

	if ( P.IsA('s_Player') )
		s_Player(P).bBuyingWeapon = true;

	newWeapon = Spawn(WeaponClass);

	if ( NewWeapon == None )
	{
		if ( P.IsA('s_Player') )
			s_Player(P).bBuyingWeapon = false;
		return none;
	}

	if ( !HaveMoney(P, s_Weapon(newWeapon).Price) || !newWeapon.IsA('s_Weapon') )
	{
		NewWeapon.Destroy();
		if ( P.IsA('s_Player') )
			s_Player(P).bBuyingWeapon = false;
		return none;
	}

	for( Inv=P.Inventory; Inv!=None; Inv=Inv.Inventory )
	{
		if ( (Inv != NewWeapon) && Inv.IsA('s_Weapon') && (s_Weapon(Inv).WeaponClass != 0)
			&& (s_Weapon(Inv).WeaponClass == s_Weapon(NewWeapon).WeaponClass) )
		{
			P.GetAxes(P.Rotation, X, Y, Z);
	    s_Weapon(Inv).DropFrom(P.Location + 0.8 * P.CollisionRadius * X + - 0.5 * P.CollisionRadius * Y);
		}
	}

	newWeapon.RespawnTime = 0.0;
	newWeapon.GiveTo(P);
	if ( s_Weapon(newWeapon).bHasMultiSkins )
		s_Weapon(newWeapon).SetSkins();
	newWeapon.PlayIdleAnim();
	newWeapon.GotoState('idle');
	s_Weapon(newWeapon).ForceStillFrame();

	newWeapon.bHeldItem = true;
	newWeapon.bTossedOut = false;
	//newWeapon.GiveAmmo(P);
	newWeapon.SetSwitchPriority(P);
	if ( P.IsA('PlayerPawn') )
		newWeapon.SetHand(PlayerPawn(P).Handedness);
	else
		newWeapon.GotoState('Idle');

	P.PendingWeapon = newWeapon;

	//Used by LastWeapon
	if (P.IsA('s_BPlayer'))

		If (P.Weapon != none)
		{
			if ( !P.Weapon.IsA('TO_Binocs') && !P.Weapon.IsA('s_Knife') && !P.Weapon.IsA('TO_Grenade'))
				s_BPlayer(P).LastSelectedWeapon = P.Weapon;
			else
				s_BPlayer(P).LastSelectedWeapon = none;
		} else {
				s_BPlayer(P).LastSelectedWeapon = none;
		}

	if ( P.Weapon == None )
		P.ChangedWeapon();
	else if ( !P.Weapon.PutDown() )
		P.PendingWeapon = None;
		//P.Weapon.GotoState('DownWeapon');

 	//P.ClientMessage(-s_Weapon(newWeapon).Price@-s_Weapon(newWeapon).default.Price);
	AddMoney(P, -s_Weapon(newWeapon).Price, nocheck);

    /* j3rky - remove default ammo */
	s_Weapon(newWeapon).RemainingClip = 0;
	s_Weapon(newWeapon).BackupClip = 0;
	s_Weapon(newWeapon).ClipAmmo = 0;
	s_Weapon(newWeapon).BackClipAmmo = 0;
	/* j3rky */

	if ( P.IsA('s_Player') )
		s_Player(P).bBuyingWeapon = false;

	return s_Weapon(newWeapon);
}

// BuyList - sell/buy stuff from buymenu
// Originally in s_player
//function xxBuyList(s_player Player, Equipment E)
function xxBuyList(s_player Player)
{
	local	s_Weapon	w;
	local	int			i, id, x;
	local	Inventory	inv;
	local	bool		boughtall;

	// buyzone
	if ( !Player.IsInBuyZone() )
		return;

   	// init
	boughtall = true;

    // sell stuff
    
    // Vest
    if (Player.VestCharge == 100 && (Player.Equip.Flags & 64) == 0 )
    {
		Player.AddMoney(350, true);
		Player.VestCharge = 0;
	}

	// Helmet
	if (Player.HelmetCharge == 100 && (Player.Equip.Flags & 128) == 0)
    {
		Player.AddMoney(250, true);
		Player.HelmetCharge = 0;
	}
	
	// Pads
	if (Player.LegsCharge == 100 && (Player.Equip.Flags & 32) == 0 )
    {
		Player.AddMoney(300, true);
		Player.LegsCharge = 0;
	}
	
	//pistols
	w = Player.FindWeaponByGroup(2);
	//if (E.Weapon2 != 0 && E.Weapon2 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class)))
	if (Player.Equip.Weapon2 != 0 && (W == None || Player.Equip.Weapon2 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class))))
	{
		// sell old weapon
		if (W != none)
			Player.CheckWeaponSell(W, Player.Equip.Weapon2);
	} else if (W != none && Player.Equip.Weapon2 == 0) {
		Player.CheckWeaponSell(W, Player.Equip.Weapon2);
	}

	//smg/shotgun
	w = Player.FindWeaponByGroup(3);
	//if (E.Weapon3 != 0 && E.Weapon3 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class)))
	if (Player.Equip.Weapon3 != 0 && (W == None || Player.Equip.Weapon3 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class))))
	{
		// sell old weapon
		if (W != none)
			Player.CheckWeaponSell(W, Player.Equip.Weapon3);
	} else if (W != none && Player.Equip.Weapon3 == 0) {
		Player.CheckWeaponSell(W, Player.Equip.Weapon3);
	}

	//rifles
	w = Player.FindWeaponByGroup(4);
	//if (E.Weapon4 != 0 && E.Weapon4 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class)))
	if (Player.Equip.Weapon4 != 0 && (W == None || Player.Equip.Weapon4 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class))))
	{
		// sell old weapon
		if (W != none)
			Player.CheckWeaponSell(W, Player.Equip.Weapon4);
	} else if (W != none && Player.Equip.Weapon4 == 0) {
		Player.CheckWeaponSell(W, Player.Equip.Weapon4);
	}
	
	//nades
	w = Player.FindWeaponByGroup(5);
	//if (E.Weapon5 != 0 && E.Weapon5 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class)))
	if (Player.Equip.Weapon5 != 0 && (W == None || Player.Equip.Weapon5 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class))))
	{
		// sell old weapon
		if (W != none)
			Player.CheckWeaponSell(W, Player.Equip.Weapon5);
	} else if (W != none && Player.Equip.Weapon5 == 0) {
		Player.CheckWeaponSell(W, Player.Equip.Weapon5);
	}
	
	// NV 
	if (Player.bHasNV && (Player.Equip.Flags & 16) == 0 )	{
		Player.AddMoney(400, true);
		xxKillNV(Player);
	}

   

    ////////////
	// buy stuff
	////////////
	
	// Vest
	if (Player.VestCharge < 100 && (Player.Equip.Flags & 64) != 0 && Player.HaveMoney(350) )
	{
		Player.AddMoney(-350, true);
		Player.VestCharge = 100;
	}  else if ( (Player.Equip.Flags & 64) != 0 && Player.VestCharge < 100) {
		boughtall = false;
	}

    // Helmet
    if (Player.HelmetCharge < 100 && (Player.Equip.Flags & 128) != 0 && Player.HaveMoney(250))
	{
		Player.AddMoney(-250, true);
		Player.HelmetCharge = 100;
	} else if ( (Player.Equip.Flags & 128) != 0 && Player.HelmetCharge < 100) {
		boughtall = false;
	}

    // Pads
	if (Player.LegsCharge < 100 && (Player.Equip.Flags & 32) != 0 && Player.HaveMoney(300) )
	{
		Player.AddMoney(-300, true);
		Player.LegsCharge = 100;
	} else if ( (Player.Equip.Flags & 32) != 0 && Player.LegsCharge < 100) {
		boughtall = false;
	}
	
    // buy/sell knives
    //GiveWeapon(self, "s_SWAT.s_Knife");
	w = s_Weapon(Player.FindInventoryType(class's_SWAT.s_Knife'));

    Player.AddMoney((w.ClipAmmo - 1)* (W.ClipPrice/W.ClipSize), true);
	w.ClipAmmo = 1;

    i = (Player.Equip.Flags & 7);
 	for (x=1; x<i && x<8; x++)
		if( Player.HaveMoney(W.ClipPrice/W.ClipSize) )
		{
			Player.AddMoney( -W.ClipPrice/W.ClipSize , true);
			w.ClipAmmo++;
		} else {
			boughtall = false;
			break;
		}
	
    ///////////////////
	// buy weapons/ammo
    ///////////////////
	
	//pistols
	w = Player.FindWeaponByGroup(2);
	//if (E.Weapon2 != 0 && E.Weapon2 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class)))
	if (Player.Equip.Weapon2 != 0 && W == none)
	{
		// buy new weapon
		W = BuyWeaponExtended(Player, Player.Equip.Weapon2, true);
		if (W == none)
			boughtall = false;
		else
			w.clipammo = w.clipSize;
	}
	if (W != none)
	{
		// buy/sell ammo
		xxs_SetAmmo(Player, Player.Equip.Ammo & 255, W.GetRemainingBullets(false), W, false);
		xxs_SetAmmo(Player, Player.Equip.BackupAmmo & 255, W.GetRemainingBullets(true), W, true);
	}

	//smg/shotgun
	w = Player.FindWeaponByGroup(3);
	//if (E.Weapon3 != 0 && E.Weapon3 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class)))
	if (Player.Equip.Weapon3 != 0 && W == none)
	{
		// buy weapon
		W = BuyWeaponExtended(Player, Player.Equip.Weapon3, true);
		if (w==none)
			boughtall = false;
		else
			w.clipammo = w.clipSize;
	}
	if (W != none)
	{
		// buy/sell ammo
		xxs_SetAmmo(Player, (Player.Equip.Ammo >> 8)& 255, W.GetRemainingBullets(false), W, false);
		xxs_SetAmmo(Player, (Player.Equip.BackupAmmo >> 8)& 255, W.GetRemainingBullets(true), W, true);
	}

	//rifles
	w = Player.FindWeaponByGroup(4);
	//if (E.Weapon4 != 0 && E.Weapon4 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class)))
	if (Player.Equip.Weapon4 != 0 && W == none)
	{
		// buy weapon
		W = BuyWeaponExtended(Player, Player.Equip.Weapon4, true);
		if (w==none)
			boughtall = false;
		else
			w.clipammo = w.clipSize;
	}
	if (W != none)
	{
		// buy/sell ammo
		xxs_SetAmmo(Player, (Player.Equip.Ammo >> 16)& 255, W.GetRemainingBullets(false), W, false);
		xxs_SetAmmo(Player, (Player.Equip.BackupAmmo >> 16)& 255, W.GetRemainingBullets(true), W, true);
	}
	//nades
	w = Player.FindWeaponByGroup(5);
	//if (E.Weapon5 != 0 && E.Weapon5 != class'TOModels.TO_WeaponsHandler'.static.GetIdByClass(string(W.Class)))
	if (Player.Equip.Weapon5 != 0 && W == none)
	{
		// buy weapon
		W = BuyWeaponExtended(Player, Player.Equip.Weapon5, true);
		if (w==none) boughtall = false;
	}
	
	//buy last - not that important
	if (!Player.bHasNV && (Player.Equip.Flags & 16) != 0 && Player.HaveMoney(400) )
	{
		Player.AddMoney(-400, true);
		xxSetNV(Player);
	} else if ( (Player.Equip.Flags & 16) != 0 && !Player.bHasNV) {
		boughtall = false;
	}

	// buymenu - complete / fail sound
	if(!boughtall)
	{
		Player.PlaySound(Sound'hithelmet', SLOT_None);
    } else {
		Player.PlaySound(Sound'Kevlar', SLOT_None);
        Player.CloseBuymenu();
    }

	xxClampMoney(Player);
	Player.CalculateWeight();

	//if (Weapon == none)
		Player.SwitchToBestWeapon();
}

function xxKillNV(s_player Player)
{
	//log("s_Player::KillNV");
	Player.bHasNV = false;
	//xxClientKillNV();
}

function xxSetNV(s_player Player)
{
	//log("s_Player::SetNV");
	Player.bHasNV = true;
	//xxClientSetNV();
}

final function xxs_SetAmmo (s_player Player, int clips, int bullets, s_Weapon w, bool secondary, optional bool buykeybind)
{
	Local	int Amount;

	if ( Player.Role < Role_Authority )
	{
		return;
	}

	if ( secondary )
	{
		Amount = (W.GetRemainingClips(true)-clips) * w.default.BackupClipPrice;
		if ( !Player.HaveMoney(-Amount) ) return;
		Player.AddMoney(Amount, true);
		w.SetRemainingAmmo(clips, bullets, true);
	}
	else
	{
		Amount = (W.GetRemainingClips(false)-clips) * w.default.ClipPrice;
        if ( !Player.HaveMoney(-Amount) ) return;
		  Player.AddMoney(Amount, true);

        //fix shotties only being loaded with one shell
        if(w.IsA('s_Mossberg') && buykeybind)
	           clips = w.clipSize;

        w.SetRemainingAmmo(clips, bullets, false);
	}
}

function xxClampMoney(s_player Player)
{
	local	int	DiffMoney;

	// Implemented this way so HUD money indicator remains correct
	DiffMoney = Clamp(Player.Money, -MaxMoney, MaxMoney) - Player.Money;

	if ( DiffMoney == 0 )
		return;

	AddMoney(Player, DiffMoney, true);
}



exec function tospsavegame() {
	local s_player player;

	player = GetPlayer();

	if (player == none)
		assert(false);

	player.ConsoleCommand("SaveGame " $ 9);
}

exec function tosploadgame() {
	local s_player player;

	player = GetPlayer();

	if (player == none)
		assert(false);

	player.ClientMessage("");
	//bExitAllMenus = true;
	player.ClientTravel( "?load=" $ 9, TRAVEL_Absolute, false);
}


defaultproperties
{
     bFirstPreRound=True
     GameSpeed=1.000000
     FriendlyFireIncidentsThresholdMin=4
     FriendlyFireIncidentsThresholdMax=30
     bReduceSFX=False
     MinAllowedScore=999
     MaxMoney=100000
     bSinglePlayer=True
     postGameTime=7
     BotConfigType=Class'TO_SinglePlayer.TO_SP_BotInfo'
     HUDType=Class'TO_SinglePlayer.TO_SP_HUD'
     MapListType=Class'TO_SinglePlayer.TO_SinglePlayerMaplist'
     BeaconName="Tactical Ops Single Player"
}
