//=============================================================================
// TO_SinglePlayerTrigger.
//=============================================================================
// Spawned by the singleplayer game. Allows special events to occur when the
// player gets in it's collision radius, such as winning the game or spawning
// special bots.
//=============================================================================
// Programmed by Mathieu 'EMH_Mark3' Mallet for Tactical Ops
// 19/08/2002
//=============================================================================

class TO_SinglePlayerTrigger extends Triggers;

var	int		triggerType;
// Allowed trigger types:
// 0 = Force Win Round trigger (player automatically wins the round)
// 1 = Spawn special bots trigger
// 2 = Spawn end round trigger
// 3 = Invoque TO_SP_HardCodedBehaviors hook 

var	bool	triggerOnceOnly;

/*function PreBeginPlay() {
	//log("TO_SinglePlayerTrigger::PreBeginPlay: Object created.");
}*/

function Touch(actor Other)
{
	local s_Player P;

	//log("TO_SinglePlayerTrigger::PreBeginPlay: Something touched me.");

	if (triggerType == -1)
		return;

	if (Other.IsA('s_Player') && TO_SinglePlayerGame(Level.Game) != None) {

		//log("TO_SinglePlayerTrigger::Touch: Player touched the trigger !!! (Triggertype == " $ triggerType $ ")");

		P = s_Player(Other);
		if (triggerOnceOnly)
			SetCollision(false);

		if (triggerType == 0) {			// Win round
			TO_SinglePlayerGame(Level.Game).ForceWinRound();
		} else if (triggerType == 1) {	// Spawn special bots
			TO_SinglePlayerGame(Level.Game).SpawnSpecialBots();
			SendMissionMessage();
		} else if (triggerType == 2) {	// Spawn end round trigger
			TO_SinglePlayerGame(Level.Game).SpawnEndGameTriggers();
			SendMissionMessage();
		} else if (triggerType == 3) {	// Invoque TO_SP_HardCodedBehaviors hook
			TO_SinglePlayerGame(Level.Game).InvoqueHCBSpecialTriggerHook(self);
			SendMissionMessage();
		} else if (triggerType == 4) {
			MakeAllBotsFollowPlayer();
		}
		else {
			warn("TO_SinglePlayerTrigger::Touch: Unrecognised trigger type (" $ triggerType $ ")");
			assert(false);
		}
	}

}

function SendMissionMessage() {
	local TO_SinglePlayerGame	g;
	g = TO_SinglePlayerGame(Level.Game);

	if (g == none) {
		warn("TO_SinglePlayerTrigger::SendMissionMessage: This actor is only to be used with single player games.");
		assert(false);
	}

	g.SpecialTriggersMSG();
}

function MakeAllBotsFollowPlayer() {
	local Pawn					PawnLink, player;
	local TO_SinglePlayerGame	g;

	g = TO_SinglePlayerGame(Level.Game);

	if (g == none) {
		warn("TO_SinglePlayerTrigger::MakeAllBotsFollowPlayer: This actor is only to be used with single player games.");
		assert(false);
	}
	
	player = g.GetPlayer();

	// Make every bot follow the player
	for ( PawnLink=Level.PawnList; PawnLink!=None; PawnLink=PawnLink.nextPawn )	{
		if ( (PawnLink.PlayerReplicationInfo != None) && !PawnLink.PlayerReplicationInfo.bWaitingPlayer ) {
			if ( ((s_Bot(PawnLink) != None) && (!s_Bot(PawnLink).bNotPlaying))) { // Is a bot
				s_Bot(PawnLink).SetOrders('Follow', player, false);
			}
		}
	}
}

defaultproperties
{
     TriggerType=-1
     triggerOnceOnly=True
     bCollideActors=False
}
